/*******************************************************************************
 * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
 * @file     w25n01.c
 * @author   UartTFT Application Team
 * @version  V0.0.1
 * @date     2023-01-01
 * @brief
 ********************************************************************************/
 
#include "w25n01.h"
#include "w25qxx.h"

/********************************************************************************
* Function Name	: W25N01GV_ReadSR
* Description  	: Read the value of the register.
* Input        	: - SRx:Register address
					  - 3 status registers
					  - Protection register 1:    address 0xA0
					  - Configuration register 2: address 0xB0
					  - Status register 3:        address 0xC0  (Read Only)	
* Output       	: None
* Return       	: None
*********************************************************************************/
uint8_t W25N01GV_ReadSR(uint8_t reg)
{  
	uint8_t byte = 0;
	Flash_SS_RESET; 
	exFlash_SPI_ReadWriteByte(W25N01GV_READ_STATUSREG);
	exFlash_SPI_ReadWriteByte(reg);
	byte = exFlash_SPI_ReadWriteByte(0Xff);
	Flash_SS_SET;    
	//W25N01GV_Write_Disable();
	return byte;   
}

/********************************************************************************
* Function Name	: W25N01GV_Write_SR
* Description  	: Write the value of the register.
* Input        	: - SRx:Register address
					  - 3 status registers
					  - Protection register 1:    address 0xA0
					  - Configuration register 2: address 0xB0
					  - Status register 3:        address 0xC0  (Read Only)	
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25N01GV_Write_SR(uint8_t reg, uint8_t val)
{   
	//W25N01GV_Write_Enable();
	Flash_SS_RESET; 
	exFlash_SPI_ReadWriteByte(W25N01GV_WRITE_STATUSREG);
	exFlash_SPI_ReadWriteByte(reg);
	exFlash_SPI_ReadWriteByte(val);
	Flash_SS_SET; 
	W25N01GV_Wait_Busy();
} 

/********************************************************************************
* Function Name	: W25N01GV_Wait_Busy
* Description  	: Wait for BUSY bit to clear.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25N01GV_Wait_Busy(void)   
{   
	while((W25N01GV_ReadSR(W25N01GV_SR3)&0x01)==0x01);
} 

/********************************************************************************
* Function Name	: W25N01GV_Write_Enable
* Description  	: Enable to write.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25N01GV_Write_Enable(void)   
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25N01GV_WRITE_ENABLE);
	Flash_SS_SET;
} 
/********************************************************************************
* Function Name	: W25N01GV_Write_Disable
* Description  	: Disable to write.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/ 
void W25N01GV_Write_Disable(void)   
{  
	Flash_SS_RESET; 
	exFlash_SPI_ReadWriteByte(W25N01GV_WRITE_DISABLE);
	Flash_SS_SET; 	
} 

/********************************************************************************
* Function Name	: W25N01GV_ContinuousRead_Mode
* Description  	: Enable continuous read mode.(BUF=0, ECC-E=1)
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25N01GV_ContinuousRead_Mode(void)
{
	uint8_t statut = 0;
	statut = W25N01GV_ReadSR(W25N01GV_SR2);
	statut &= 0xe7;
	statut |= 0x10;
	W25N01GV_Write_SR(W25N01GV_SR2,statut);
}

/********************************************************************************
* Function Name	: W25N01GV_BufferRead_Mode
* Description  	: Enable buffer read mode.(BUF=1, ECC-E=1)
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25N01GV_BufferRead_Mode(void)
{
	uint8_t statut = 0;
	statut = W25N01GV_ReadSR(W25N01GV_SR2);
	statut |= 0x08;
	statut |= 0x10;
	W25N01GV_Write_SR(W25N01GV_SR2,statut);
}
/********************************************************************************
* Function Name	: W25N01GV_ReadID
* Description  	: Read W25N01GV ID.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
uint32_t W25N01GV_ReadID(void)
{
	uint8_t Mfr_id = 0;
	uint8_t Device_idL = 0;
	uint8_t Device_idH = 0;
	uint32_t id = 0;
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25N01GV_ID);
	exFlash_SPI_ReadWriteByte(0xff);
	Mfr_id = exFlash_SPI_ReadWriteByte(0xff);
	Device_idH = exFlash_SPI_ReadWriteByte(0xff);
	Device_idL = exFlash_SPI_ReadWriteByte(0xff);
	Flash_SS_SET;

	id = Mfr_id;
	id = (id<<8)+Device_idH;
	id = (id<<8)+Device_idL;
	return id;
}

//Init
void W25N01GV_Init(void)
{
	W25N01GV_ContinuousRead_Mode();
//	LT_W25N01GV_PageDataRead(0);	// buffer page 0 data
//	DelayUS(50);
//	Nand_page = 0;
}

//-------------------------------------------------Erase--------------------------------------------------------
/********************************************************************************
* Function Name	: LT_W25N01GV_BlockErase128KB
* Description  	: Erase a sector (minimum time to erase a sector: 150ms).
* Input        	: - addr:sector address
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT_W25N01GV_BlockErase128KB(uint32_t addr)   
{
	uint16_t Page = (addr/2048);
	
	W25N01GV_Write_SR(W25N01GV_SR1, 0x00); // disable flash protection function
	W25N01GV_Write_Enable();               // set WEL 
	W25N01GV_Wait_Busy();

	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25N01GV_128KBLOCK_ERASE); 
	exFlash_SPI_ReadWriteByte(0xff); 
	exFlash_SPI_ReadWriteByte((uint8_t)(Page>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)(Page));
	Flash_SS_SET;
	
	W25N01GV_Wait_Busy();   				// wait for chip erase to finish
	W25N01GV_Write_SR(W25N01GV_SR1, 0x7c);  // enable flash protection function
}

/********************************************************************************
* Function Name	: LT_W25N01GV_FullChipErase
* Description  	: Full Erase.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT_W25N01GV_FullChipErase(void)
{
	uint16_t k;
	
	for(k=0;k<1024;k++) LT_W25N01GV_BlockErase128KB(NandBlockSize*k);
}

/********************************************************************************
* Function Name	: LT_W25N01GV_PageDataRead
* Description  	: Buffer one page of data and wait for the host to read.
* Input        	: - PageNum: Page
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT_W25N01GV_PageDataRead(uint16_t page)
{
	W25N01GV_Wait_Busy();
	Flash_SS_RESET;                            	    
	exFlash_SPI_ReadWriteByte(W25N01GV_READ_PAGE);
	exFlash_SPI_ReadWriteByte(0x00);
	exFlash_SPI_ReadWriteByte((uint8_t)(page>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)page);
	Flash_SS_SET;

	W25N01GV_Wait_Busy();
}

/********************************************************************************
* Function Name	: LT_W25N01GV_ReadData
* Description  	: Read data of specified length from the specified address.
* Input        	: - pBuffer: buffer for save data
                  - ReadAddr: Start address of reading data
                  - NumByteToRead: number bytes to read
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT_W25N01GV_ReadData(uint8_t* pBuffer,uint32_t ReadAddr,uint32_t NumByteToRead)   																							//2kݣҳ0λÿʼ
{ 
	unsigned short page,endpage,remainder;
	uint32_t i;
	
	remainder = ReadAddr%2048;
	page = (ReadAddr/2048);

	LT_W25N01GV_PageDataRead(page);				 // copy data to buffer, waiting for read
	
	Flash_SS_RESET;   	
	exFlash_SPI_ReadWriteByte(W25N01GV_READ);
	exFlash_SPI_ReadWriteByte(0x00);              //dummy clock
	exFlash_SPI_ReadWriteByte(0x00); 	
	exFlash_SPI_ReadWriteByte(0x00);
	for(i=0;i<remainder;i++) exFlash_SPI_ReadWriteByte(0XFF);   	//dummy data
	for(i=0;i<NumByteToRead;i++)
	{
		pBuffer[i]=exFlash_SPI_ReadWriteByte(0XFF);
	}
	Flash_SS_SET;
	//W25N01GV_Wait_Busy(); 	
} 

//---------------------------------------------------write--------------------------------------------------
/********************************************************************************
* Function Name	: LT_W25N01GV_Write_Page
* Description  	: On a page, write data of specified length at the specified column address.
* Input        	: - pBuffer: data start address
                  - page: page
                  - column: column
                  - NumByteToWrite: number bytes to write(max one page:2048 bytes)
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT_W25N01GV_Write_Page(uint8_t* pBuffer,uint16_t page,uint16_t column,uint16_t NumByteToWrite)
{
	uint16_t i;

	//  write the data to buffer
	W25N01GV_Write_Enable();                  	// set WEL
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25N01GV_LOADPRODATA);
	exFlash_SPI_ReadWriteByte((column)>>8); 
	exFlash_SPI_ReadWriteByte(column);
	for(i=0;i<NumByteToWrite;i++)exFlash_SPI_ReadWriteByte(pBuffer[i]);
	Flash_SS_SET; 
	W25N01GV_Wait_Busy();					    // wait for writing to end
	
	// write the data in buffer to flash
	W25N01GV_Write_Enable();
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25N01GV_PROEXECUTE);
	exFlash_SPI_ReadWriteByte(0xff);
	exFlash_SPI_ReadWriteByte((uint8_t)((page)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)(page));
	Flash_SS_SET;  							
	W25N01GV_Wait_Busy();					    // wait for writing to end
}

/********************************************************************************
* Function Name	: LT_W25N01GV_Write_NoCheck
* Description  	: Write SPI FLASH without inspection.
                  You must ensure that all the data written in the address range is 0XFF, otherwise the data written in non-0XFF will fail.
                  With auto-feed function
* Input        	: - pBuffer: data start address
                  - WriteAddr: Address to start writing
                  - NumByteToWrite: number bytes to write(max 0xFFFF bytes)
* Output       	: None
* Return       	: None
*********************************************************************************/
void LT_W25N01GV_Write_NoCheck(uint8_t* pBuffer,uint32_t WriteAddr,uint16_t NumByteToWrite)
{
	uint16_t i;
	uint16_t page,endpage,column,Fdata;
	uint16_t datalen=NumByteToWrite;

	page   = (WriteAddr/2048);				
	endpage= ((WriteAddr+NumByteToWrite-1)/2048);	
	column = (WriteAddr%2048);	
	
	W25N01GV_Write_SR(W25N01GV_SR1, 0x00);      // disable flash protection function
	
	if(page==endpage)	
		LT_W25N01GV_Write_Page(pBuffer,page,column,NumByteToWrite);
	else
	{
		Fdata = 2048-column;
		LT_W25N01GV_Write_Page(pBuffer,page++,column,Fdata);
		
		for(i=0;datalen>2048;i++)
		{
			LT_W25N01GV_Write_Page(&pBuffer[Fdata+i*2048],page++,0,NumByteToWrite);
			datalen-=2048;
		}
		if(datalen!=0) LT_W25N01GV_Write_Page(&pBuffer[Fdata+i*2048],page,0,NumByteToWrite);

	}
	
	W25N01GV_Write_SR(W25N01GV_SR1, 0x7c);              // enable flash protection function
} 


