/*******************************************************************************
 * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
 * @file     w25n01.h
 * @author   UartTFT Application Team
 * @version  V0.0.1
 * @date     2023-01-01
 * @brief
 ********************************************************************************/

#ifndef _W25N01_H
#define _W25N01_H


#include "data.h"

//      W25N01GV capacity is 1G-Bit1024 Block33280 Page
#define NandPageSize (2048)       // one Page = 2KB (Program)
#define NandBlockSize (64 * 2048) // one Block = 128KB (Erase)

//----------------------------------------------------------------Instruction--------------------------------------------------------------------------- 
#define W25N01GV_READ_STATUSREG  0x0f 
#define W25N01GV_WRITE_STATUSREG 0x1f 
#define W25N01GV_WRITE_ENABLE    0x06
#define W25N01GV_WRITE_DISABLE   0x04
#define W25N01GV_SR1  0xA0  
#define W25N01GV_SR2  0xB0
#define W25N01GV_SR3  0xC0
#define W25N01GV_READ_PAGE 0x13
#define W25N01GV_READ  			0x03
#define W25N01GV_READ_FAST  	0x0B
#define W25N01GV_READ_FAST_DUAL 0x3b
#define W25N01GV_READ_FAST_DUAL_IO 0xbb
#define W25N01GV_LOADPRODATA 	0x02
#define W25N01GV_PROEXECUTE  	0x10
#define W25N01GV_128KBLOCK_ERASE  	0xD8
#define W25N01GV_ID     0x9f
#define W25N01GV_DEVICE_ID    0XEFAA21



/*************************************************************** winbond NandFlash **********************************************************************/
//status register
uint8_t W25N01GV_ReadSR(uint8_t reg);
void W25N01GV_Write_SR(uint8_t reg, uint8_t val);
void W25N01GV_Wait_Busy(void);
void W25N01GV_Write_Enable(void);   
void W25N01GV_Write_Disable(void);
void W25N01GV_ContinuousRead_Mode(void);
void W25N01GV_BufferRead_Mode(void);
uint32_t W25N01GV_ReadID(void);

//Init
void W25N01GV_Init(void);

//erase
void LT_W25N01GV_BlockErase128KB(uint32_t addr);
void LT_W25N01GV_FullChipErase(void);

/*--------------------------BufferRead_Mode-----------------------------------*/
//None

/*------------------------ContinuousRead_Mode--------------------------------*/
//-------------------------------------STD_Mode---------------------------------------------------------------
//read
void LT_W25N01GV_PageDataRead(uint16_t page);
void LT_W25N01GV_ReadData(uint8_t* pBuffer,uint32_t ReadAddr,uint32_t NumByteToRead);
//write
void LT_W25N01GV_Write_Page(uint8_t* pBuffer,uint16_t page,uint16_t column,uint16_t NumByteToWrite);
void LT_W25N01GV_Write_NoCheck(uint8_t* pBuffer,uint32_t WriteAddr,uint16_t NumByteToWrite);

//-------------------------------------Dual_Mode---------------------------------------------------------------
//None




#endif /* _W25N01_H */
