/*******************************************************************************
 * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
 * @file     w25qxx.c
 * @author   UartTFT Application Team
 * @version  V0.0.1
 * @date     2023-01-01
 * @brief
 ********************************************************************************/
 
#include "w25qxx.h"
#include "spi_drv.h"
#include "dmac_drv.h"

										 
uint8_t Flash_Type = 0; // Flash_Type = 0, NorFlash 24bits address
						// Flash_Type = 1, NorFlash 32bits address
						// Flash_Type = 2, Nand Flash
uint16_t Flash_ID = 0;
uint16_t Nand_page = 0; // Nand Flash current page,used to determine whether to switch page



void Flash_SS(uint8_t bitval)
{
	if(bitval==0)       Flash_SS_RESET;
	else if(bitval==1)  Flash_SS_SET;
}



void exFlash_SPI_Init(void)
{
	SPI_InitTypeDef SPI_InitStruct;
	
	SPI_InitStruct.SPI_Mode = SPI_Mode_Master;
	SPI_InitStruct.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;
	SPI_InitStruct.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStruct.SPI_CPOL = SPI_CPOL_Low;
	SPI_InitStruct.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStruct.SPI_FirstBit = SPI_FirstBit_MSB;

	SPI_Init(exFlash,&SPI_InitStruct);
	
	//DMA_Init(DMA1_BASE_ADDR);
	exFlash_dma_control->DMA_CONFIG = 1;
	//tx
	exFlash_dma_channel[0]->DMA_DADDR = (uint32_t)&exFlash->SPIDR;
	exFlash_dma_channel[0]->DMA_CTRL = DNOCHG|SIEC|M2P_DMA;
	exFlash_dma_channel[0]->DMA_CFG = (HS_SEL_SRC_SOFT);
	exFlash_dma_channel[0]->DMA_CFG_HIGH = DST_PER_SPI_TX(1)|SRC_PER_SPI_TX(1);
	
	//rx
	exFlash_dma_channel[1]->DMA_SADDR = (uint32_t)&exFlash->SPIDR;
	exFlash_dma_channel[1]->DMA_CTRL = SNOCHG|DIEC|P2M_DMA;
	exFlash_dma_channel[1]->DMA_CFG = (HS_SEL_DST_SOFT);
	exFlash_dma_channel[1]->DMA_CFG_HIGH = SRC_PER_SPI_RX(4)|DST_PER_SPI_RX(4);

//	exFlash_dma_control->DMA_CONFIG = 0;
	exFlash->SPIDMACR &= (~0x03);	//disable SPI TX RX DMA
}

uint8_t exFlash_SPI_ReadWriteByte(uint8_t Send)
{
	uint8_t ucTemp = 0;
	
	while(((exFlash->SPISRHW)&(SPISR_TXFFULL_MASK)));
	exFlash->SPIDR = Send;
	while(!(0x80 & exFlash->SPISR));
	while(((exFlash->SPISRHW) & (SPISR_RXFEMP_MASK)));
	ucTemp = exFlash->SPIDR;
	
	return ucTemp;
}

void exFlash_SPI_DMA_Tran(uint8_t* Tx_Addr, uint8_t* Rx_Addr, uint32_t length)
{
	//DMA_Init(DMA1_BASE_ADDR);
	exFlash->SPIDMACR |= 0x03;	//enable SPI TX RX DMA
	exFlash_dma_control->DMA_CONFIG = 1;
	
	//tx
	exFlash_dma_channel[0]->DMA_SADDR = (uint32_t)Tx_Addr;
	exFlash_dma_channel[0]->DMA_DADDR = (uint32_t)&exFlash->SPIDR;
	exFlash_dma_channel[0]->DMA_CTRL_HIGH = length ;
	exFlash_dma_channel[0]->DMA_CTRL = DNOCHG|SIEC|M2P_DMA;
	exFlash_dma_channel[0]->DMA_CFG = (HS_SEL_SRC_SOFT);
	exFlash_dma_channel[0]->DMA_CFG_HIGH = DST_PER_SPI_TX(1)|SRC_PER_SPI_TX(1);
	
	//rx
	exFlash_dma_channel[1]->DMA_SADDR = (uint32_t)&exFlash->SPIDR;
	exFlash_dma_channel[1]->DMA_DADDR = (uint32_t)Rx_Addr;
	exFlash_dma_channel[1]->DMA_CTRL_HIGH = length ;
	exFlash_dma_channel[1]->DMA_CTRL = SNOCHG|DIEC|P2M_DMA;
	exFlash_dma_channel[1]->DMA_CFG = (HS_SEL_DST_SOFT);
	exFlash_dma_channel[1]->DMA_CFG_HIGH = SRC_PER_SPI_RX(4)|DST_PER_SPI_RX(4);
 
//	exFlash_dma_control->DMA_MASKTFR = CHANNEL_UMASK(0)|CHANNEL_UMASK(1);
	exFlash_dma_control->DMA_CHEN = CHANNEL_WRITE_ENABLE(0)|CHANNEL_ENABLE(0)|CHANNEL_WRITE_ENABLE(1)|CHANNEL_ENABLE(1);
}

void exFlash_SPI_DMA_Wait(void)
{
//	while((exFlash_dma_control->DMA_RAWTFR & (CHANNEL_STAT(0) | CHANNEL_STAT(1))) != (CHANNEL_STAT(0) | CHANNEL_STAT(1)));
//	exFlash_dma_control->DMA_CLRTFR = (CHANNEL_STAT(0) | CHANNEL_STAT(1));
	
	while(exFlash_dma_control->DMA_CHEN & (CHANNEL_STAT(0)|CHANNEL_STAT(1)))  {  ; }	
	exFlash_dma_control->DMA_CHEN &= ~(CHANNEL_ENABLE(0)|CHANNEL_ENABLE(1));
//	exFlash_dma_control->DMA_CONFIG = 0;
	exFlash->SPIDMACR &= (~0x03);	//disable SPI TX RX DMA
}


//----------------------------------------------status register---------------------------------------
/********************************************************************************
* Function Name	: W25QXX_ReadSR
* Description  	: Read the value of the register.
* Input        	: - SRx:Register address
                      - 3 status registers
                      - W25X_READ_SR1: address 0x05
                          BIT   7   6   5   4   3   2   1   0
                               SPR  RV TB  BP2  BP1 BP0 WEL BUSY
                      - W25X_READ_SR2: address 0x35
                      - W25X_READ_SR3: address 0x15
* Output       	: None
* Return       	: None
*********************************************************************************/
uint8_t W25QXX_ReadSR(uint8_t SRx)
{
	uint8_t byte=0;
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(SRx); 
	byte=exFlash_SPI_ReadWriteByte(0xff);
	Flash_SS_SET;
	return byte;
}

/********************************************************************************
* Function Name	: W25QXX_WriteSR
* Description  	: Write the value of the register.
* Input        	: - SRx:Register address
					 - 3 status registers
					    - W25X_WRITE_SR1: address 0x01
					    - W25X_WRITE_SR2: address 0x31
					    - W25X_WRITE_SR3: address 0x11
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_WriteSR(uint8_t SRx,uint8_t Val)
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(SRx);
	exFlash_SPI_ReadWriteByte(Val);
	Flash_SS_SET;
}

/********************************************************************************
* Function Name	: W25QXX_Wait_Busy
* Description  	: Wait for BUSY bit to clear.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_Wait_Busy(void)
{
	while((W25QXX_ReadSR(W25X_READ_SR1)&0x01)==0x01);
}

/********************************************************************************
* Function Name	: W25QXX_Write_Enable
* Description  	: Write Enable(set WEL to 1).
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_Write_Enable(void)
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_WRITE_ENABLE);
	Flash_SS_SET;
}

/********************************************************************************
* Function Name	: W25QXX_Write_EnableVolatile
* Description  	: Write Enable for Volatile Status Register.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_Write_EnableVolatile(void)
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_WRITE_ENABLE_VSR);
	Flash_SS_SET; 
}

/********************************************************************************
* Function Name	: W25QXX_Write_Disable
* Description  	: Write Disable(set WEL to 0).
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_Write_Disable(void)
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_WRITE_DISABLE);
	Flash_SS_SET; 
}

/********************************************************************************
* Function Name	: W25QXX_4ByteAddress_Mode
* Description  	: Enter 4-Byte Address Mode.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_4ByteAddress_Mode(void)
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_ENTER_4BYTE_ADDR);
	Flash_SS_SET;
}

/********************************************************************************
* Function Name	: W25QXX_ReadID
* Description  	: Read W25qxx ID.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
uint16_t W25QXX_ReadID(void)
{
	uint16_t Temp = 0;
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_MANUFACTDEVICE_ID);//ͶȡID
	exFlash_SPI_ReadWriteByte(0x00);
	exFlash_SPI_ReadWriteByte(0x00);
	exFlash_SPI_ReadWriteByte(0x00);
	Temp|=exFlash_SPI_ReadWriteByte(0xFF)<<8;
	Temp|=exFlash_SPI_ReadWriteByte(0xFF);
	Flash_SS_SET;
	return Temp;
}

void W25QXX_SoftReset(void)
{
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(0x66);
	Flash_SS_SET;
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(0x99);
	Flash_SS_SET;
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(0xFF);
	Flash_SS_SET;
}


//------------------------------------------------------------------ Erase --------------------------------------------------------------------------
/********************************************************************************
* Function Name	: W25QXX_SectorErase4K
* Description  	: Erase a sector (4*1024 bytes).
* Input        	: - addr:sector address
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_SectorErase4K(uint32_t addr)
{
	W25QXX_Write_Enable();                 	//set WEL
	
	W25QXX_Wait_Busy();
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_SECTOR_ERASE);
	if(Flash_Type)
		exFlash_SPI_ReadWriteByte((uint8_t)(addr>>24));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>16));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)addr);
	Flash_SS_SET;
	W25QXX_Wait_Busy();   				   	//wait for chip erase to finish
}

/********************************************************************************
* Function Name	: W25QXX_BlockErase32KB
* Description  	: Erase a block (32*1024 bytes).
* Input        	: - addr:block address
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_BlockErase32KB(uint32_t addr)
{
	W25QXX_Write_Enable();                 	//set WEL
	
	W25QXX_Wait_Busy();
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_32KBLOCK_ERASE);
	if(Flash_Type)
		exFlash_SPI_ReadWriteByte((uint8_t)(addr>>24));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>16));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)addr);
	Flash_SS_SET;
	W25QXX_Wait_Busy();   				   	//wait for chip erase to finish
}

/********************************************************************************
* Function Name	: W25QXX_BlockErase64KB
* Description  	: Erase a block (64*1024 bytes).
* Input        	: - addr:block address
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_BlockErase64KB(uint32_t addr)
{
	W25QXX_Write_Enable();                 	//set WEL
	
	W25QXX_Wait_Busy();
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_64KBLOCK_ERASE);
	if(Flash_Type)
		exFlash_SPI_ReadWriteByte((uint8_t)(addr>>24));	
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>16)); 
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)addr);
	Flash_SS_SET; 
	W25QXX_Wait_Busy();   				   	//wait for chip erase to finish
}

/********************************************************************************
* Function Name	: W25QXX_FullChipErase
* Description  	: Full Erase.
* Input        	: None
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_FullChipErase(void)
{
	W25QXX_Write_Enable();                 	 	  //set WEL
	
	W25QXX_Wait_Busy();
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_CHIP_ERASE);
	Flash_SS_SET;
	W25QXX_Wait_Busy();   				   		  //wait for chip erase to finish
}

// W25QXX_Init
void W25QXX_Init(void)
{
	
	if(Flash_ID==0xEF18||Flash_ID==0xC818||Flash_ID==0x0B18)  //256M flash
	{
		Flash_Type=1; 
		W25QXX_4ByteAddress_Mode();
	}

	//W25QXX_QE_Enable();       			//QE
	
}


//------------------------------------------------------------------ Write --------------------------------------------------------------------------
/********************************************************************************
* Function Name	: W25QXX_STD_Write_Page
* Description  	: On a page, write data of specified length at the specified address.
* Input        	: - buf: data start address
                  - addr: Start address of writing data
                  - num: number bytes to write(max one page:256 bytes)
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_STD_Write_Page(uint8_t* buf,uint32_t addr,uint16_t num)
{
	uint16_t i;
	
	W25QXX_Write_Enable();	//set WEL
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_PAGE_PROGRAM);
	if(Flash_Type)exFlash_SPI_ReadWriteByte((uint8_t)(addr>>24));		
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>16));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)addr);
	for(i=0;i<num;i++)exFlash_SPI_ReadWriteByte(buf[i]);
	Flash_SS_SET;
	W25QXX_Wait_Busy();	//wait for writing to end
}

/********************************************************************************
* Function Name	: W25QXX_STD_Write_NoCheck
* Description  	: Write SPI FLASH without inspection.
                  You must ensure that all the data written in the address range is 0XFF, otherwise the data written in non-0XFF will fail.
                  With auto-feed function.
* Input        	: - buf: data start address
                  - addr: Start address of writing data
                  - num: number bytes to write(max 0xFFFF bytes)
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_STD_Write_NoCheck(uint8_t* buf,uint32_t addr,uint16_t num)
{
	uint16_t pageremain;

	pageremain=256-addr%256; // Number of bytes remaining on a single page
	if(num<=pageremain)pageremain=num; // <= 256 bytes
	while(1)
	{
		W25QXX_STD_Write_Page(buf,addr,pageremain);
		if(num==pageremain)break; // Write finished
	 	else //NumByteToWrite>pageremain
		{
			buf+=pageremain;
			addr+=pageremain;

			num-=pageremain;	// Subtract the number of bytes written
			if(num>256)pageremain=256; // 256 bytes can be written at a time
			else pageremain=num; 	  // Not enough 256 bytes
		}
	}
}

/********************************************************************************
* Function Name	: W25QXX_STD_Write_Page_DMA
* Description  	: On a page, write data of specified length at the specified address.
* Input        	: - buf: data start address
                  - addr: Start address of writing data
                  - num: number bytes to write(max one page:256 bytes)
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_STD_Write_Page_DMA(uint8_t* buf,uint32_t addr,uint16_t num)
{
	uint8_t rbuff[256] = {0};

	W25QXX_Write_Enable();	//set WEL
	Flash_SS_RESET; 
	exFlash_SPI_ReadWriteByte(W25X_PAGE_PROGRAM); 
	if(Flash_Type)exFlash_SPI_ReadWriteByte((uint8_t)(addr>>24)); 
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>16)); 
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)addr);
	exFlash_SPI_DMA_Tran(buf,rbuff,num);        //DMA
	exFlash_SPI_DMA_Wait();		
	Flash_SS_SET; 
	W25QXX_Wait_Busy();		// wait for writing to end
}

/********************************************************************************
* Function Name	: W25QXX_STD_Write_NoCheck_DMA
* Description  	: Write SPI FLASH without inspection.
                  You must ensure that all the data written in the address range is 0XFF, otherwise the data written in non-0XFF will fail.
                  With auto-feed function
* Input        	: - buf: data start address
                  - addr: Start address of writing data
                  - num: number bytes to write(max 0xFFFF bytes)
* Output       	: None
* Return       	: None
*********************************************************************************/
void W25QXX_STD_Write_NoCheck_DMA(uint8_t* buf,uint32_t addr,uint16_t num)
{
	uint16_t pageremain;

	pageremain=256-addr%256; // Number of bytes remaining on a single page
	if(num<=pageremain)pageremain=num;// <= 256 bytes
	while(1)
	{
		W25QXX_STD_Write_Page_DMA(buf,addr,pageremain);
		if(num==pageremain)break; // Write finished
	 	else //NumByteToWrite>pageremain
		{
			buf+=pageremain;
			addr+=pageremain;

			num-=pageremain;	// Subtract the number of bytes written
			if(num>256)pageremain=256; // 256 bytes can be written at a time
			else pageremain=num; 	  // Not enough 256 bytes
		}
	}
}


//------------------------------------------------------------------ Read --------------------------------------------------------------------------
void W25QXX_STD_Read(uint8_t* buf, uint32_t addr, uint32_t num)
{
	uint32_t i;
	
	Flash_SS_RESET;
	exFlash_SPI_ReadWriteByte(W25X_READ);
	if(Flash_Type)exFlash_SPI_ReadWriteByte((uint8_t)(addr>>24));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>16));
	exFlash_SPI_ReadWriteByte((uint8_t)((addr)>>8));
	exFlash_SPI_ReadWriteByte((uint8_t)addr);
	for(i=0;i<num;i++)
	{
		buf[i]=exFlash_SPI_ReadWriteByte(0XFF);
	}
	Flash_SS_SET;	
}

void NorFlash_DMA_Read(uint8_t* Tx_Addr, uint8_t* Rx_Addr, uint32_t length)
{
	exFlash->SPIDMACR |= 0x03;	//enable SPI TX RX DMA
	exFlash_dma_control->DMA_CONFIG = 1;
	
	exFlash_dma_channel[0]->DMA_SADDR = (uint32_t)0xff;
	exFlash_dma_channel[0]->DMA_CTRL_HIGH = length ;
	exFlash_dma_channel[1]->DMA_DADDR = (uint32_t)Rx_Addr;
	exFlash_dma_channel[1]->DMA_CTRL_HIGH = length ;
	
//	exFlash_dma_control->DMA_MASKTFR |= (CHANNEL_UMASK(0)|CHANNEL_UMASK(1));
	exFlash_dma_control->DMA_CHEN |= (CHANNEL_WRITE_ENABLE(0)|CHANNEL_ENABLE(0)|CHANNEL_WRITE_ENABLE(1)|CHANNEL_ENABLE(1));
}

void NorFlash_DMA_Wait(void)
{
//	while((exFlash_dma_control->DMA_RAWTFR & (CHANNEL_STAT(0) | CHANNEL_STAT(1))) != (CHANNEL_STAT(0) | CHANNEL_STAT(1)));
//	exFlash_dma_control->DMA_CLRTFR = (CHANNEL_STAT(0) | CHANNEL_STAT(1));
	while(exFlash_dma_control->DMA_CHEN & (CHANNEL_STAT(0)|CHANNEL_STAT(1)))  {  ; }	
	exFlash_dma_control->DMA_CHEN &= ~(CHANNEL_ENABLE(0)|CHANNEL_ENABLE(1));
//	exFlash_dma_control->DMA_CONFIG = 0;
	exFlash->SPIDMACR &=~0x03;	//disable SPI TX RX DMA	

}



