/************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
  * @file     debug.c
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-01
  * @brief    
 *************************************************************************/

#include "debug.h"
#include "ioctrl_drv.h"
#include "dma.h"

// Add the following code to support the printf function without selecting use MicroLIB
#pragma import(__use_no_semihosting)
// Solve some bugs that may report errors when using HAL library
int _ttywrch(int ch)
{
	ch = ch;
	return ch;
}
// Support functions required by the standard library
struct __FILE
{
	int handle;
	/* Whatever you require here. If the only file you are using is */
	/* standard output using printf() for debugging, no file handling */
	/* is required. */
};
/* FILE is typedef d in stdio.h. */
FILE __stdout;
// define_sys_exit() to avoid using half-host mode
void _sys_exit(int x)
{
	x = x;
}

/*****************************uart printf for debug*****************************/
// redefine printf function
int fputc(int ch, FILE *f)
{
#if UART_DEBUG
	UART_TypeDef *UARTx = SCI1;

	UARTx->SCICR2 |= SCICR2_TE_MASK;

	while ((UARTx->SCISR1 & SCISR1_TDRE_MASK) == 0x00)
	{
		;
	}
	UARTx->SCIDRL = (uint8_t)ch;
	while ((UARTx->SCISR1 & SCISR1_TC_MASK) == 0x00)
	{
		;
	}

	UARTx->SCICR2 &= ~SCICR2_TE_MASK;

	return ch;
#else

	while ((COMuart->SCISR1 & 0x40) == 0x00)
	{
	};
	COMuart->SCIDRL = (uint8_t)ch;

	return ch;
#endif
}

static UART_TypeDef *UART;
// pclk:PCLK clock frequency(Mhz)
// bound:Baud rate
void UART_Debug_Init(UART_TypeDef *UARTx, uint32_t pclk, uint32_t bound)
{
	// reserved
	//	if(UARTx==SCI1)
	//	{
	//		IO_Ctrl_SCI_Swap(TX1_GINT4,TRUE);	//OPEN TX1 RX1 function
	//		IO_Ctrl_SCI_Swap(RX1_GINT0,TRUE);
	//	}
	//	else if(UARTx==SCI2)
	//	{
	//		IO_Ctrl_SCI_Swap(TX2_GINT5,TRUE);
	//		IO_Ctrl_SCI_Swap(RX2_GINT3,TRUE);
	//	}
	//	else if(UARTx==SCI3)
	//	{
	//		IO_Ctrl_SCI_Swap(TX3_GINT1,TRUE);
	//		IO_Ctrl_SCI_Swap(RX3_GINT2,TRUE);
	//	}

	uint32_t band_rate = 0;

	UART = UARTx;

	UART->SCIBRDF = (((pclk * 8 / bound) + 1) / 2) & 0x003f;
	band_rate = (pclk * 4 / bound) >> 6;
	UART->SCIBDH = (uint8_t)((band_rate >> 8) & 0x00ff);
	UART->SCIBDL = (uint8_t)(band_rate & 0x00ff);

	UART->SCICR2 = 0x00;
	UART->SCICR1 = 0x00;
}

void assert_failed(uint8_t *file, uint32_t line)
{
//	printf("Wrong parameters value: file %s on line %d\r\n", file, line);
	while (1)
		;
}
