/************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
  * @file     dma.h
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-01
  * @brief    
 *************************************************************************/

#ifndef _DMA_H
#define _DMA_H

#include "type.h"
#include "dmac_drv.h"

void SPI2_DMA_Tran(uint8_t *Tx_Addr, uint8_t *Rx_Addr, uint32_t length);
void SPI2_DMA_Wait(void);

// Configure the destination hardware handshake interface
// DMAC1
#define DST1_SPI1_TX (0x0 << 11)
#define DST1_SPI2_TX (0x1 << 11)
#define DST1_SSI2_TX (0x2 << 11)
#define DST1_SPI1_RX (0x3 << 11)
#define DST1_SPI2_RX (0x4 << 11)
#define DST1_SSI2_RX (0x5 << 11)
#define DST1_ADC 	 (0x6 << 11)
#define DST1_SCI1_TX (0xa << 11)
#define DST1_SCI1_RX (0xb << 11)
#define DST1_SCI3_TX (0xc << 11)
#define DST1_SCI3_RX (0xd << 11)
// DMAC2
#define DST2_SCI2_RX (0x0 << 11)
#define DST2_SCI2_TX (0x1 << 11)
#define DST2_SSI1_RX (0x2 << 11)
#define DST2_SSI1_TX (0x3 << 11)
#define DST2_SPI3_RX (0x4 << 11)
#define DST2_SPI3_TX (0x5 << 11)
#define DST2_DAC 	 (0x8 << 11)
#define DST2_SDDC 	 (0xc << 11)

// Configure the source hardware handshake interface
// DMAC1
#define SRC1_SPI1_TX (0x0 << 7)
#define SRC1_SPI2_TX (0x1 << 7)
#define SRC1_SSI2_TX (0x2 << 7)
#define SRC1_SPI1_RX (0x3 << 7)
#define SRC1_SPI2_RX (0x4 << 7)
#define SRC1_SSI2_RX (0x5 << 7)
#define SRC1_ADC	 (0x6 << 7)
#define SRC1_SCI1_TX (0xa << 7)
#define SRC1_SCI1_RX (0xb << 7)
#define SRC1_SCI3_TX (0xc << 7)
#define SRC1_SCI3_RX (0xd << 7)
// DMAC2
#define SRC2_SCI2_RX (0x0 << 7)
#define SRC2_SCI2_TX (0x1 << 7)
#define SRC2_SSI1_RX (0x2 << 7)
#define SRC2_SSI1_TX (0x3 << 7)
#define SRC2_SPI3_RX (0x4 << 7)
#define SRC2_SPI3_TX (0x5 << 7)
#define SRC2_DAC 	 (0x8 << 7)
#define SRC2_SDDC 	 (0xc << 7)

/*DMA resource*/
/*----------------------------------------------------------------------------------------*/
//		DMA1_BASE_ADDR	:  spi1			|	spi2	|	SSI2	|	SCI1	|	SCI3	|
//		channel			:	0	  		|	2		|	none	|	     	|	3		| 		//  dma1 have 4 channels: 0~3
//		function		: SD,SPI_tft	|	exFlash |			|	 	    |   uart    |
//------------------------------------------------------------------------------------------
//		DMA1_BASE_ADDR	:	spi3		|	SSI1	|	SCI2	|	DAC		|
//		channel			:	1			|	0		|			|	1		|		//dma2 have 2 channels: 0~1
//		function		:	tft		    |	        |			|	WAV		|
/*----------------------------------------------------------------------------------------*/

// TFTspi dma
#define TFTSPI SPI1 // spix
#define TFTSPI_DST_TX DST1_SPI1_TX
#define TFTSPI_SRC_RX SRC1_SPI1_RX
#define TFTSPI_DMA_BASE_ADDR DMA1_BASE_ADDR   // dma bus
#define TFTSPI_CHNUM 2                        // dma channel
extern DMA_CHANNEL_REG *TFTspi_dma_channel[]; // global struct variable for for Channel registers
extern DMA_CONTROL_REG *TFTspi_dma_control;   // global struct variable for for DMAC registers

// SD_SPI dma
#define SDSPI SPI1 // spix
#define SDSPI_DST_TX DST1_SPI1_TX
#define SDSPI_SRC_RX SRC1_SPI1_RX
#define SDSPI_DMA_BASE_ADDR DMA1_BASE_ADDR   // dma bus
#define SDSPI_CHNUM 2                        // dma channel
extern DMA_CHANNEL_REG *SDSPI_dma_channel[]; // global struct variable for for Channel registers
extern DMA_CONTROL_REG *SDSPI_dma_control;   // global struct variable for for DMAC registers

// external FLASH dma
#define exFlash SPI2 // spix
#define exFlash_DST_TX DST1_SPI2_TX
#define exFlash_SRC_RX SRC1_SPI2_RX
#define exFlash_DMA_BASE_ADDR DMA1_BASE_ADDR   // dma bus
#define exFlash_CHNUM 1                       // dma channel
extern DMA_CHANNEL_REG *exFlash_dma_channel[]; // global struct variable for for Channel registers
extern DMA_CONTROL_REG *exFlash_dma_control;   // global struct variable for for DMAC registers

// uart communication dma
#define COMuart SCI3 // scix,NVIC_Init(0, 0, SCI1_IRQn, 2);
#define COMuart_DST_TX DST1_SCI3_TX
#define COMuart_SRC_RX SRC1_SCI3_RX
#define COMuart_DMA_BASE_ADDR DMA1_BASE_ADDR   // dma bus
#define COMuart_CHNUM 3                        // dma channel
extern DMA_CHANNEL_REG *COMuart_dma_channel[]; // global struct variable for for Channel registers
extern DMA_CONTROL_REG *COMuart_dma_control;   // global struct variable for for DMAC registers

// DAC dma
#define WAVdac DAC
#define WAVdac_DST_TX DST2_DAC
#define WAVdac_DMA_BASE_ADDR DMA2_BASE_ADDR   // dma bus
#define WAVdac_CHNUM 1                        // dma channel
extern DMA_CHANNEL_REG *WAVdac_dma_channel[]; // global struct variable for for Channel registers
extern DMA_CONTROL_REG *WAVdac_dma_control;   // global struct variable for for DMAC registers

#endif /* _DMA_H */
