/************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
  * @file     qspi.c
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-01
  * @brief    
 *************************************************************************/

#include "qspi.h"
#include "sys.h"
#include "dmac_drv.h"
#include "ssi_drv.h"
#include "w25qxx.h"
#include "transport.h"
#include "dma.h"

void SSI1_STD_Init(SSI_TypeDef *SSIx)
{
	while (SSIx->SSI_SR & SR_BUSY)
	{
		;
	}
	SSIx->SSI_SSIENR = 0x00;
	SSIx->SSI_BAUDR = 0x0C;	 // 0x0c
	SSIx->SSI_CTRLR1 = 0x00;
	//SSIx->SSI_CTRLR0 = (CTRLR0_SPI_FRF_STD|CTRLR0_FRF_MOT|CTRLR0_SSTE_MASK(0)|CTRLR0_TMOD_TX_AND_RX|CTRLR0_SCPOL_MASK(0)|CTRLR0_SCPH_MASK(0)|CTRLR0_DFS_VALUE(7));
	SSIx->SSI_CTRLR0 = (CTRLR0_SPI_FRF_STD | CTRLR0_FRF_MOT | CTRLR0_DFS_VALUE(7) | CTRLR0_TMOD_TX_AND_RX);

	SSIx->SSI_SPI_CTRLR0 = (CTRLR0_TRANS_TYPE_TT0		   // CMD,address,dummy model
							| CTRLR0_INST_L_VALUE(0)	   // cmd_len:     0=0bit1=4bit2=8bit, 3=16bit
							| CTRLR0_ADDR_L_VALUE(0)	   // addr_len:    n*4 bit (max n = 15)
							| CTRLR0_WAIT_CYCLES_VALUE(0)  // dummy_len:   n bit   (max n = 31)
							| CTRLR0_CLK_STRETCH_EN_MASK); // Clock extension enable(TX_ONLY necessary)

	SSIx->SSI_TXFTLR = TXFTLR_TFT(0) | TXFTLR_TXFTHR(0);
	SSIx->SSI_RXFTLR = RXFTLR_TFT(0);
	if (QUAD_BAUDR == 0x02 && g_sys_clk >= 80000000) // high speed
	{
		SSIx->SSI_RX_SAMPLE_DELAY = 0x00010001;
	}
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}

void SSI1_STD_DMA_Send(SSI_TypeDef *SSIx, int dma_ch, uint8_t *psend, uint32_t addr, uint32_t length, BOOL binten)
{
	int num;

	m_dma2_control->DMA_CONFIG = 1;
	// Tx
	m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)psend;
	m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)&SSIx->SSI_DR;
	if (binten == TRUE)
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = INTEN | DNOCHG | SIEC | M2P_DMA | DWIDTH_B | SWIDTH_B | SBSIZE_4 | DBSIZE_4;
	}
	else
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = DNOCHG | SIEC | M2P_DMA | DWIDTH_B | SWIDTH_B | SBSIZE_4 | DBSIZE_4;
	}
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = length & 0xFFF; // max 0x0FFF
	m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_DST_HARD);
	m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(3);

	// enable dma channel
	m_dma2_control->DMA_CHEN |= (CHANNEL_WRITE_ENABLE(dma_ch) | CHANNEL_ENABLE(dma_ch));

	SSIx->SSI_DR = W25X_PAGE_PROGRAM;
	if (Flash_Type == 1)
		SSIx->SSI_DR = (addr >> 24) & 0xff;
	SSIx->SSI_DR = (addr >> 16) & 0xff;
	SSIx->SSI_DR = (addr >> 8) & 0xff;
	SSIx->SSI_DR = (addr >> 0) & 0xff;
	SSIx->SSI_DMACR = DMACR_TDMAE;

	while (m_dma2_control->DMA_CHEN & CHANNEL_STAT(dma_ch))
	{
	}
	m_dma2_control->DMA_CHEN &= ~(CHANNEL_WRITE_ENABLE(dma_ch) | CHANNEL_ENABLE(dma_ch));
	m_dma2_channel[dma_ch]->DMA_CFG &= ~(1 << 8);

	SSIx->SSI_DMACR = 0;
}

void SSI1_QUAD_Init(int read, int num, int frame_len, int cmd_len, int addr_len, int dummy_len)
{
	SSI_TypeDef *SSIx = SSI1;

	SSIx->SSI_SSIENR = 0x00;
	SSIx->SSI_BAUDR = 0x04;
	SSIx->SSI_CTRLR1 = ((num == 0) ? 0 : (num - 1)); // max = 0xFFFF

	if (read)
		SSIx->SSI_CTRLR0 = (CTRLR0_SPI_FRF_QUAD | CTRLR0_FRF_MOT | CTRLR0_SSTE_MASK(0) | CTRLR0_TMOD_RX_ONLY | CTRLR0_SCPOL_MASK(0) | CTRLR0_SCPH_MASK(0) | CTRLR0_DFS_VALUE(frame_len));
	else
		SSIx->SSI_CTRLR0 = (CTRLR0_SPI_FRF_QUAD | CTRLR0_FRF_MOT | CTRLR0_SSTE_MASK(0) | CTRLR0_TMOD_TX_ONLY | CTRLR0_SCPOL_MASK(0) | CTRLR0_SCPH_MASK(0) | CTRLR0_DFS_VALUE(frame_len));
	// |CTRLR0_CFS_VALUE(frame_len)
	SSIx->SSI_SPI_CTRLR0 = (CTRLR0_TRANS_TYPE_TT2				  // CMD,address,dummy model
							| CTRLR0_INST_L_VALUE(cmd_len)		  // cmd_len:     0=0bit1=4bit2=8bit, 3=16bit
							| CTRLR0_ADDR_L_VALUE(addr_len)		  // addr_len:    n*4 bit (max n = 15)
							| CTRLR0_WAIT_CYCLES_VALUE(dummy_len) // dummy_len:   n bit   (max n = 31)
							| CTRLR0_CLK_STRETCH_EN_MASK);		  // Clock extension enable(TX_ONLY necessary)

	SSIx->SSI_TXFTLR = TXFTLR_TFT(0) | TXFTLR_TXFTHR(1);
	SSIx->SSI_RXFTLR = RXFTLR_TFT(7);
	//		SSIx->SSI_TXFLR = RXFTLR_TFT(1);
	//		SSIx->SSI_RXFLR = RXFTLR_TFT(1);
	if (SSIx->SSI_BAUDR == 0x02 && g_sys_clk >= 80000000) // high speed
	{
		SSIx->SSI_RX_SAMPLE_DELAY = RXSDR_SE(1) | RXSDR_RSD(1); // Sampling delay
	}
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}

void SSI1_QUAD_Transmit_TX(unsigned char *buf, unsigned int addr, int num)
{
	volatile unsigned char temp;
	SSI_TypeDef *SSIx = SSI1;

	SSI1_QUAD_Init(SSI_TX, num, SSI_16BIT, 2, 4, 0);

	SSIx->SSI_DR = 0x12;
	SSIx->SSI_DR = 0x002C00;

	while (num > 0)
	{
		if (SSIx->SSI_SR & SR_TFNF)
		{
			SSIx->SSI_DR = *buf;
			buf++;
			num--;
		}
	}

	while (SSIx->SSI_SR & SR_BUSY)
	{
	}

	while (SSIx->SSI_SR & SR_RFNE)
	{
		temp = SSIx->SSI_DR;
	}
}

void SSI1_QUAD_Transmit_DMA(int read, int buf_model, uint16_t *buff, uint32_t num)
{
	SSI_TypeDef *SSIx = SSI1;
	//	SSIx->SSI_DMACR=DMACR_TDMAE;
	TFTspi_dma_control->DMA_CONFIG = 1;
	if (read) // RX
	{
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_SADDR = (uint32_t)&SSIx->SSI_DR;
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_DADDR = (uint32_t)buff;
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL = DIEC | SNOCHG | P2M_DMA | DWIDTH_HW | SWIDTH_HW | SBSIZE_1 | DBSIZE_1; // 16bit*1/frame
		if (buf_model == 1)
			TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL |= DDEC;
		else if (buf_model == 2)
			TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL |= DNOCHG;

		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL_HIGH = (num & 0xFFF); // frame max 0x0FFF
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG = (HS_SEL_SRC_HARD | HS_SEL_DST_SOFT);
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG_HIGH = TFTSPI_SRC_RX;
	}
	else // Tx
	{
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_SADDR = (uint32_t)buff;
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_DADDR = (uint32_t)&SSIx->SSI_DR;
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL = DNOCHG | SIEC | M2P_DMA | DWIDTH_HW | SWIDTH_HW | SBSIZE_1 | DBSIZE_1; // 16bit*1/frame
		if (buf_model == 1)
			TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL |= SDEC;
		else if (buf_model == 2)
			TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL |= SNOCHG;
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CTRL_HIGH = (num & 0xFFF); // frame max 0x0FFF
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG = (HS_SEL_SRC_SOFT | HS_SEL_DST_HARD);
		TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG_HIGH = TFTSPI_DST_TX;
	}

	// enable dma channel
	TFTspi_dma_control->DMA_CHEN |= (CHANNEL_WRITE_ENABLE(TFTSPI_CHNUM) | CHANNEL_ENABLE(TFTSPI_CHNUM));

	SSIx->SSI_DR = 0x3200;
	SSIx->SSI_DR = 0x2C00;
	SSIx->SSI_DMACR = DMACR_TDMAE;
}

void SSI1_QUAD_Transmit_DMA_Waite(void)
{
	SSI_TypeDef *SSIx = SSI1;
	while (TFTspi_dma_control->DMA_CHEN & CHANNEL_STAT(TFTSPI_CHNUM))
	{
	}
	TFTspi_dma_control->DMA_CHEN &= ~(CHANNEL_WRITE_ENABLE(TFTSPI_CHNUM) | CHANNEL_ENABLE(TFTSPI_CHNUM));
	// TFTspi_dma_channel[TFTSPI_CHNUM]->DMA_CFG &= ~(1<<8);
	SSIx->SSI_DMACR = 0;
}
