/************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
  * @file     uart.h
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-01
  * @brief     
 *************************************************************************/

#ifndef _UART_H
#define _UART_H

#include "type.h"
#include "sys.h"
#include "stdio.h"
#include "memmap.h"
#include "uart_drv.h"
#include "transport.h"

#define BAUDRATE 115200

extern uint8_t page_refreshing;

// extern uint8_t Rx_Buffer_short[256];
extern uint8_t Rx_Buffer_long[];

extern uint16_t Rx_Count_long;
extern uint16_t Rx_Num;

unsigned short CRC16(unsigned char *puchMsg, unsigned short usDataLen);

void Uart_Init(uint32_t pclk, uint32_t bound);

void UartTFT_SCI_Init(UART_TypeDef *UARTx, uint32_t bound);
void LT_SingleByteToPc(uint8_t val);
void LT_ClearRxBuf(void);
void LT_SendData_CRC_Frame(uint8_t *buf, uint8_t len);
void LT_SendData_CRC_NoFrame(uint8_t *buf, uint8_t len);
void Main_From_UartRxBuff(void);

// MODBUS: variables sent and received by the host
extern volatile uint8_t tx_en_flag;
extern volatile uint8_t tx_repeat_flag;
extern volatile uint8_t tx_repeat_cnt;

typedef struct
{
	uint8_t Flag;	  // Valid flag bit is 0xA5
	uint8_t Sla_Add;  // Modbus slave node
	uint8_t CMD;	  // Modbus read and write instructions
	uint16_t REG_Add; // Modbus slave register address
	uint8_t Len;	  // read and write data length(Bytes)
	uint16_t Var_Add; // Modbus host variable address
	uint16_t Timeout; // Maximum response time
	uint8_t RP_Cnt;	  // Maximum number of retransmissions
	uint8_t Mode;	  // Instruction execution mode
	uint16_t Opt_Add; // Parameters corresponding to instruction execution mode
} USART_TX_INFO;
extern volatile USART_TX_INFO gUsartTx; // For serial port reception

extern volatile uint8_t Master_mode03_flag[]; // Customized variables
extern volatile uint8_t Master_mode03_Var[];  // Customized variables

void Uart_cmd_Send(void);

#endif /* _UART_H */
