/************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
  * @file     uart_update.h
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-01
  * @brief     
 *************************************************************************/

#ifndef _UART_UPDATE_H
#define _UART_UPDATE_H

#include "type.h"

#define U_MCUFLASH_FLAG 2                // 0 set current mcu as factory version(need jump to factoy when use uart update)
                                         // 1 set current mcu as update version
                                         // 2 NULL
#define U_MCUFLASH_VERSION "V23.0130.10" // current mcu version(strlen <= 12)

/*----------------------------mcu flash---------------------------------------------*/
#define U_MCUFLASH_FACTORY_ADDR 0x08008000 	// mcu factory start address( size <= 1024*236 )
#define U_MCUFLASH_UPDATE_ADDR  0x08043000  // mcu update start address( size <= 1024*236 )
#define U_MCUFLASH_INFOADDR     0x0807E400	// mcu info start address( size <= 256 )

/*---------------------------spi flash-------------------------------*/
#define U_SPIFLASHADDR	   (128 / 8 * 1024 * 1024 / 2)	// 128Mbit spi flash buffer start address
// #define U_SPIFLASHADDR  (256 / 8 * 1024 * 1024 / 2)	//256Mbit spi flash buffer start address

/*analyse*/
uint8_t U_UpdateCMDanalyse(uint8_t cmd, uint8_t *buf);
uint32_t GetCrc32(uint8_t *InStr, uint32_t len);

/* MCU */
uint8_t U_McuFlash_Update(uint32_t u_addr, uint16_t u_len); // address + U_MCUFLASH_UPDATE_ADDR
uint32_t U_McuFlash_ReadDataCrc(uint32_t u_addr, uint32_t u_len); // address + U_MCUFLASH_UPDATE_ADDR
uint32_t U_McuFlash_DataToWorkspace(uint32_t u_addr, uint32_t u_len, uint32_t u_crc);
uint8_t U_McuFlash_GetVersion(uint8_t *buf);
uint8_t U_McuFlash_Factory(void);
uint8_t U_McuFlash_Bsp(void);
uint8_t U_McuFlash_InfoChange(uint8_t *buf, uint16_t u_len);
uint8_t U_McuFlash_InfoClear(void);
uint8_t U_McuFlash_ReadData(uint8_t *buf, uint16_t u_addr, uint16_t u_len);

/* SPI Flash  */
uint8_t U_SpiFlash_Update(uint32_t u_addr, uint16_t u_len);
uint32_t U_SpiFlash_ReadDataCrc(uint32_t u_addr, uint32_t u_len);
uint32_t U_SpiFlash_DataToWorkspace(uint32_t u_addr, uint32_t u_len, uint32_t u_crc);
uint8_t U_SpiFlash_ReadData(uint8_t *buf, uint16_t u_addr, uint16_t u_len);

uint8_t U_SpiFlashDisableScreen(void); // disable screen
uint8_t U_McuFlash_CheckBsp(void);     // check update package is ok ? jump to update version: jump to factory version
void APP(uint32_t appxaddr);           // application jump

typedef struct
{
  uint8_t index;
  uint8_t msg;
  uint32_t addr;
  uint32_t bak_addr;

  uint16_t dsum;
  uint8_t dbuf[2048];
  uint8_t updateflag; // disable screen

} UART_UPDATE_;
extern UART_UPDATE_ U_UPDATE;

#define U_MSG_OK            0
#define U_MSG_FORMAT_ERROR  1
#define U_MSG_CRC_ERROR     2
#define U_MSG_CARRY_FAILURE 3

#endif /* _UART_UPDATE_H */
