/*******************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.   
  * @file     levetop.h
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-03
  * @brief    This file contains the interface information acquisition and 
  *           refresh of the UartTFT Application.
********************************************************************************/ 

/* public version: 2023.03.08 000 */ 

#ifndef _LEVETOP_H
#define _LEVETOP_H

#include "bsp.h"
#include "data.h"
#include "transport.h"
#include "tftlcd.h"
#include "function.h"

extern uint32_t page_info_addr ;
extern uint32_t page_info_sum ;
extern uint32_t ZK_info_addr ;
extern uint32_t ZK_info_sum ;
extern uint32_t Wav_info_addr ;
extern uint32_t Wav_info_sum ;
extern uint32_t pic_info_addr ;
extern uint32_t pic_info_sum ;
extern uint32_t gif_info_addr ;
extern uint32_t gif_info_sum ;
extern uint32_t icon_info_addr ;
extern uint32_t icon_info_sum ;

extern uint32_t ModBus_CMD_info_addr;	
extern uint32_t ModBus_CMD_info_sum;
extern uint32_t addr_index[];

void LT_ReadParam(void);
void Address_Index(void);
void Get_ModBus_CMD_info(void);

uint8_t Get_Touch_info(uint16_t id);            //get page info
uint8_t Get_date_variable_Id(uint16_t id);      //get page info

uint8_t page_date_update(uint16_t id);                     //page refresh
uint8_t Get_date_variable_Id_Regular_update(uint16_t id);  //var refresh

uint8_t Get_date_variable_Id_download(uint16_t id);
uint8_t Get_Touch_info_download(uint16_t id);


/***********************************************************************************
* Function Name	: 
* Description	: Show a complete picture.
* Input			: - x,y: display coordinates of pictures on the screen
				  - Width,High,Addr: width,height and first address of the picture
* Output		: None
* Return		: None
************************************************************************************/
void LT268_TFT_ShowPicture    		  (uint16_t x,uint16_t y,uint16_t Width,uint16_t High,uint32_t Addr);
void LT268_TFT_ShowPicture_Png		  (uint16_t x,uint16_t y,uint16_t Width,uint16_t High,uint32_t Addr);
void LT268_TFT_ShowPicture_Png_Pixel(uint16_t x,uint16_t y,uint16_t Width,uint16_t High,uint32_t Addr);
void display_JPG					(uint16_t x,uint16_t y,uint16_t Width,uint16_t High,uint32_t Addr);
void LT268_TFT_ShowPicture_zip      (uint16_t x,uint16_t y,uint16_t Width,uint16_t High,uint32_t Addr);

/***************************************************************************************
* Function Name	: LT268_dispaly_CutPicture
* Description	: Show part of a picture.
* Input			:  - tft_x,tft_y: display coordinates of pictures on the screen
				   - cut_x,cut_y,cut_w,cut_h: display area of picture
				   - pic_w,pic_addr,pic_len,pic_flag: width,first address,len ang flag of the picture
* Output		: None
* Return		: None
****************************************************************************************/
void LT268_dispaly_CutPicture(
	uint16_t tft_x,uint16_t tft_y,
	uint16_t cut_x,uint16_t cut_y,uint16_t cut_w,uint16_t cut_h,
	uint16_t pic_w,uint32_t pic_addr,uint32_t pic_len,uint8_t pic_flag);

//void LT268_TFT_FullScreen_Cut	(uint16_t x,uint16_t y,uint16_t Width,uint16_t High,uint32_t Addr,uint16_t Color);

/******************************************************************************************************
* Function Name	: LT268_pic_to_buffer
* Description	: Capture a part of the image to the specified area of the buffer
* Input			:  - buff_x,buff_y,buff_w,m_Data: start coordinate on buffer,total width and first address of buffer(array m_PngData)
				   - pic_x,pic_y,pic_w,pic_addr: start coordinate on picture,pic_w and first address of the picture
				   - Width,High: Width and height of transmitted pictures
				   - Cflag: Cflag=0 Fill pic data to m_ Data.
						    Cflag=1 Filter the black data in pic RGB565 data and fill it to m_ Data.
						    Cflag=2 Mix the pic ARGB4444 data with the RGB565 data in buff and fill it to m_ Data.
						    Cflag=3 Decoded RGB565 data of JPG picture from m_ Data [0] starts linear filling.
						    Cflag=4 Decode the zip image data from m_ Data [0] starts linear filling.
* Output		: None
* Return		: None
********************************************************************************************************/
uint8_t LT268_pic_to_buffer(
uint16_t buff_x,uint16_t buff_y,uint16_t buff_w,uint8_t *m_Data,
uint16_t pic_x, uint16_t pic_y, uint16_t pic_w, uint32_t pic_addr,
uint16_t Width, uint16_t High,  uint8_t Cflag); 



//uint8_t LT268_PicBlending(
//uint16_t pic_x0, uint16_t pic_y0, uint16_t pic_w0, uint32_t pic_addr0,
//uint16_t pic_x1, uint16_t pic_y1, uint16_t pic_w1, uint32_t pic_addr1,
//uint16_t buff_x, uint16_t buff_y, uint16_t buff_w, uint8_t *m_Data,
//uint16_t Width,  uint16_t High,  uint16_t alpha, uint8_t Type);  


/********************************************************************************
* Function Name	: LT268_buffer_to_TFT
* Description  	: Transfer the color data of the buffer to the screen.
* Input        	: - buff_x,buff_y,buff_w,m_Data:  start coordinate on buffer,total width and first address of buffer(array m_PngData)
				  - TFT_x,TFT_y: display start coordinate of screen
				  - Width,High: Width and height of pixels
* Output		: None
* Return		: None
*********************************************************************************/
uint8_t LT268_buffer_to_TFT(
uint16_t buff_x,uint16_t buff_y,uint16_t buff_w,uint8_t *m_Data,
uint16_t TFT_x, uint16_t TFT_y, uint16_t Width, uint16_t High);




#endif	/* _LEVETOP_H */
