// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : eflash_demo.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "libEFlash.h"
#include "eflash_drv.h"
#include "eflash_demo.h"
#include "type.h"
#include "debug.h"
#include "cpm_drv.h"

#define DATA_FIELDS_BEGIN_ADDR   0x08004000
#define DTAT_FIELDS_PAGE_NUM     0x100		

void EFLASH_Words_Read(uint32_t addr, uint32_t *des, uint32_t len);

void EFlash_Demo(void)
{
	uint32_t page = 0;
	uint32_t eflash_clk;
	uint32_t success_counts_page_erase = 0;
	uint32_t success_counts_program = 0;
	uint32_t success_counts_compare = 0;
	uint32_t failure_counts_page_erase = 0;
	uint32_t failure_counts_program = 0;
	uint32_t failure_counts_compare = 0;
	uint32_t page_addr = DATA_FIELDS_BEGIN_ADDR;
	uint32_t write_buf[EFLASH_PAGE_SIZE_WORD];
	uint32_t read_buf[EFLASH_PAGE_SIZE_WORD];
	uint8_t st;
	uint8_t i;
	
	printf ("*******************Eflash Test start*******************\r\n");
	
	eflash_clk = g_sys_clk;
	EFLASH_Init(eflash_clk/1000);
	EFLASH_SetWritePermission();
	for (page = 0; page < DTAT_FIELDS_PAGE_NUM; page++, page_addr +=  0x200)
	{
		st = EFLASH_PageErase(page_addr);
		if (st == FALSE)
		{
			success_counts_page_erase ++;
			printf("EFLASH_PageErase [page_addr = 0x%08x] Failure \r\n", page_addr);
			continue;
		}
		success_counts_page_erase ++;
		//printf("EFLASH_PageErase [page_addr = 0x%08x] success \r\n", page_addr);
		//printf("write data: \r\n");
		for (i = 0; i < EFLASH_PAGE_SIZE_WORD; i++)
		{
			write_buf[i] = (page<<8)+i;;
			//printf("0x%08x ", write_buf[i]);
		}
		//printf("\r\n");
		st = EFLASH_WordsProg(page_addr,write_buf,EFLASH_PAGE_SIZE_WORD);
		if (st == FALSE)
		{
			failure_counts_program ++;
			printf("EFLASH_WordsProg [page_addr = 0x%08x] Failure \r\n", page_addr);
			//continue;
		}
		else
			success_counts_program ++;
			//printf("EFLASH_WordsProg [page_addr = 0x%08x] success \r\n", page_addr);
		
		EFLASH_Words_Read(page_addr, read_buf, EFLASH_PAGE_SIZE_WORD);

		st = 0;
		//printf("read data: \r\n");
		for (i = 0; i < EFLASH_PAGE_SIZE_WORD; i++)
		{
			//printf("0x%08x ", read_buf[i]);
			if (write_buf[i] != read_buf[i])
			{
				st = 1;
			}
		}
		//printf("\r\n");
		if (!st)
		{
			success_counts_compare ++;
			//printf("***********Test EFLASH Success****************\r\n");
		}
		else
		{
			failure_counts_compare ++;
			printf("Compare [page_addr = 0x%08x] Failure \r\n", page_addr);
			//printf("***********Test EFLASH Failure****************\r\n");
		}
		EFLASH_PageErase(page_addr);
	}
	
	printf("Test page counts         :0x%08x\r\n", DTAT_FIELDS_PAGE_NUM);
	printf("Page erase failure counts:0x%08x\r\n", failure_counts_page_erase);
	printf("Program failure counts:   0x%08x\r\n", failure_counts_program);
	printf("Compare failure counts:   0x%08x\r\n", failure_counts_compare);
	printf("Page erase success counts:0x%08x\r\n", success_counts_page_erase);
	printf("Program success counts:   0x%08x\r\n", success_counts_program);
	printf("Compare success counts:   0x%08x\r\n", success_counts_compare);
	printf("*******************Eflash Test End*******************\r\n");
}

/*******************************************************************************
* Function Name  : EFLASH_WordsRead
* Description    :  EFLASHĿַȡΪlenword
* Input          : - addr: EFLASH destination address
*                  - len:  Ҫȡwordݵĳ
*
* Output         : - des:  ȡwordݻ
* Return         : None
******************************************************************************/
void EFLASH_Words_Read(uint32_t addr, uint32_t *des, uint32_t len)
{
	uint32_t i;

	for(i = 0; i < len; i ++)
	{
		des[i] = (*(volatile uint32_t *)(addr+i*4));
	}
}

