#include "type.h"
#include "delay.h"
#include "debug.h"
#include "ssi_drv.h"
#include "ssi_demo.h"
#include "dmac_drv.h"
#include "trng_drv.h"
#include "ioctrl_drv.h"
#include <string.h>

#define SSI_PAGE_SIZE		0x100
#define SSI_SECTOR_SIZE	0x1000
#define SSI_DATA_LEN		SSI_PAGE_SIZE 

volatile unsigned int sector_addr = 0x000000;

unsigned char __attribute__((aligned (4))) src_data[300] ={0} ;//source bufer addr
unsigned char __attribute__((aligned (4))) dst_data[300] ={0};//destination bufer addr

/*
 * init data buf
 * input:
 * 1.DMA_DMACCHCTRL_REG *dmacctrl
 * 2.unsigned int buflen
 * 3.unsigned char inittype
 * 4.unsigned char initval
 * output:
 *
 */
void Init_Buf(unsigned char *u8buf,unsigned int buflen,unsigned char inittype,unsigned char initval)
{
	unsigned int i;

	if(inittype == 0)
	{
		memset(u8buf,initval,buflen);
	}
	else if(inittype == 1)
	{
		Init_Trng();
		for(i=0;i<buflen;i++)
		{
			u8buf[i] = (uint8_t)GetRandomWord();
		}
		Trng_Disable();
	}
	else
	{
		for(i=0;i<buflen;i++)
		{
			u8buf[i] = i;
		}
	}
}


/*
 * Note: SSI(QUAD spi eflash demo for Winbond w25q128)
 */
 
unsigned char buf[512];
unsigned char buf2[512]={0};

void SSI_Wait_Eflash_Idle(SSI_TypeDef *SSIx)
{
	 volatile unsigned char status;
 	 
	 do
	 {
		 status=SSI_EFlash_Get_Status1(SSIx);
		 //printf("status: 0x%02x\r\n", status);
	 }while(status&0x01);
}

void SSI_Wait_Eflash_QPI_Idle(SSI_TypeDef *SSIx)
{
	 volatile unsigned char status;
 	 
	 do
	 {
		 status=SSI_EFlash_QPI_Get_Status1(SSIx);
		 //printf("status: 0x%02x\r\n", status);
	 }while(status&0x01);
}	

/*******************************************************************************
* Function Name  : Read_ID_Test
* Description    : ȡSPIFLASH ID 
* Input          :- SSIx   : SSI ַ							 
* Output         : None
* Return         : None
******************************************************************************/
void Read_ID_Test(SSI_TypeDef *SSIx)
{
   unsigned short ID;
	
	 ID = Read_ID(SSIx);
	 printf("MID: 0x%02x, CID: 0x%02x\r\n", ID&0XFF,ID>>8);
}

void SSI_STD_DMA_Demo(SSI_TypeDef *SSIx)
{
	int i,j,ch=0;
		
	if(SSIx==SSI1)	
	{
	  DMA_REG_Init(DMA2_BASE_ADDR);

#ifdef  SSI_DMA_INT
//		NVIC_Init(3, 3, DMA2_IRQn, 2);
#endif
	}

	if(SSIx==SSI2)	
	{
		DMA_REG_Init(DMA1_BASE_ADDR);

#ifdef  SSI_DMA_INT 
//		NVIC_Init(3, 3, DMA1_IRQn, 2);
#endif
	}
	
	SSI_Standard_Init(SSIx);

	SSI_EFlash_Sector_Erase(SSIx, sector_addr);

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
    {
        Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00); 
        SSI_STD_DMA_Read(SSIx, ch, dst_data, sector_addr+i, SSI_PAGE_SIZE,0);       

        for(j = 0; j < SSI_PAGE_SIZE; j ++)
        {    
            if(dst_data[j+4] != 0xFF)
            {
                printf("Erase data compare failed!addr:%08x\r\n",(sector_addr+i+j));
                __ASM volatile("bkpt 0x12");
            }
        }
    }

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(src_data,SSI_PAGE_SIZE,1,0);
		SSI_EFlash_Write_Enable(SSIx);	
		SSI_STD_DMA_Send(SSIx, ch, src_data, sector_addr+i, SSI_PAGE_SIZE,0);
		SSI_Wait_Eflash_Idle(SSIx);//wait ssi finish
		printf("\r\n std DMA write ...\r\n");	
		for(j=0;j<SSI_PAGE_SIZE;j++)
		{
			printf("%02x ", *(src_data+j+4));
		}
		
		//DMA read  
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00); 	 
		SSI_STD_DMA_Read(SSIx, ch, dst_data, sector_addr+i, SSI_PAGE_SIZE,0);   //have 4Byte dummmy
		printf("\r\n STD DMA read ...\r\n");
		for(j=0;j<SSI_PAGE_SIZE;j++)
		{
			printf("%02x ", *(dst_data+j+4));
		}

		if(memcmp(src_data, dst_data+4, SSI_PAGE_SIZE)!=0x00)
		{
			printf("STD DMA data compare failed!addr:%08x\r\n",(sector_addr+i));
		 __ASM volatile("bkpt 0x12");
		} 
		else
		{
			printf("STD DMA passed!addr:%08x\r\n",(sector_addr+i));
		} 	 	 
	}
	
	printf("***********************SSI STD DMA test finish***********************\r\n");
}	

void SSI_STD_Demo(SSI_TypeDef *SSIx)
{
	unsigned int i,j;

	if(SSIx==SSI2)	
	{
		IOCTRL->SWAP_CONTROL_REG|=0x18;	//ʹSSI2
	}
	SSI_Standard_Init(SSIx);

	Read_ID_Test(SSIx);
	

	SSI_EFlash_Sector_Erase(SSIx, sector_addr);

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
		SSI_EFlash_Read(SSIx, sector_addr+i, dst_data, SSI_PAGE_SIZE);	
		for(j = 0; j < SSI_PAGE_SIZE; j ++)
		{
			if(dst_data[j] != 0xFF)
			{
				printf("Erase failed!addr:%08x\r\n",(sector_addr+i));
				__ASM volatile("bkpt 0x12");
			}
		}
		
		Init_Buf(src_data,SSI_PAGE_SIZE,1,0);	
		SSI_EFlash_Program(SSIx, sector_addr+i, src_data, SSI_PAGE_SIZE);
		printf("\r\nSSI STD write...\r\n");
		for(j=0;j<SSI_PAGE_SIZE;j++)
		{
			printf("%02x ", *(src_data+j+4));
		}
		
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
		SSI_EFlash_Read(SSIx, sector_addr+i, dst_data, SSI_PAGE_SIZE);	
		printf("SSI STD read...\r\n");
		for(j=0;j<SSI_PAGE_SIZE;j++)
		{
			printf("%02x ", *(dst_data+j+4));
		}
		
		if(memcmp(dst_data, src_data, SSI_PAGE_SIZE)!=0x00)
		{
			printf("STD failed!addr:%08x\r\n",(sector_addr+i));
		 __ASM volatile("bkpt 0x12");
		} 
		else
		{
			printf("STD passed!addr:%08x\r\n",(sector_addr+i));
		}
		
	}
		
	printf("***********************SSI STD test finish***********************\r\n");
	
}
	
void SSI_DUAL_DMA_Demo(SSI_TypeDef *SSIx)
{
	int i,j,ch=0;
	
	if(SSIx==SSI1)	
	{
	  DMA_REG_Init(DMA2_BASE_ADDR);

#ifdef  SSI_DMA_INT 
		NVIC_Init(3, 3, DMA2_IRQn, 2);
#endif
	}

	if(SSIx==SSI2)	
	{
		DMA_REG_Init(DMA1_BASE_ADDR);

#ifdef  SSI_DMA_INT 
		NVIC_Init(3, 3, DMA1_IRQn, 2);
#endif
	}
	
	SSI_Standard_Init(SSIx);

	SSI_EFlash_Sector_Erase(SSIx, sector_addr);

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00); 
		SSI_DUAL_Init(SSIx,SSI_PAGE_SIZE,8);
		SSI_DUAL_DMA_Read(SSIx, ch, dst_data, sector_addr+i, SSI_PAGE_SIZE,0);
		
		for(j = 0; j < SSI_PAGE_SIZE; j ++)
		{
			if(dst_data[j] != 0xFF)
			{
				printf("Erase data compare failed!addr:%08x\r\n",(sector_addr+i+j));
				__ASM volatile("bkpt 0x12");
			}
		}
	}
	
	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(src_data,SSI_PAGE_SIZE,1,0);
		SSI_Standard_Init(SSIx);  
		SSI_EFlash_Write_Enable(SSIx);	
		SSI_STD_DMA_Send(SSIx, ch, src_data, sector_addr+i, SSI_PAGE_SIZE,0);
		SSI_Wait_Eflash_Idle(SSIx);//wait ssi finish
//		printf("\r\n ssi DUAL write ...\r\n");
//		for(j=0;j<SSI_PAGE_SIZE;j++)
//		{
//			printf("%02x ", *(src_data+j));
//		}
		
		//DMA read  
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00); 
		SSI_DUAL_Init(SSIx,SSI_PAGE_SIZE,8);
		SSI_DUAL_DMA_Read(SSIx, ch, dst_data, sector_addr+i, SSI_PAGE_SIZE,0);	
//		printf("\r\n ssi DUAL DMA read ...\r\n");
//		for(j=0;j<SSI_PAGE_SIZE;j++)
//		{
//			printf("%02x ", *(dst_data+j));
//		}
//		
		if(memcmp(src_data, dst_data, SSI_PAGE_SIZE)!=0x00)
		{
			printf("DUAL DMA data compare failed!addr:%08x\r\n",(sector_addr+i));
		 __ASM volatile("bkpt 0x12");
		} 
		else
		{
			printf("DUAL DMA passed!addr:%08x\r\n",(sector_addr+i));
		} 	 	 
	}
	printf("***********************SSI DUAL DMA test finish***********************\r\n");
}	

void SSI_DUAL_Demo(SSI_TypeDef *SSIx)
{
	unsigned int i,j;

	if(SSIx==SSI2)	
	{
		IOCTRL->SWAP_CONTROL_REG|=0x18;	//ʹSSI2
	}
	
	SSI_Standard_Init(SSIx);

	SSI_EFlash_Sector_Erase(SSIx, sector_addr);

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
		SSI_EFlash_DUAL_Read(SSIx, sector_addr+i, dst_data, SSI_PAGE_SIZE);	
		for(j = 0; j < SSI_PAGE_SIZE; j ++)
		{
			if(dst_data[j] != 0xFF)
			{
				printf("Erase data cpmpare failed!addr:%08x\r\n",(sector_addr+i+j));
				__ASM volatile("bkpt 0x12");
			}
		}
	}
	
	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(src_data,SSI_PAGE_SIZE,1,0);
		SSI_Standard_Init(SSIx);
		SSI_EFlash_Program(SSIx, sector_addr+i, src_data, SSI_PAGE_SIZE);
//		printf("\r\n SSI DUAL write...\r\n");
//		for(j=0;j<SSI_PAGE_SIZE;j++)
//		{
//			printf("%02x ", *(src_data+j));
//		}
		
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
		SSI_EFlash_DUAL_Read(SSIx, sector_addr+i, dst_data, SSI_PAGE_SIZE);	
//		printf("\r\n SSI DUAL  read ...\r\n");
//		for(j=0;j<SSI_PAGE_SIZE;j++)
//		{
//			printf("%02x ", *(dst_data+j));
//		}
		
		if(memcmp(dst_data, src_data, SSI_PAGE_SIZE)!=0x00)
		{
			printf("DUAL data compare failed!addr:%08x\r\n",(sector_addr+i));
		 __ASM volatile("bkpt 0x12");
		} 
		else
		{
			printf("DUAL passed!addr:%08x\r\n",(sector_addr+i));
		}
	}
	
	printf("***********************SSI DUAL test finish***********************\r\n");
	
}

void SSI_QUAD_DMA_Demo(SSI_TypeDef *SSIx)
{
//	unsigned char temp;
	int i,j,ch=0;;
    uint16_t ret;
	
	if(SSIx==SSI1)	
	{
	  DMA_REG_Init(DMA2_BASE_ADDR);

#ifdef  SSI_DMA_INT 
		NVIC_Init(3, 3, DMA2_IRQn, 2);
#endif
	}

	if(SSIx==SSI2)	
	{
		DMA_REG_Init(DMA1_BASE_ADDR);

#ifdef  SSI_DMA_INT 
		NVIC_Init(3, 3, DMA1_IRQn, 2);
#endif
	}
	
	SSI_Standard_Init(SSIx);
	
//	SSI_EFlash_Prog_Status12(SSIx, 0x00,0x00);    //SET Status1 and Status2 = O
	SSI_EFlash_Prog_Status12(SSIx, 0x00,0x02);
	//ret=SSI_EFlash_Get_Status1(SSIx); printf("Status1=0x%x   \r\n",ret);
	//ret=SSI_EFlash_Get_Status2(SSIx); printf("Status2=0x%x   \r\n",ret);	
	SSI_EFlash_Sector_Erase(SSIx, sector_addr);

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
		SSI_QUAD_DMA_Read(SSIx, ch, dst_data, sector_addr+i, SSI_PAGE_SIZE, 0);	
		for(j = 0; j < SSI_PAGE_SIZE; j ++)
		{
			if(dst_data[j] != 0xFF)
			{
				printf("Erase data compare failed!addr:%08x\r\n",(sector_addr+i+j));
				__ASM volatile("bkpt 0x12");
			}
		}
	}

	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(src_data,SSI_PAGE_SIZE,1,0x00);
		SSI_EFlash_QUAD_Program(SSIx, sector_addr+i, src_data, SSI_PAGE_SIZE);
//		printf("\r\n QUAD SPI DMA write ...\r\n"); 
//		for(j=0;j<SSI_PAGE_SIZE;j++)
//		{
//			printf("%02x ", *(src_data+j));
//		}   
		
		//quad spi dma read
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
		SSI_QUAD_DMA_Read(SSIx, ch, dst_data, sector_addr+i, SSI_PAGE_SIZE, 0);	
//		printf("\r\n QUAD SPI DMA read ...\r\n");
//		for(j=0;j<SSI_PAGE_SIZE;j++)
//		{
//			printf("%02x ", *(dst_data+j));
//		}

		if(memcmp(src_data, dst_data, SSI_PAGE_SIZE)!=0x00)
		{
			printf("QUAD DMA data compare failed!addr:%08x\r\n",(sector_addr+i));
//			__ASM volatile("bkpt 0x12");
		}	
		else
		{
			printf("QUAD DMA passed!addr:%08x\r\n",(sector_addr+i));
		}
	}
	printf("******************SSI QUAD DMA test finish******************\r\n\r\n");
}

void SSI_QUAD_Demo(SSI_TypeDef *SSIx)
{
    uint16_t ret;
	int i,j;
//	unsigned char temp;
	
	SSI_Standard_Init(SSIx);
//	SSI_EFlash_Prog_Status12(SSIx, 0x00,0x00);    //SET Status1 and Status2 = O
	SSI_EFlash_Prog_Status12(SSIx, 0x00,0x02);
	//ret=SSI_EFlash_Get_Status1(SSIx); printf("Status1=0x%x   \r\n",ret);
	//ret=SSI_EFlash_Get_Status2(SSIx); printf("Status2=0x%x   \r\n",ret);
    SSI_EFlash_Sector_Erase(SSIx, sector_addr);
	
    for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
    {
        Init_Buf(dst_data,SSI_PAGE_SIZE,0,0x00);
        SSI_EFlash_QUAD_Read(SSIx, sector_addr+i, dst_data, SSI_PAGE_SIZE);
		for(j = 0; j < SSI_PAGE_SIZE; j ++)
		{
			if(dst_data[j] != 0xFF)
			{
				printf("Erase data compeare failed!addr:%08x\r\n",(sector_addr+i+j));
				__ASM volatile("bkpt 0x12");
			}
		}
    }
	
	for(i = 0; i < SSI_SECTOR_SIZE; i += SSI_PAGE_SIZE)
	{
		Init_Buf(src_data,SSI_PAGE_SIZE,1,0);
		SSI_EFlash_QUAD_Program(SSIx, sector_addr+i, src_data, SSI_PAGE_SIZE);
//		printf("\r\n quad program:\r\n");
//		for(j=0; j<SSI_PAGE_SIZE; j++)
//		{
//			 printf("%02x ", src_data[j]);
//		}
		
		Init_Buf(dst_data,SSI_PAGE_SIZE,0,0);
		SSI_EFlash_QUAD_Read(SSIx, sector_addr+i, dst_data, SSI_PAGE_SIZE);
//		printf("\r\n quad read:\r\n");
//		for(j=0; j<SSI_PAGE_SIZE; j++)
//		{
//			 printf("%02x ", dst_data[j]);
//		}
	
		if(memcmp(src_data, dst_data, SSI_PAGE_SIZE)!=0x00)
		{
			printf("QUAD test data compare failed!addr:%08x\r\n",(sector_addr+i));
//			__ASM volatile("bkpt 0x12");
		}
		else
		{
			printf("QUAD test passed!addr:%08x\r\n",(sector_addr+i));
		}
	}		
	
	printf("******************SSI QUAD test finish******************\r\n\r\n");
}

void SSI_Demo(void)
{
	uint32_t times = 0,i=0;
	SSI_TypeDef *ssi;
	ssi = SSI1;
	
	while(times < 10000)
	{
		for(i = 0; i < 0x1000; i ++)
		{
			sector_addr = i*SSI_SECTOR_SIZE;

			//STD mode
			SSI_STD_Demo(ssi);
			SSI_STD_DMA_Demo(ssi);
	
			//DUAL mode
			SSI_DUAL_Demo(ssi);
			SSI_DUAL_DMA_Demo(ssi);
	
			//QUAD mode
			SSI_QUAD_Demo(ssi);
			SSI_QUAD_DMA_Demo(ssi);

			printf("times:%05d\r\n",times);
			printf("sector_addr:%08x\r\n",sector_addr);
		}
		times ++;
		sector_addr = 0;
	}
}	
