/*
 * alg_demo.c
 *
 *  Created on: 201769
 *      Author: 
 */

#include "cpm_drv.h"
#include "alg_drv.h"
#include "string.h"
#include "debug.h"
#include "trng_drv.h"

const uint8_t charGx[]           = "32C4AE2C1F1981195F9904466A39C9948FE30BBFF2660BE1715A4589334C74C7";
const uint8_t charGy[]           = "BC3736A2F4F6779C59BDCEE36B692153D0A9877CC62A474002DF32E52139F0A0";
const uint8_t charPrimeN[]       = "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123";
const uint8_t charPrimeP[]       = "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFF";
const uint8_t charCoefficientA[] = "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFC";
const uint8_t charCoefficientB[] = "28E9FA9E9D9F5E344D5A9E4BCF6509A7F39789F515AB8F92DDBCBD414D940E93";

//SM2
SM2_STU_PRAMS    stuEccPrams;
SM2_STU_PUBKEY   stuPubKey;
ECC_STU_BIGINT32 stuPrivKey;
ECC_STU_BIGINT32 stuSignatureR;
ECC_STU_BIGINT32 stuSignatureS;

//rsa
STU_RSA_PUBKEY  publicKeyRSA;
STU_RSA_PRIVKEY privateKeyRSA;

/*******************************************************************************
* Function Name  : IsSRAMAddr
* Description    : ΪSRAMַ
* Input          : addr  ĵַ
* Output         : None
* Return         : TRUE or FLASE
******************************************************************************/
static bool IsSRAMAddr(uint32_t addr)
{
	if((addr < SRAM_BEGIN)||(addr > SRAM_END))
	{
		return FALSE;
	}
	return TRUE;
}

/*******************************************************************************
* Function Name  : IsEFLASHAddr
* Description    : ΪEFLASHַ
* Input          : addr  ĵַ
* Output         : None
* Return         : TRUE or FLASE
******************************************************************************/
static bool IsEFLASHAddr(uint32_t addr)
{
	if((addr < EFLASH_BEGIN)||(addr > EFLASH_END))
	{
		return FALSE;
	}
	return TRUE;
}

/*******************************************************************************
* Function Name  : CheckAddrType
* Description    : ַ
* Input          : addr  ĵַ
* Output         : None
* Return         : int     0ַЧ    1SRAMַ   2EFLASHַ
******************************************************************************/
static TYPE_ADDR CheckAddrType(uint32_t addr)
{
	if(IsSRAMAddr(addr) == TRUE)
	{
		return TYPE_SRAM;
	}

	if(IsEFLASHAddr(addr) == TRUE)
	{
		return TYPE_EFLASH;
	}
	return TYPE_ERROR;
}

uint32_t Get_Random_Word(void)
{
	return GetRandomWord();
}



/*******************************************************************************
* Function Name  : U32toU8
* Description    : ͣСˣתΪBYTEˣ
* Input          : -srcԴݵַ
*                  -lenֳ
*                  -dst: Ŀĵַ
* Output         : None
* Return         : uint16_t byteֽڳ
******************************************************************************/
static uint16_t U32toU8(uint32_t *src_p,uint16_t len,uint8_t *dst_p,uint8_t append)
{
	uint16_t j;
	uint16_t block;
	uint8_t *p = (uint8_t*)src_p;
	uint8_t temp[256];

	if(!len) return 0;

	//16 ֽڿ
	block = len/4;

	for(j = 0x0;j < block;j++)
	{
		temp[j*4]   = p[len-1-j*4];
		temp[j*4+1] = p[len-1-j*4-1];
		temp[j*4+2] = p[len-1-j*4-2];
		temp[j*4+3] = p[len-1-j*4-3];
	}
	memcpy(dst_p,temp+append,block*4-append);

	return block*4-append;
}

/*******************************************************************************
* Function Name  : U8toU32
* Description    : BYTEˣתΪͣСˣ
* Input          : -srcԴݵַ
*                  -lenֳ
*                  -dst: Ŀĵַ
* Output         : None
* Return         : uint16_t ֵĳ
******************************************************************************/
static uint16_t U8toU32(uint8_t *src_p,uint16_t len,uint32_t *dst_p)
{
	uint16_t end_index,i,shift;
	uint32_t temp;

	if(len < 4) return 0;
	//ĩβλ
	end_index = len/4;

	temp = 0;
	shift = 0;
	while(len%4)
	{
		temp = (temp << 8) & 0xFFFFFF00;
		temp |= src_p[shift];
		len--;
		shift++;
	}

	if(shift != 0)
	{
		dst_p[end_index] = temp;
		//end_index++;
	}

	for(i = 0; i < end_index; i++)
	{
		temp = (src_p[shift+i*4]<<24)|(src_p[shift+i*4+1]<<16)|(src_p[shift+i*4+2]<<8)|src_p[shift+i*4+3];
		dst_p[end_index-1-i] = temp;
	}
	return end_index;
}

/*******************************************************************************
* Function Name  : wordswap
* Description    : Ϊλ巭תڲ
* Input          : -destĿĵַ
*                  -srcԴַ
*                  -wordLen: ֳ
* Output         : None
* Return         : None
******************************************************************************/
static void wordswap(uint32_t *dest, uint32_t *src, uint32_t wordLen)
{
	uint32_t i;
	uint32_t temp;

	//ʹͬһBUF
	if (src == dest)
	{
		for (i=0; i<wordLen/2; i++)
		{
			temp = dest[i];
			dest[i] = src[wordLen-1-i];
			dest[wordLen-1-i] = temp;
		}
	}
	else
	{
		for (i=0; i<wordLen; i++)
		{
			dest[i] = src[wordLen-1-i];
		}
	}
}

/*******************************************************************************
* Function Name  : AES_EnDecrypt_2
* Description    : AESӽ
* 				         ע⣺
* 				    None
* Input          : -EncDe_Mode      ӽģʽ
* 				   -CBC_ECB_MODE    ECB  CBCģʽ
* 				   -key             :Կ
* 				   -cbc_iv          :CBCʼ
* 				   -key_mode        Կģʽ
* 				   -src_buf         :Դbuf
* 				   -data_len        :Դݳ
* Output         : -out_buf         :buf
*
* Return         : 
******************************************************************************/
int AES_EnDecrypt_2(uint8_t EncDe_Mode,uint8_t CBC_ECB_MODE,uint8_t *key,uint8_t *cbc_iv,uint32_t key_mode, uint8_t *src_buf, uint8_t *out_buf, uint32_t data_len)
{
	uint32_t key_buf[8];
	uint32_t cbc_iv_buf[4];

	if((key==NULL)  ||(src_buf==NULL) ||(out_buf==NULL))
	{
		return ERROR_DATA_BUF;
	}
	
	if((CheckAddrType((uint32_t)key) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)src_buf) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)out_buf) != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if ((0x0 == data_len)||((data_len%16) != 0))	//ݳȴ
	{
		return ERROR_DATA_LENGTH;
	}
	if ((EncDe_Mode!=0) && (EncDe_Mode!=1))
	{
		return ERROR_ENDE_MODE;
	}
	if ((CBC_ECB_MODE!=0) && (CBC_ECB_MODE!=1))
	{
		return ERROR_ECBCBC_MODE;
	}
	if ((key_mode!=0) && (key_mode!=1) && (key_mode!=2))
	{
		return ERROR_KEY_MODE;
	}

	memcpy(key_buf,key,16 + (key_mode<<3));
	if(CBC_ECB_MODE)
	{
		if(NULL == cbc_iv)
		{
			return ERROR_DATA_BUF;
		}
		if(CheckAddrType((uint32_t)cbc_iv) == TYPE_ERROR)
		{
			return ERROR_DATA_BUF;
		}
		memcpy(cbc_iv_buf,cbc_iv,16);
	}

	AES_Enable();
	AES_EnDecrypt(EncDe_Mode,CBC_ECB_MODE,key_buf,cbc_iv_buf,key_mode,(uint32_t *)src_buf,(uint32_t *)out_buf,data_len);
	AES_Disable();

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : DES_EnDecrypt_2
* Description    : DESӽ
* 				         ע⣺
* 				    None
* Input          : -EncDe_Mode      ӽģʽ
* 				   -CBC_ECB_MODE    ECB  CBCģʽ
* 				   -key             :Կ
* 				   -key_len         :Կ
* 				   -iv              ʼ
* 				   -src_buf         :Դbuf
* 				   -data_len        :Դݳ
* 				   -check_parity    :keyǷҪУ飬У򷵻keyУټӽ
* Output         : -out_buf         :buf
*
* Return         : 
******************************************************************************/
int DES_EnDecrypt_2(uint8_t EncDe_Mode, uint8_t ECB_CBC_Mode, uint8_t *key, uint32_t key_len, uint8_t *iv, uint8_t *src_buf, uint8_t *out_buf,uint32_t data_len, uint8_t check_parity)
{
	uint32_t key_buf[6];
	uint32_t iv_buf[2];

	if((key==NULL)  ||(src_buf==NULL) ||(out_buf==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)key) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)src_buf) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)out_buf) != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if ((0x0 == data_len)||((data_len%8) != 0))	//ݳȴ
	{
		return ERROR_DATA_LENGTH;
	}
	if ((EncDe_Mode!=0) && (EncDe_Mode!=1))
	{
		return ERROR_ENDE_MODE;
	}
	if ((ECB_CBC_Mode!=0) && (ECB_CBC_Mode!=1))
	{
		return ERROR_ECBCBC_MODE;
	}
	//жԿ
	if((0x08 != key_len)&&(0x10 != key_len)&&(0x18 != key_len))
	{
		return ERROR_KEY_MODE;
	}
	if (check_parity != 0)
	{
		return ERROR_OTHER_PARA;
	}
	if(ECB_CBC_Mode)
	{
		if(NULL == iv)
		{
			return ERROR_DATA_BUF;
		}
		if(CheckAddrType((uint32_t)iv) == TYPE_ERROR)
		{
			return ERROR_DATA_BUF;
		}
		memcpy(iv_buf,iv,8);
	}
	memcpy(key_buf,key,key_len);

	DES_Enable();
	DES_EnDecrypt(EncDe_Mode,ECB_CBC_Mode,key_buf,key_len,iv_buf,(uint32_t *)src_buf,(uint32_t *)out_buf,data_len,check_parity);
	DES_Disable();

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SMS4_EnDecrypt_2
* Description    : SMS4ӽ
* 				         ע⣺
* 				    None
* Input          : -EncDe_Mode      ӽģʽ
* 				   -CBC_ECB_MODE    ECB  CBCģʽ
* 				   -key             :Կ
* 				   -iv              ʼ
* 				   -src_buf         :Դbuf
* 				   -data_len        :ݳ
* Output         : -out_buf         :buf
*
* Return         : 
******************************************************************************/
int SMS4_EnDecrypt_2(uint8_t EncDe_Mode,uint8_t ECB_CBC_Mode,uint8_t *key,uint8_t *iv,uint8_t *src_buf,uint8_t *out_buf,uint32_t data_len)
{
	uint32_t key_buf[4];
	uint32_t iv_buf[4];

	if((key==NULL)  ||(src_buf==NULL) ||(out_buf==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)key)     == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)src_buf) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)out_buf) != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if ((0x0 == data_len)||((data_len%16) != 0))	//ݳȴ
	{
		return ERROR_DATA_LENGTH;
	}
	if ((EncDe_Mode!=0) && (EncDe_Mode!=1))
	{
		return ERROR_ENDE_MODE;
	}
	if ((ECB_CBC_Mode!=0) && (ECB_CBC_Mode!=1))
	{
		return ERROR_ECBCBC_MODE;
	}

	if(ECB_CBC_Mode)
	{
		if(NULL == iv)
		{
			return ERROR_DATA_BUF;
		}
		if((CheckAddrType((uint32_t)iv) == TYPE_ERROR))
		{
			return ERROR_DATA_BUF;
		}
		memcpy(iv_buf,iv,16);
	}
	memcpy(key_buf,key,16);

	SMS4_Enable();
	SMS4_EnDecrypt(EncDe_Mode,ECB_CBC_Mode,key_buf,iv_buf,(uint32_t *)src_buf,(uint32_t *)out_buf,data_len);
	SMS4_Disable();

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SHA_Init_2
* Description    : HASHʼ
* 				         ע⣺
* 				    None
* Input          : -phash_tx        ϣ㷨ݽṹģʽ
* 				   -HashModeNum    ϣ㷨ţȡֵΧ0~4
* Output         : None
* Return         : 
******************************************************************************/
int SHA_Init_2(Hash_tx *phash_tx, unsigned char HashModeNum)
{
	if(phash_tx==NULL)
	{
		return ERROR_DATA_BUF;
	}
	if (HashModeNum>HASH_SHA512)
	{
		return ERROR_OTHER_PARA;
	}

	SHA_Init(phash_tx, HashModeNum);
	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SHA_Update_2
* Description    : ҪHASH
* 				         ע⣺
* 				    None
* Input          : -phash_tx         ϣ㷨ݽṹģʽ
* 				   -pMessageBlock    Ϣ
* 				   -DataLen          Ϣ鳤
* Output         : None
* Return         : ִн
******************************************************************************/
int SHA_Update_2(Hash_tx *phash_tx, unsigned char *pMessageBlock, unsigned int DataLen)
{
	uint8_t MessageBlock[64];
	uint32_t offset;
	uint8_t len;
	if((phash_tx==NULL) || (pMessageBlock==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if (DataLen == 0)
	{
		return SUCCESS_ALG;
	}
	offset = 0;
	while(offset < DataLen)
	{
		len = (DataLen-offset)> 0x40?0x40:(DataLen-offset);
		memcpy(MessageBlock,pMessageBlock+offset,len);

		SHA_Update(phash_tx, MessageBlock, len);
		offset += len;
	}
	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SHA_Final_2
* Description    :  HASHȡ
* 				         ע⣺
* 				    None
* Input          : -phash_tx         ϣ㷨ݽṹģʽ
* Output         : -pRes             HASH
* Return         : ִн
******************************************************************************/
int SHA_Final_2(Hash_tx *phash_tx, unsigned char *pRes)
{
	uint32_t sha_result[16];
	uint32_t res_Len;

	if((phash_tx==NULL) || (pRes==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if(CheckAddrType((uint32_t)pRes) != TYPE_SRAM)
	{
		return ERROR_DATA_BUF;
	}

	SHA_Final(phash_tx, sha_result);

	//Ҫصݳ
	switch(phash_tx->HASHMode)
	{
		case HASH_SM3:
			res_Len = SM3_R_Len;
			break;
		case HASH_SHA0:
			res_Len = SHA0_R_Len;
			break;
		case HASH_SHA1:
			res_Len  = SHA1_R_Len;
			break;
		case HASH_SHA224:
			res_Len = SHA224_R_Len;
			break;
		case HASH_SHA256:
			res_Len = SHA256_R_Len;
			break;
		case HASH_SHA384:
			res_Len = SHA384_R_Len;
			break;
		case HASH_SHA512:
			res_Len = SHA512_R_Len;
			break;
		default:
			return ERROR_OTHER_PARA;
	}

	memcpy(pRes, (uint8_t*)sha_result, res_Len);
	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SM2Init
* Description    : SM2ģʼ
* Input          : testbits  SM2λ
* Output         : None
* Return         : None
******************************************************************************/
int SM2Init(uint16_t testbits)
{
	uint8_t databuf[64];

	if (testbits != 256)
	{
		return ERROR_OTHER_PARA;
	}

	stuEccPrams.uBits = testbits;

	//ʼECC
	EccBig32Initial(&(stuEccPrams.stuPrimeP));
	EccBig32Initial(&(stuEccPrams.stuCoefficientA));
	EccBig32Initial(&(stuEccPrams.stuCoefficientB));
	EccBig32Initial(&(stuEccPrams.stuGx));
	EccBig32Initial(&(stuEccPrams.stuGy));
	EccBig32Initial(&(stuEccPrams.stuPrimeN));

	EccBig32Initial(&(stuPrivKey));
	EccBig32Initial(&(stuPubKey.stuQx));
	EccBig32Initial(&(stuPubKey.stuQy));
	EccBig32Initial(&stuSignatureR);
	EccBig32Initial(&stuSignatureS);

	EccCharToHex((uint8_t *)charGx,databuf,stuEccPrams.uBits/4);
	EccByBufToBigInt32(databuf,stuEccPrams.uBits/8,&(stuEccPrams.stuGx));

	EccCharToHex((uint8_t *)charGy,databuf,stuEccPrams.uBits/4);
	EccByBufToBigInt32(databuf,stuEccPrams.uBits/8,&(stuEccPrams.stuGy));

	EccCharToHex((uint8_t *)charPrimeN,databuf,stuEccPrams.uBits/4);
	EccByBufToBigInt32(databuf,stuEccPrams.uBits/8,&(stuEccPrams.stuPrimeN));

	EccCharToHex((uint8_t *)charPrimeP,databuf,stuEccPrams.uBits/4);
	EccByBufToBigInt32(databuf,stuEccPrams.uBits/8,&(stuEccPrams.stuPrimeP));

	EccCharToHex((uint8_t *)charCoefficientA,databuf,stuEccPrams.uBits/4);
	EccByBufToBigInt32(databuf,stuEccPrams.uBits/8,&(stuEccPrams.stuCoefficientA));

	EccCharToHex((uint8_t *)charCoefficientB,databuf,stuEccPrams.uBits/4);
	EccByBufToBigInt32(databuf,stuEccPrams.uBits/8,&(stuEccPrams.stuCoefficientB));

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SM2GenerateKeyPair_2
* Description    : SM2Կ
* Input          : None
* Output         : -pub_key         Կ
*                  -pri_key         ˽Կ
* Return         : 
******************************************************************************/
int SM2GenerateKeyPair_2(uint8_t *pub_key,uint8_t *pri_key)
{
	char flag;

	if((pub_key==NULL)  ||(pri_key==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)pub_key) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)pri_key) != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	SM2_Enable();
	flag  =  SM2GenerateKeyPair(&stuEccPrams,&stuPrivKey,&stuPubKey);
	SM2_Disable();

	//cpy Կ
	U32toU8(stuPrivKey.auValue,8*4,pri_key,0);
	U32toU8(stuPubKey.stuQx.auValue,8*4,pub_key,0);
	U32toU8(stuPubKey.stuQy.auValue,8*4,pub_key+32,0);

	//Կɽ
	return flag;
}

/*******************************************************************************
* Function Name  : SM2Encrypt_2
* Description    : SM2
* Input          : -mes             
*                  -len             :ĳ
*                  -pub_key         :Կ
* Output         : -cipher          
* Return         : 
******************************************************************************/
int SM2Encrypt_2(uint8_t *mes,uint16_t len,uint8_t *pub_key,uint8_t *cipher)
{
	if((mes==NULL)  ||(pub_key==NULL) ||(cipher==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)pub_key) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)mes)     != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)cipher)  != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}


	if(len > 256*4)		//ĳȲ1Kֽ
	{
		return ERROR_DATA_LENGTH;
	}

	stuPubKey.stuQx.uLen = 8;
	stuPubKey.stuQy.uLen = 8;

	U8toU32(pub_key,32,stuPubKey.stuQx.auValue);
	U8toU32(pub_key+32,32,stuPubKey.stuQy.auValue);

	SM2_Enable();
	SM2Encrypt_V2(mes,len,&stuPubKey,&stuEccPrams,cipher);
	SM2_Disable();

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SM2Decrypt_2
* Description    : SM2
* Input          : -cipher          
*                  -len             :ĳ
*                  -pri_key         :˽Կ
* Output         : -mes             
* Return         : 
******************************************************************************/
int SM2Decrypt_2(uint8_t *cipher,uint16_t len,uint8_t *pri_key,uint8_t *mes)
{
	uint32_t cipher_buf[256+24];	//1024ֽ+96ֽC1/C3
	uint8_t result;

	if((mes==NULL)  ||(pri_key==NULL) ||(cipher==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)pri_key) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)cipher)  == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)mes)     != TYPE_SRAM))
	{
		printf("faile\r\n");
		return ERROR_DATA_BUF;
	}

	if(len > 256*4)		//ĳȲ1Kֽ
	{
		return ERROR_DATA_LENGTH;
	}

	memcpy(cipher_buf,cipher,len+96);

	stuPrivKey.uLen = 8;
	U8toU32(pri_key,32,stuPrivKey.auValue);

	SM2_Enable();
	result = SM2Decrypt_V2((uint8_t*)cipher_buf,len,&stuPrivKey,&stuEccPrams,mes);
	SM2_Disable();

	return result;
}

/*******************************************************************************
* Function Name  : SM2Signature_2
* Description    : SM2ǩ
* Input          : -mes             
*                  -len             :ĳ
*                  -pri_key         :˽Կ
* Output         : -r               ǩֵR
*                  -s               :ǩֵS
* Return         : 
******************************************************************************/
int SM2Signature_2(uint8_t *mes,uint16_t len,uint8_t *pri_key,uint8_t *r,uint8_t *s)
{
	uint32_t signplain[8];
	//uint8_t  key[32];

	if((mes==NULL)  ||(pri_key==NULL) ||(r==NULL) ||(s==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)mes)    == TYPE_ERROR)
	||(CheckAddrType((uint32_t)pri_key) == TYPE_ERROR)
	||(CheckAddrType((uint32_t)r)       != TYPE_SRAM)
	||(CheckAddrType((uint32_t)s)       != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if(len != 32)
	{
		return ERROR_DATA_LENGTH;
	}

	memcpy((uint8_t*)signplain, mes, 32);
	wordswap(signplain,signplain,8);

	stuPrivKey.uLen = 8;
	U8toU32(pri_key,32,stuPrivKey.auValue);

	EccBig32Initial(&stuSignatureR);
	EccBig32Initial(&stuSignatureS);

	SM2_Enable();
	SM2Signature((uint8_t*)signplain,len,&stuEccPrams,&stuPrivKey,&stuSignatureR,&stuSignatureS);
	SM2_Disable();

	U32toU8(stuSignatureR.auValue,8*4,r,0);
	U32toU8(stuSignatureS.auValue,8*4,s,0);

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SM2Verification_2
* Description    : SM2ǩ
* Input          : -mes             
*                  -len             :ĳ
*                  -pri_key         :˽Կ
*                  -r               ǩֵR
*                  -s               :ǩֵS
* Output         : None
* Return         : ǩ
******************************************************************************/
int SM2Verification_2(uint8_t *mes,uint16_t len,uint8_t *pub_key,uint8_t *r,uint8_t *s)
{
	uint8_t flag;
	uint32_t signplain[8];

	if((mes==NULL)  ||(pub_key==NULL) ||(r==NULL) ||(s==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)mes)     == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)pub_key) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)r)       != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)s)       != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if(len != 32)
	{
		return ERROR_DATA_LENGTH;
	}

	memcpy((uint8_t*)signplain, mes, 32);
	wordswap(signplain,signplain,8);

	stuPubKey.stuQx.uLen = 8;
	stuPubKey.stuQy.uLen = 8;

	U8toU32(pub_key,32,stuPubKey.stuQx.auValue);
	U8toU32(pub_key+32,32,stuPubKey.stuQy.auValue);

	stuSignatureR.uLen = 8;
	stuSignatureS.uLen = 8;

	U8toU32(r,32,stuSignatureR.auValue);
	U8toU32(s,32,stuSignatureS.auValue);

	SM2_Enable();
	flag = SM2Verification((uint8_t*)signplain,len,&stuEccPrams,&stuPubKey,&stuSignatureR,&stuSignatureS);
	SM2_Disable();

	//0 : fail
	return flag;
}

/*******************************************************************************
* Function Name  : SM2GenerateE_2
* Description    : SM2Eֵ
* Input          : -IDA             ûID
*                  -ida_len         :ûID
*                  -pub_key         :Կ
*                  -pMessage        Ϣ
*                  -mesLen          :Ϣ
* Output         : -res_E           :Eֵ
* Return         : ִн
******************************************************************************/
int SM2GenerateE_2(uint8_t *IDA,uint16_t ida_len,uint8_t *pub_key,uint8_t *pMessage,uint32_t mesLen,uint8_t *res_E)
{
	uint32_t za_buf[8];
	uint32_t i,block,bytelen;
	uint8_t oneblock=64;
	Hash_tx stru_sm3;

	if((IDA==NULL)  ||(pub_key==NULL) ||(pMessage==NULL) ||(res_E==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)IDA)     != TYPE_SRAM)
	||(CheckAddrType((uint32_t)res_E)    != TYPE_SRAM)
	||(CheckAddrType((uint32_t)pub_key)  == TYPE_ERROR)
	||(CheckAddrType((uint32_t)pMessage) == TYPE_ERROR))
	{
		return ERROR_DATA_BUF;
	}

	if (mesLen==0)
	{
		return ERROR_DATA_LENGTH;
	}

	//Կ
	stuPubKey.stuQx.uLen = 8;
	stuPubKey.stuQy.uLen = 8;

	U8toU32(pub_key,32,stuPubKey.stuQx.auValue);
	U8toU32(pub_key+32,32,stuPubKey.stuQy.auValue);

	SM2_Enable();
	//zaֵ
	sign(IDA,ida_len,&stuPubKey,&stuEccPrams,za_buf);
	SM2_Disable();
	
	ModuleClk_On(MODULE_CLK_SHA);
	
	SHA_Init_2(&stru_sm3, HASH_SM3);
	SHA_Update_2(&stru_sm3, (uint8_t*)za_buf, 32);		//ȶZֵSM3

	block = mesLen/oneblock;
	bytelen = mesLen%oneblock;
	for(i = 0 ; i < block ; i++)		//ٶSM3
	{
		SHA_Update_2(&stru_sm3, pMessage + i*oneblock, oneblock);
	}
	if (bytelen!=0)
	{
		SHA_Update_2(&stru_sm3, pMessage + i*oneblock, bytelen);
	}

	SHA_Final_2(&stru_sm3, res_E);
	
	ModuleClk_Off(MODULE_CLK_SHA);
	
	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : SM2KeyExchange_2
* Description    : SM2Կ
* Input          : -self_pri        ˽Կ
*                  -self_temp_pub   :ʱԿ
*                  -self_temp_pri   :ʱ˽Կ
*                  -other_pub       ԷԿ
*                  -other_temp_pub  :ԷʱԿ
*                  -za              Zֵ
*                  -zb              :ԷZֵ
*                  -AgreedKeyLen    ЭԿĳ
*                  -Mode			Կ𷽡0ʾѷΪA1ʾѷΪB
* Output         : -agree_key       :ЭԿ
*                  -sA              Sֵ
*                  -sB              :ԷSֵ
* Return         : 
******************************************************************************/
int SM2KeyExchange_2(uint8_t*self_pri,uint8_t*self_temp_pub,uint8_t *self_temp_pri,uint8_t*other_pub,uint8_t*other_temp_pub,uint8_t*za,uint8_t*zb,uint8_t*agree_key,uint32_t AgreedKeyLen,uint8_t*sA,uint8_t*sB,uint32_t Mode)
{
	SM2_STU_PUBKEY   self_temp_pub_key;
	ECC_STU_BIGINT32 self_pri_key;
	ECC_STU_BIGINT32 self_temp_pri_key;
	SM2_STU_PUBKEY   other_pub_key;
	SM2_STU_PUBKEY   other_temp_pub_key;

	uint32_t za_buf[8];
	uint32_t zb_buf[8];
	uint32_t sa_buf[8];
	uint32_t sb_buf[8];
	uint32_t key_buf[8];
	uint8_t result;


	if((NULL == self_pri) ||(NULL == self_temp_pub) ||(NULL == self_temp_pri)
	 ||(NULL == other_pub)||(NULL == other_temp_pub)||(NULL == za)||(NULL == zb)
	 ||(NULL == agree_key)||(NULL == sA)||(NULL == sB))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)sA)     != TYPE_SRAM)
	||(CheckAddrType((uint32_t)sB)      != TYPE_SRAM)
	||(CheckAddrType((uint32_t)self_pri)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)self_temp_pub)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)self_temp_pri)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)other_pub)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)other_temp_pub)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)za)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)zb)== TYPE_ERROR)
	||(CheckAddrType((uint32_t)agree_key)!= TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if ((Mode > 1)||(AgreedKeyLen > 32))
	{
		return ERROR_DATA_LENGTH;
	}

 	EccBig32Initial(&self_temp_pub_key.stuQx);
 	EccBig32Initial(&self_temp_pub_key.stuQy);
 	EccBig32Initial(&other_pub_key.stuQx);
 	EccBig32Initial(&other_pub_key.stuQy);
 	EccBig32Initial(&other_temp_pub_key.stuQx);
 	EccBig32Initial(&other_temp_pub_key.stuQy);
 	EccBig32Initial(&self_pri_key);
 	EccBig32Initial(&self_temp_pri_key);


	//Կת
	U8toU32(self_temp_pub,32,self_temp_pub_key.stuQx.auValue);
	U8toU32(self_temp_pub+32,32,self_temp_pub_key.stuQy.auValue);
	self_temp_pub_key.stuQx.uLen = 8;
	self_temp_pub_key.stuQy.uLen = 8;

	U8toU32(other_pub,32,other_pub_key.stuQx.auValue);
	U8toU32(other_pub+32,32,other_pub_key.stuQy.auValue);
	other_pub_key.stuQx.uLen = 8;
	other_pub_key.stuQy.uLen = 8;

	U8toU32(other_temp_pub,32,other_temp_pub_key.stuQx.auValue);
	U8toU32(other_temp_pub+32,32,other_temp_pub_key.stuQy.auValue);
	other_temp_pub_key.stuQx.uLen = 8;
	other_temp_pub_key.stuQy.uLen = 8;


	//˽Կת
	U8toU32(self_pri,32,self_pri_key.auValue);
	self_pri_key.uLen = 8;

	U8toU32(self_temp_pri,32,self_temp_pri_key.auValue);
	self_temp_pri_key.uLen = 8;

	memcpy(za_buf,za,32);
	memcpy(zb_buf,zb,32);

	SM2_Enable();
	result = SM2KeyExchange(&stuEccPrams,
							&self_pri_key,
							&self_temp_pub_key,
							&self_temp_pri_key,
							&other_pub_key,
							&other_temp_pub_key,
							za_buf,
							zb_buf,
							key_buf,
							AgreedKeyLen,
							sa_buf,
							sb_buf,
							Mode);
	SM2_Disable();
	if(!result) return FAIL_ALG;

	memcpy(sA,sa_buf,32);
	memcpy(sB,sb_buf,32);
	memcpy(agree_key,key_buf,AgreedKeyLen);

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : sign_2
* Description    : SM2Zֵ
* Input          : -IDA             ûID
*                  -entla           :ûID
*                  -pub_key         :Կ
* Output         : -za              Zֵ
* Return         : ִн
******************************************************************************/
int sign_2(uint8_t *IDA, uint16_t entla, uint8_t *pub_key,uint8_t*za)
{
	uint32_t za_buf[8];
	SM2_STU_PUBKEY pub_key_buf;

	if((IDA==NULL)  ||(pub_key==NULL) ||(za==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)IDA)     != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)pub_key) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)za)     != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if (entla==0)
	{
		return ERROR_DATA_LENGTH;
	}
	//ԿҪдСת
	U8toU32(pub_key,32,pub_key_buf.stuQx.auValue);
	U8toU32(pub_key+32,32,pub_key_buf.stuQy.auValue);
	pub_key_buf.stuQx.uLen = 8;
	pub_key_buf.stuQy.uLen = 8;
	//ǩ
	SM2_Enable();
	sign(IDA,entla,&pub_key_buf,&stuEccPrams,za_buf);
	SM2_Disable();
	
	memcpy(za,za_buf,32);

	return SUCCESS_ALG;
}

/*******************************************************************************
* Function Name  : rsa_keypair_gen
* Description    : RSAԿ
* Input          : -rsa_bit_len     RSA㷨λ
*                  -fixkey          :EֵǷ̶
* Output         : -n_addr          N
*                  -e_addr          :E
*                  -d_addr          :D
*                  -p_addr          :P
*                  -q_addr          :Q
*                  -dp_addr         :dP
*                  -dq_addr         :dQ
*                  -qinv_addr       :PQ
* Return         : ִн
******************************************************************************/
int rsa_keypair_gen(uint32_t rsa_bit_len, uint8_t fixkey, uint8_t* n_addr, uint8_t* e_addr, uint8_t *d_addr, uint8_t* p_addr, uint8_t* q_addr, uint8_t* dp_addr, uint8_t* dq_addr, uint8_t* qinv_addr)
{
	uint32_t n_len,p_len,q_len;
	uint8_t result;

	if ((rsa_bit_len<512) ||(rsa_bit_len>2048) ||((rsa_bit_len%32)!=0))	//32
	{
		return ERROR_KEY_LENGTH;
	}
	if((n_addr==NULL) ||(d_addr==NULL)  ||(p_addr==NULL) ||(q_addr==NULL) ||(dp_addr==NULL) ||(dq_addr==NULL) ||(qinv_addr==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)n_addr) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)d_addr) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)p_addr) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)q_addr) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)dp_addr) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)dq_addr) != TYPE_SRAM)
	 ||(CheckAddrType((uint32_t)qinv_addr) != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if ((fixkey!=0) && (fixkey!=1))
	{
		return ERROR_KEY_MODE;
	}
	if ((fixkey == 0) && (e_addr==NULL))		//Կ
	{
		return ERROR_DATA_BUF;
	}

	if (fixkey == 0)
	{
		if(CheckAddrType((uint32_t)e_addr) != TYPE_SRAM)
		{
			return ERROR_DATA_BUF;
		}
	}

	n_len = rsa_bit_len / 32;
	q_len = n_len/2;
	if(n_len %2)
	{
		q_len++;
	}
	p_len = q_len;


	RsaModuleEnable();
	result = RsaGenerateKeyPair(rsa_bit_len,&publicKeyRSA,&privateKeyRSA, fixkey, 1);
	RsaModuleDisable();

	U32toU8(privateKeyRSA.modulus,n_len*4,n_addr,(rsa_bit_len>>3)%4);
	U32toU8(privateKeyRSA.exponent,n_len*4,d_addr,(rsa_bit_len>>3)%4);

	U32toU8(privateKeyRSA.prime[0],p_len*4,p_addr,(rsa_bit_len>>4)%4);
	U32toU8(privateKeyRSA.prime[1],q_len*4,q_addr,(rsa_bit_len>>4)%4);
	U32toU8(privateKeyRSA.primeExponent[0],p_len*4,dp_addr,(rsa_bit_len>>4)%4);
	U32toU8(privateKeyRSA.primeExponent[1],q_len*4,dq_addr,(rsa_bit_len>>4)%4);
	U32toU8(privateKeyRSA.coefficient,p_len*4,qinv_addr,(rsa_bit_len>>4)%4);

	if (fixkey == 0)		//Կ
	{
		U32toU8(publicKeyRSA.exponent,n_len*4,e_addr,(n_len>>1)%4);
	}

	if (result == 0)
	{
		return SUCCESS_ALG;
	}
	else
	{
		return FAIL_ALG;
	}
}

/*******************************************************************************
* Function Name  : RsaPubKey_2
* Description    : RSAԿ
* Input          : -fixkey          :EֵǷ̶
*                  -n_addr          N
*                  -e_addr          :E
*                  -in              :
*                  -in_len          :ݳ
* Output         : -out             
*                  -out_len         ݳ
* Return         : ִн
******************************************************************************/
int RsaPubKey_2(uint8_t fixkey, uint8_t *n_addr,uint8_t* e_addr, uint8_t *in,uint32_t in_len,uint8_t *out,uint32_t *out_len)
{
	int result;
	uint32_t in_buf[256/4],out_buf[256/4];

	if ((in_len<64) ||(in_len>256) ||((in_len%4)!=0))	// 4
	{
		return ERROR_DATA_LENGTH;
	}
	if((n_addr==NULL)  ||(in==NULL)  ||(out==NULL)  ||(out_len==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)n_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)in)     == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)out)    != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	if ((fixkey!=0) && (fixkey!=1))
	{
		return ERROR_KEY_MODE;
	}
	if ((fixkey == 0) && (e_addr==NULL))		//Կ
	{
		return ERROR_DATA_BUF;
	}
	if(fixkey == 0)
	{
		if(CheckAddrType((uint32_t)e_addr) == TYPE_ERROR)
		{
			return ERROR_DATA_BUF;
		}

	}
	memset(&publicKeyRSA,0x0,sizeof(publicKeyRSA));

	//֯Կ
	publicKeyRSA.bits = (in_len<<3);

	if (fixkey == 1)
	{
		publicKeyRSA.exponent[0] = 0x00010001;
	}
	else
	{
		U8toU32(e_addr,in_len,publicKeyRSA.exponent);
	}
	U8toU32(n_addr,in_len,publicKeyRSA.modulus);

	//֯
	U8toU32(in,in_len,in_buf);

	RsaModuleEnable();
	//
	result = RsaPubKey(out_buf,out_len,in_buf,in_len/4,&publicKeyRSA);
	RsaModuleDisable();

	if (result == 0)
	{
		*out_len = in_len;
		U32toU8(out_buf,in_len,out,in_len%4);
		return SUCCESS_ALG;
	}
	else
	{
		return result;
	}
}

/*******************************************************************************
* Function Name  : RsaPrivKey_2
* Description    : RSA˽Կ
* Input          :-n_addr          N
*                 -d_addr          :D
*                  -in              :
*                  -in_len          :ݳ
* Output         : -out             
*                  -out_len         ݳ
* Return         : ִн
******************************************************************************/
int RsaPrivKey_2(uint8_t*n_addr,uint8_t *d_addr,uint8_t *in,uint32_t in_len,uint8_t *out,uint32_t *out_len)
{
	int result;
	uint32_t in_buf[256/4],out_buf[256/4];

	if ((in_len<64) ||(in_len>256) ||((in_len%4)!=0))	// 4
	{
		return ERROR_DATA_LENGTH;
	}
	if((n_addr==NULL)  ||(d_addr==NULL) ||(in==NULL)  ||(out==NULL)  ||(out_len==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)n_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)d_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)in)     == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)out)    != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}


	memset(&privateKeyRSA,0x0,sizeof(privateKeyRSA));
	//֯˽Կ
	privateKeyRSA.bits = (in_len<<3);
	//privateKeyRSA.publicExponent[0] = 0x00010001;
	U8toU32(d_addr,in_len,privateKeyRSA.exponent);
	U8toU32(n_addr,in_len,privateKeyRSA.modulus);

	//֯
	U8toU32(in,in_len,in_buf);

	RsaModuleEnable();
	result = RsaPrivKey(out_buf,out_len,in_buf,in_len/4,&privateKeyRSA);
	RsaModuleDisable();

	if (result == 0)
	{
		*out_len = in_len;
		U32toU8(out_buf,in_len,out,in_len%4);
		return SUCCESS_ALG;
	}
	else
	{
		return result;
	}
}

/*******************************************************************************
* Function Name  : RsaPrivKey_3
* Description    : RSA˽Կ(DPA = High)
* Input          :-n_addr          N
*                 -d_addr          :D
*                  -in              :
*                  -in_len          :ݳ
* Output         : -out             
*                  -out_len         ݳ
* Return         : ִн
******************************************************************************/
int RsaPrivKey_3(uint8_t*n_addr,uint8_t *d_addr,uint8_t *e_addr,uint8_t *in,uint32_t in_len,uint8_t *out,uint32_t *out_len)
{
	int result;
	uint32_t in_buf[256/4],out_buf[256/4];

	if ((in_len<64) ||(in_len>256) ||((in_len%4)!=0))	// 4
	{
		return ERROR_DATA_LENGTH;
	}
	if((n_addr==NULL)  ||(d_addr==NULL) ||(e_addr==NULL) ||(in==NULL)  ||(out==NULL)  ||(out_len==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)n_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)d_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)in)     == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)out)    != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}


	memset(&privateKeyRSA,0x0,sizeof(privateKeyRSA));
	memset(&publicKeyRSA,0x0,sizeof(publicKeyRSA));
	//֯˽Կ
	privateKeyRSA.bits = (in_len<<3);
	//privateKeyRSA.publicExponent[0] = 0x00010001;
	U8toU32(d_addr,in_len,privateKeyRSA.exponent);
	U8toU32(n_addr,in_len,privateKeyRSA.modulus);

	U8toU32(e_addr,in_len,publicKeyRSA.exponent);
	U8toU32(n_addr,in_len,publicKeyRSA.modulus);

	//֯
	U8toU32(in,in_len,in_buf);

	RsaModuleEnable();
	result = RsaPrivKey(out_buf,out_len,in_buf,in_len/4,&privateKeyRSA,&publicKeyRSA);
	RsaModuleDisable();

	if (result == 0)
	{
		*out_len = in_len;
		U32toU8(out_buf,in_len,out,in_len%4);
		return SUCCESS_ALG;
	}
	else
	{
		return result;
	}
}

/*******************************************************************************
* Function Name  : RsaPrivKeyCRT_2
* Description    : RSA CRTģʽ
* Input          : -p_addr          :P
*                  -q_addr          :Q
*                  -dp_addr         :dP
*                  -dq_addr         :dQ
*                  -qinv_addr       :PQ
*                  -in              :
*                  -in_len          :ݳ
* Output         : -out             
*                  -out_len         :ݳ
* Return         : ִн
******************************************************************************/
int RsaPrivKeyCRT_2(uint8_t*p_addr,uint8_t*q_addr,uint8_t*dp_addr,uint8_t*dq_addr,uint8_t*qinv_addr,uint8_t *in,uint32_t in_len,uint8_t *out,uint32_t *out_len)
{
	int result;
	uint32_t in_buf[256/4],out_buf[256/4];
//	uint32_t n_len;

	if ((in_len<64) ||(in_len>256) ||((in_len%4)!=0))	// 4
	{
		return ERROR_DATA_LENGTH;
	}
	if((p_addr==NULL)  ||(q_addr==NULL)  ||(dp_addr==NULL)  ||(dq_addr==NULL)  ||(qinv_addr==NULL) ||(in==NULL)  ||(out==NULL)  ||(out_len==NULL))
	{
		return ERROR_DATA_BUF;
	}

	if((CheckAddrType((uint32_t)p_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)q_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)dp_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)dq_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)qinv_addr) == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)in)     == TYPE_ERROR)
	 ||(CheckAddrType((uint32_t)out)    != TYPE_SRAM))
	{
		return ERROR_DATA_BUF;
	}

	memset(&privateKeyRSA,0x0,sizeof(privateKeyRSA));

	//֯˽Կ
	privateKeyRSA.bits = (in_len<<3);
	//privateKeyRSA.publicExponent[0] = 0x00010001;
	//ע⣺ CRTģʽҪóȫFF
	memset(privateKeyRSA.modulus,0xFF,sizeof(privateKeyRSA.modulus));
	memset(privateKeyRSA.prime,0,sizeof(privateKeyRSA.prime[0]));
	memset(privateKeyRSA.prime,0,sizeof(privateKeyRSA.prime[1]));
	memset(privateKeyRSA.primeExponent,0,sizeof(privateKeyRSA.primeExponent[0]));
	memset(privateKeyRSA.primeExponent,0,sizeof(privateKeyRSA.primeExponent[1]));
	memset(privateKeyRSA.coefficient,0,sizeof(privateKeyRSA.coefficient));

//	n_len = in_len /4;
	U8toU32(p_addr,in_len/2,privateKeyRSA.prime[0]);
	U8toU32(q_addr,in_len/2,privateKeyRSA.prime[1]);
	U8toU32(dp_addr,in_len/2,privateKeyRSA.primeExponent[0]);
	U8toU32(dq_addr,in_len/2,privateKeyRSA.primeExponent[1]);
	U8toU32(qinv_addr,in_len/2,privateKeyRSA.coefficient);
	//֯
	U8toU32(in,in_len,in_buf);

	RsaModuleEnable();
	result = RsaPrivKeyCRT(out_buf,out_len,in_buf,in_len/4,&privateKeyRSA);
	RsaModuleDisable();

	if (result == 0)
	{
		*out_len = in_len;
		U32toU8(out_buf,in_len,out,0);
		return SUCCESS_ALG;
	}
	else
	{
		return result;
	}
}

