// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : ccm_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#include "ccm_drv.h"


/*******************************************************************************
* Function Name  : read_chip_cid
* Description    : read  the chips ID and Revision number
* Input          : None
*
* Output         : - cid:  cid[15~8]a unique version identification number for the chip
*                          cid[15~8]the full-layer mask revision number;
*                          The revision numbers are assigned in chronological order.
* Return         : None
******************************************************************************/
void read_chip_cid(uint16_t *cid)
{
	*cid = CCM->CIR;//OK
}


/*******************************************************************************
* Function Name  : config_freq_detect_point
* Description    : Configuration the frequency detect point
* Input          :  - fd_hi:  Configure the high frequency detect point
*                   - fd_low: Configure the low frequency detect point
*
* Output         : None
*
* Return         : None
******************************************************************************/
//void config_freq_detect_point(uint8_t fd_hi, uint8_t fd_low)
//{
//	CCM->FDCR = (fd_hi<<8) + fd_low;
//}


/*******************************************************************************
* Function Name  : config_osc_bias_trim_value
* Description    : config osc bias trim value
* Input          :  - trim_val:  osc bias trim value
*
*
* Output         : None
*
* Return         : None
******************************************************************************/
void config_osc_bias_trim_value(uint8_t trim_val)
{
	CCM->RTCCFG12 &= 0xfffffff0;
	CCM->RTCCFG12 |= (trim_val&0x0f);
}


/*******************************************************************************
* Function Name  : CCM_ConfigGpio
* Description    : оƬclock outPOR(reset out)ܽóGPIO;
* Input          : - CCM_name: оƬӦPINţȡֵCLOCK_OUTRESET_OUT
*                  - CCM_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*
* Output         : None
* Return         : None
******************************************************************************/
void CCM_ConfigGpio(uint8_t CCM_Name, uint8_t CCM_Dir)
{
	if (CCM_Name == CLOCK_OUT)
	{
		//ʹClock Out GPIO
		*(volatile unsigned int *)(0x40000044) &= ~(1 << 28);
		*(volatile unsigned int *)(0x40000044) |= (1 << 30);
		CCM->PCFG12 |= (1 << 4);

		if(CCM_Dir == GPIO_OUTPUT)
		{
			CCM->PCFG12 &= ~(1 << 7);
			CCM->PCFG12 |= (1 << 5);
		}
		else if(CCM_Dir == GPIO_INPUT)
		{
			CCM->PCFG12 &= ~(1 << 5);
			CCM->PCFG12 |= (1 << 7);
		}
	}
	else if (CCM_Name == RESET_OUT)
	{
		//ʹReset Out GPIO
		*(volatile unsigned int *)(0x40000044) &= ~(1 << 29);
		*(volatile unsigned int *)(0x40000044) |= ((uint32_t)1 << 31);
		CCM->PCFG12 |= 1;

		if(CCM_Dir == GPIO_OUTPUT)
		{
			CCM->PCFG12 &= ~(1 << 3);
			CCM->PCFG12 |= (1 << 1);
		}
		else if(CCM_Dir == GPIO_INPUT)
		{
			CCM->PCFG12 &= ~(1 << 1);
			CCM->PCFG12 |= (1 << 3);
		}
	}
}

/*******************************************************************************
* Function Name  : CCM_Gpio_Diable
* Description    : оƬclock outPOR(reset out)ܽŵGPIOܹصָ֮ǰĹ
* Input          : - CCM_name: оƬӦPINţȡֵCLOCK_OUTRESET_OUT
*
* Output         : None
* Return         : None
******************************************************************************/
void CCM_Gpio_Diable(uint8_t CCM_Name)
{
	if (CCM_Name == CLOCK_OUT)
	{
		CCM->PCFG12 &= ~(1 << 4);
	}
	else if (CCM_Name == RESET_OUT)
	{
		CCM->PCFG12 &= ~1;
	}
}

/*******************************************************************************
* Function Name  : CCM_ReadGpioData
* Description    : ȡоƬGPIOӦŵĵƽ
* Input          : - CCM_name: оƬӦPINţȡֵCLOCK_OUTRESET_OUT
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ
******************************************************************************/
uint8_t CCM_ReadGpioData(uint8_t CCM_Name)
{
	uint32_t bitstatus = 0x00;

	bitstatus = CCM->PCFG12;

	if (CCM_Name == CLOCK_OUT)
	{
		if (bitstatus & (Bit_SET << 9))
		{
			bitstatus = Bit_SET;
		}
		else
		{
			bitstatus = Bit_RESET;
		}
	}
	else if (CCM_Name == RESET_OUT)
	{
		if (bitstatus & (Bit_SET << 8))
		{
			bitstatus = Bit_SET;
		}
		else
		{
			bitstatus = Bit_RESET;
		}
	}

	return bitstatus;
}

/*******************************************************************************
* Function Name  : CCM_WriteGpioData
* Description    : оƬGPIOܽŶӦܽŵĵƽ
* Input          : - CCM_name: оƬӦPINţȡֵCLOCK_OUTRESET_OUT
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : None
******************************************************************************/
void CCM_WriteGpioData(uint8_t CCM_Name, uint8_t bitVal)
{
	if (CCM_Name == CLOCK_OUT)
	{
		if (bitVal == Bit_SET)
		{
			CCM->PCFG12 |= (Bit_SET << 6);
		}
		else
		{
			CCM->PCFG12 &= (~(Bit_SET << 6));
		}
	}
	else if (CCM_Name == RESET_OUT)
	{
		if (bitVal == Bit_SET)
		{
			CCM->PCFG12 |= (Bit_SET << 2);
		}
		else
		{
			CCM->PCFG12 &= (~(Bit_SET << 2));
		}
	}
}

void CCM_PullupConfig(CCM_PINx CCM_Name, FunctionalState NewState)
{
	if (CCM_Name == CLOCK_OUT) //
	{
		if (NewState != DISABLE)
		{
			CCM->PCFG3 |= (1<<8);
		}
		else
		{
			CCM->PCFG3 &= (~(1<<8));
		}
	}
	else if (CCM_Name == RESET_OUT)
	{
		if (NewState != DISABLE)
		{
			CCM->PCFG3 |= (1<<9);
		}
		else
		{
			CCM->PCFG3 &= (~(1<<9));
		}
	}
}

void GINT37_PS(FunctionalState NewState)
{
		if (NewState != DISABLE)
		{
			CCM->PCFG3 |= (1<<14);          //Pull up
		}
		else
		{
			CCM->PCFG3 &= (~(1<<14));        //Pull down
		}
}

#include "pwm_drv.h"
void PWM_function(uint8_t PWMx,FunctionalState NewState)
{
		if (PWMx >= PWM_PORT3) return;
	
		if (NewState != DISABLE)
		{
			CCM->PCFG3 |= (1<<PWMx);
		}
		else
		{
			CCM->PCFG3 &= ~(1<<PWMx);
		}
}