// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : crc_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef CRC_DRV_H_
#define CRC_DRV_H_

#include "type.h"
#include "crc_reg.h"
#include "memmap.h"

#define CRC0        ((CRC_TypeDef *)CRC0_BASE_ADDR)   
#define CRC1        ((CRC_TypeDef *)CRC1_BASE_ADDR)   

#define CRC_DATA_FORMAT_BIGENDDING         (0x01)//Data format is big endding
#define CRC_DATA_FORMAT_LITTLEENDDING      (0x00)//Data format is little endding

#define IS_CRC_DATA_FORMAT(DATA_FORMAT) (((DATA_FORMAT) == CRC_DATA_FORMAT_BIGENDDING) || \
                                         ((DATA_FORMAT) == CRC_DATA_FORMAT_LITTLEENDDING))


#define CRC_SOURCE_SEL_FROM_DMAC           (0x01)//Data from DMAC write
#define CRC_SOURCE_SEL_FROM_CPU            (0x00)//Data from CPU transport

#define IS_CRC_SOURCE_SEL(SOURCE_SEL) (((SOURCE_SEL) == CRC_SOURCE_SEL_FROM_DMAC) || \
                                       ((SOURCE_SEL) == CRC_SOURCE_SEL_FROM_CPU))

#define CRC_MODE_CRC_32                    (0x04)//32bit CRC data mode
#define CRC_MODE_CRC_16                    (0x02)//16bit CRC data mode
#define CRC_MODE_CRC_8                     (0x01)//8bit CRC data mode

#define IS_CRC_MODE(MODE) (((MODE) == CRC_MODE_CRC_32) || \
                           ((MODE) == CRC_MODE_CRC_16) || \
                           ((MODE) == CRC_MODE_CRC_8))

typedef struct
{

  uint8_t CRC_Mode;//CRC data mode
  uint8_t CRC_SourceSel;//Data from sram transport or CPU write
  uint8_t CRC_DataFormat;//Data format when in CRC32 mode

}CRC_InitTypeDef;


/*******************************************************************************
* Function Name  : CRC_Config
* Description    : CRC.
* Input          : - CRC_InitTypeDef : CRC_InitTypeDefṹָ룬£
*                    CRC_InitStruct->CRC_Mode:CRCģʽȡֵΪCRC_MODE_CRC_xxȡֵ8,16,32
*                    CRC_InitStruct->CRC_SourceSel: CRCԴѡȡֵCRC_SOURCE_SEL_FROM_DMACRC_SOURCE_SEL_FROM_CPU
*                    CRC_InitStruct->CRC_DataFormatCRCݸʽģʽСģʽCRC_MODE_CRC_32£Ч
*
*
* Output         : None
* Return         : None
*******************************************************************************/
extern void CRC_Config(CRC_InitTypeDef *CRC_InitStruct);

/*******************************************************************************
* Function Name  : CRC_SetInitValue
* Description    : CRCʼֵ
* Input          :  - InitValue : õCRCʼֵ
*
* Output         : None
* Return         : None
*******************************************************************************/
void CRC_SetInitValue(uint32_t InitValue);

/*******************************************************************************
* Function Name  : CRC_CalcCRC32
* Description    : CRC 32 
* Input          : - pBuffer : ҪCRCݻ
*                  - BufferLen : ҪCRCݻĳ
*
* Output         : CRC
* Return         : None
*******************************************************************************/
uint32_t CRC_CalcCRC32(uint8_t *pBuffer, uint32_t BufferLen);

/*******************************************************************************
* Function Name  : CRC_CalcCRC16
* Description    : CRC 16 
* Input          : - pBuffer : ҪCRCݻ
*                  - BufferLen : ҪCRCݻĳ
*
* Output         : CRC
* Return         : None
*******************************************************************************/
uint16_t CRC_CalcCRC16(uint8_t *pBuffer, uint32_t BufferLen);

/*******************************************************************************
* Function Name  : CRC_CalcCRC8
* Description    : CRC 8 
* Input          : - pBuffer : ҪCRCݻ
*                  - BufferLen : ҪCRCݻĳ
*
* Output         : CRC
* Return         : None
*******************************************************************************/
uint8_t CRC_CalcCRC8(uint8_t *pBuffer, uint32_t BufferLen);

#endif /* CRC_DRV_H_ */
