// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : eflash_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#include "common.h"
#include "libEflash.h"
#include "eflash_drv.h"
#include <string.h>

uint32_t  eflash_buffer[EFLASH_PAGE_SIZE_BYTE/4];

/*******************************************************************************
* Function Name  : EFLASH_Set_RWSC
* Description    : EFLASH RWSCֵ
* Input          : -rwsc: rwscֵĬΪ0x07
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_Set_RWSC(uint8_t rwsc)
{
	uint32_t efcr_reg = readl(EFCR);
	
	efcr_reg &= (~0x0000000F);
	efcr_reg |= (rwsc & 0x0F);
	
	writel(EFCR,efcr_reg);
}

/*******************************************************************************
* Function Name  : EFLASH_Init
* Description    : EFLASHʼ
* Input          : - f_sys_khz: ϵͳʱӣλkhz
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_Init(uint32_t sys_clk_khz)
{
	EFM_Init(sys_clk_khz);
}

/*******************************************************************************
* Function Name  : EFLASH_SetWritePermission
* Description    : EFLASH׿
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_SetWritePermission(void)
{
	EFlash_Set_Main_Permission(EFLASH_WRITEABLE, EFLASH_READABLE);
}

/*******************************************************************************
* Function Name  : EFLASH_ClrWritePermission
* Description    : رEFLASH׿
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_ClrWritePermission(void)
{
	EFlash_Set_Main_Permission(EFLASH_DISWRITEABLE, EFLASH_READABLE);
}

/*******************************************************************************
* Function Name  : EFLASH_ByteRead
* Description    : EFLASHĿַȡһbyte
* Input          : - addr: EFLASH destination address
*
* Output         : None
* Return         : ȡbyte
******************************************************************************/
uint8_t EFLASH_ByteRead(uint32_t addr)
{
	return (*(volatile uint8_t *)(addr));
}

/*******************************************************************************
* Function Name  : EFLASH_HalfWordRead
* Description    : EFLASHĿַȡһhalf word
* Input          : - addr: EFLASH destination address
*
* Output         : None
* Return         : ȡhalf word
******************************************************************************/
uint16_t EFLASH_HalfWordRead(uint32_t addr)
{
	return (*(volatile uint16_t *)(addr));
}

/*******************************************************************************
* Function Name  : EFLASH_WordRead
* Description    : EFLASHĿַȡһword
* Input          : - addr: EFLASH destination address
*
* Output         : None
* Return         : ȡword
******************************************************************************/
uint32_t EFLASH_WordRead(uint32_t addr)
{
	return (*(volatile uint32_t *)(addr));
}

/*******************************************************************************
* Function Name  : EFLASH_WordsRead
* Description    :  EFLASHĿַȡΪlenword
* Input          : - addr: EFLASH destination address
*                  - len:  Ҫȡwordݵĳ
*
* Output         : - des:  ȡwordݻ
* Return         : None
******************************************************************************/
void EFLASH_WordsRead(uint32_t addr, uint32_t *des, uint32_t len)
{
	uint32_t i;

	for(i = 0; i < len; i ++)
	{
		des[i] = (*(volatile uint32_t *)(addr+i*4));
	}
}

/*******************************************************************************
* Function Name  : EFLASH_PageErase
* Description    : EFLASHҳ
* Input          : - addr: ҪEFLASHҳַ
*
* Output         : None
* Return         :  TRUEɹ   FALSEʧ
******************************************************************************/
uint8_t EFLASH_PageErase(uint32_t addr)
{
	if(EFlash_Page_Erase(addr))
	{
		return FALSE;
	}
	return TRUE;
}

/*******************************************************************************
* Function Name  : EFLASH_WordProg
* Description    : EFLASHĵword
* Input          : - addr: Ҫ̵EFLASHַ
*                  - dat:  ̵word
*
* Output         : None
* Return         :  TRUE̳ɹ   FALSEʧ
******************************************************************************/
uint8_t EFLASH_WordProg(uint32_t addr, uint32_t dat)
{
	if(EFlash_Bulk_Program((addr), 1, &dat))
	{
		return FALSE;
	}
	return TRUE;
}

/*******************************************************************************
* Function Name  : EFLASH_WordsProg
* Description    :  EFLASHĶword
* Input          : - addr: Ҫ̵EFLASHַ
*                  - des:  ̵wordָ
*                  - len:  Ҫ̵wordݳ
*
* Output         : None
* Return         :  TRUE̳ɹ   FALSEʧ
******************************************************************************/
uint8_t EFLASH_WordsProg(uint32_t addr, uint32_t *des, uint32_t len)
{
	if(EFlash_Bulk_Program((addr), len, des))
	{
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
* Function Name  : FLASH_checkAddr
* Description    :  EFLASHַ
* Input          : - flash_addr: Ҫ̵EFLASHַ
*                  - len:  Ҫ̵byteݳ
*
* Output         : None
* Return         :  TRUE̳ɹ   FALSEʧ
******************************************************************************/
static uint8_t FLASH_checkAddr(uint32_t flash_addr,uint32_t len)
{
	if((flash_addr%4) != 0)
	{
		return FALSE;
	}
	if((flash_addr < EFLASH_BASE) || (flash_addr+len) > (EFLASH_BASE + EFLASH_BYTE_SIZE - 0x1000))
	{
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
* Function Name  : EFLASH_Write
* Description    :  EFLASHַд
* Input          : - flash_addr: Ŀĵַ
*                  - ram_addr:  Դַָ
*                  - len:  ݳ
*
* Output         : None
* Return         :  TRUE̳ɹ   FALSEʧ
******************************************************************************/
uint8_t EFLASH_Write(uint32_t flash_addr, const uint8_t* ram_addr, uint32_t len)
{
	uint16_t offset;
	uint16_t pageleft;
	uint16_t pageright;
	uint32_t dest = flash_addr;
	uint8_t* src  = (uint8_t*)ram_addr;
	uint8_t result;
	
	pageright = pageright;

	if (FLASH_checkAddr(flash_addr,len) == FALSE)
	{
		return FALSE;
	}

	offset  = ((uint32_t)dest & PAGE_MASK);
	pageleft  = EFLASH_PAGE_SIZE_BYTE - offset;
	pageright = ((uint16_t)(dest + len) & PAGE_MASK);

	/*ҳûж룬ȶԵһҳ*/
	if (offset != 0)
	{
		dest -= offset;/*destָתǰҳͷ*/
		memcpy((uint8_t*)eflash_buffer, (uint8_t*)dest, EFLASH_PAGE_SIZE_BYTE);
		if (len>pageleft)
		{
			memcpy((uint8_t*)eflash_buffer + offset, src, pageleft);
			len  -= pageleft;
			src  += pageleft;
		}
		else
		{
			memcpy((uint8_t*)eflash_buffer + offset, src, len);
			len=0;
		}
		result = EFLASH_PageErase(dest);
		result = EFLASH_WordsProg(dest, eflash_buffer, EFLASH_PAGE_SIZE_BYTE/4);
		dest += EFLASH_PAGE_SIZE_BYTE;
		//offset = 0;
	}
	while (len > 0)
	{
		/*мҳ*/
		if (len >= EFLASH_PAGE_SIZE_BYTE)
		{
			memcpy((uint8_t*)eflash_buffer, src, EFLASH_PAGE_SIZE_BYTE);
			result = EFLASH_PageErase(dest);
			result = EFLASH_WordsProg(dest, eflash_buffer, EFLASH_PAGE_SIZE_BYTE/4);

			dest += EFLASH_PAGE_SIZE_BYTE;
			src  += EFLASH_PAGE_SIZE_BYTE;
			len  -= EFLASH_PAGE_SIZE_BYTE;
		}
		else
		{   /*ҳ*/
			memcpy((uint8_t*)eflash_buffer, (uint8_t*)dest, EFLASH_PAGE_SIZE_BYTE);
			memcpy((uint8_t*)eflash_buffer, src, len);
			result = EFLASH_PageErase(dest);
			result = EFLASH_WordsProg(dest, eflash_buffer, EFLASH_PAGE_SIZE_BYTE/4);
			len = 0;
		}
	}

	if (result == 0)
		return TRUE;
	else
		return FALSE;
}
