// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : eflash_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifndef __EFLASH_DRV_H__
#define __EFLASH_DRV_H__

#include "type.h"
#include "memmap.h"

#define EFCR	EFM_BASE_ADDR

#define EFLASH_BASE				          (0x8000000)
#define EFLASH_PAGE_SIZE_BYTE       (0x200)		//512
#define EFLASH_PAGE_SIZE_HFWORD     (0x100)		//256
#define EFLASH_PAGE_SIZE_WORD   	  (0x80)		//128
#define EFLASH_BYTE_SIZE	           (0x80000)	//512kbytes, 128k words
#define EFLASH_ALL_PAGE_LEN         (1024)      //1024page
#define PAGE_MASK                   (EFLASH_PAGE_SIZE_BYTE-1)

#define EFLASH_WRITEABLE	  TRUE
#define EFLASH_READABLE	    TRUE
#define EFLASH_DISWRITEABLE	FALSE
#define EFLASH_DISREADABLE	FALSE

#define EFM0_MAIN_BASEADDR	(0x08000000)
#define EFM0_MAIN_LEN				(0x00080000)

#define EFLASH_PAGE_SIZE_BYTE			(0x200)
#define EFLASH_PAGE_SIZE_HFWORD   (0x100)		//256
#define EFLASH_PAGE_SIZE_WORD   	(0x80)		//128

#define EFLASH_OP_PASS		0
#define EFLASH_OP_FAIL		1
#define EFLASH_PARA_ERR		2

/*******************************************************************************
* Function Name  : EFlash_Disboot
* Description    : disboot
* Input          : - None
*
* Output         : None
* Return         : TRUE or FALSE
******************************************************************************/
extern uint8_t EFlash_Disboot(void);

/*******************************************************************************
* Function Name  : EFlash_Recovery_to_Boot
* Description    : صROM BOOT״̬
* Input          : - None
*
* Output         : None
* Return         : TRUE or FALSE
******************************************************************************/
extern uint8_t EFlash_Recovery_to_Boot(void);

/*******************************************************************************
* Function Name  : EFLASH_Set_RWSC
* Description    : EFLASH RWSCֵ
* Input          : -rwsc: rwscֵĬΪ0x07
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_Set_RWSC(uint8_t rwsc);

/*******************************************************************************
* Function Name  : EFLASH_Init
* Description    : EFLASHʼ
* Input          : - f_sys_khz: ϵͳʱӣλkhz
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_Init(uint32_t sys_clk_khz);

/*******************************************************************************
* Function Name  : EFLASH_SetWritePermission
* Description    : EFLASH׿
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_SetWritePermission(void);

/*******************************************************************************
* Function Name  : EFLASH_ClrWritePermission
* Description    : رEFLASH׿
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void EFLASH_ClrWritePermission(void);

/*******************************************************************************
* Function Name  : EFLASH_ByteRead
* Description    : EFLASHĿַȡһbyte
* Input          : - addr: EFLASH destination address
*
* Output         : None
* Return         : ȡbyte
******************************************************************************/
extern uint8_t   EFLASH_ByteRead(uint32_t addr);

/*******************************************************************************
* Function Name  : EFLASH_HalfWordRead
* Description    : EFLASHĿַȡһhalf word
* Input          : - addr: EFLASH destination address
*
* Output         : None
* Return         : ȡhalf word
******************************************************************************/
extern uint16_t  EFLASH_HalfWordRead(uint32_t addr);

/*******************************************************************************
* Function Name  : EFLASH_WordRead
* Description    : EFLASHĿַȡһword
* Input          : - addr: EFLASH destination address
*
* Output         : None
* Return         : ȡword
******************************************************************************/
extern uint32_t  EFLASH_WordRead(uint32_t addr);

/*******************************************************************************
* Function Name  : EFLASH_WordsRead
* Description    :  EFLASHĿַȡΪlenword
* Input          : - addr: EFLASH destination address
*                  - len:  Ҫȡwordݵĳ
*
* Output         : - des:  ȡwordݻ
* Return         : None
******************************************************************************/
extern void    EFLASH_WordsRead(uint32_t addr, uint32_t *des, uint32_t len);

/*******************************************************************************
* Function Name  : EFLASH_PageErase
* Description    : EFLASHҳ
* Input          : - addr: ҪEFLASHҳַ
*
* Output         : None
* Return         :  0ɹ   1ʧ
******************************************************************************/
extern uint8_t   EFLASH_PageErase(uint32_t addr);

/*******************************************************************************
* Function Name  : EFLASH_WordProg
* Description    : EFLASHĵword
* Input          : - addr: Ҫ̵EFLASHַ
*                  - dat:  ̵word
*
* Output         : None
* Return         :  0̳ɹ   1ʧ
******************************************************************************/
extern uint8_t EFLASH_WordProg(uint32_t addr, uint32_t dat);

/*******************************************************************************
* Function Name  : EFLASH_WordsProg
* Description    :  EFLASHĶword
* Input          : - addr: Ҫ̵EFLASHַ
*                  - des:  ̵wordָ
*                  - len:  Ҫ̵wordݳ
*
* Output         : None
* Return         :  0̳ɹ   1ʧ
******************************************************************************/
extern uint8_t   EFLASH_WordsProg(uint32_t addr, uint32_t *des, uint32_t len);

/*******************************************************************************
* Function Name  : EFLASH_Write
* Description    :  EFLASHַд
* Input          : - flash_addr: Ŀĵַ
*                  - ram_addr:  Դַָ
*                  - len:  ݳ
*
* Output         : None
* Return         :  TRUE̳ɹ   FALSEʧ
******************************************************************************/
uint8_t EFLASH_Write(uint32_t flash_addr, const uint8_t* ram_addr, uint32_t len);

#endif // end of __EFLASH_DRV_H__
