/*
 * icc_conf.h
 *
 *  Created on: 2017526
 *      Author: YangWenfeng
 */

#ifndef ICC_CONF_H_
#define ICC_CONF_H_
#include "type.h"
#include "cpm_drv.h"
#include "eport_drv.h"

#define COLD_RESET		0
#define WARM_RESET	    1

/* 7816 protocol constants ---------------------------------------------------*/
#define T0_PROTOCOL        0x00  /* T0 protocol */
#define T1_PROTOCOL        0x01  /* T1 protocol */
#define DIRECT             0x3B  /* Direct bit convention */
#define INDIRECT           0x3F  /* Indirect bit convention */
#define SETUP_LENGTH       20
#define HIST_LENGTH        20
#define LCmax              258//300ʵʳȣֹԽ, 111029

/* 7816 timeout Constants ----------------------------------------------------*/
/* Timeout constants are decided by CPU's performence , the default CPU's freq is 36MHZ*/
/*RESET Timeout is about 55 ms*/
#define SC_Receive_Timeout      0x355548

#define STATUS_SUCCESS          0x00
#define STATUS_TIMEOUT_ERROR 	0xF0

/*-------------------------- Answer to reset Commands ------------------------*/
#define SC_GET_A2R         0x12

#define OPEN_ICCARD_1V8		Card2_Power_Switch(card_1v8) 		//1V8
#define CLOSE_ICCARD_1V8	Card2_Power_Off()		            //ر1V8
#define OPEN_ICCARD_3V		Card2_Power_Switch(card_3v) 		//3V
#define CLOSE_ICCARD_3V		Card2_Power_Off()		            //ر3V
#define OPEN_ICCARD_5V		Card2_Power_Switch(card_5v)  		//5V
#define CLOSE_ICCARD_5V		Card2_Power_Off()		            //ر5V
#define ICCARD_PRESENT    1//!EPORT_Read_Gpio_Data(EPORT_PIN7)      //ο˿
/* Exported types ------------------------------------------------------------*/

typedef enum
{
	ERROR = 0,
	SUCCESS  = !ERROR
} ErrorStatus;


typedef enum
{
	SC_POWER_ON = 0x00,
	SC_RESET_LOW = 0x01,
	SC_RESET_HIGH = 0x02,
	SC_ACTIVE = 0x03,
	SC_ACTIVE_ON_T0 = 0x04,
	SC_ACTIVE_ON_T1 = 0x05,
	SC_POWER_OFF = 0x06
} SC_State;


/* ATR structure - Answer To Reset -------------------------------------------*/
typedef struct
{
	uint8_t TS;               /* Bit Convention */
	uint8_t T0;               /* High nibble = Number of setup byte; low nibble = Number of historical byte */
	uint8_t T[SETUP_LENGTH];  /* Setup array */
	uint8_t H[HIST_LENGTH];   /* Historical array */
	uint8_t Tlength;          /* Setup array dimension */
	uint8_t Hlength;          /* Historical array dimension */
	uint8_t SW1;              /* Status byte*/
	uint8_t SW2;              /* Status byte*/
} SC_ATR;

/* ADPU-Header command structure ---------------------------------------------*/
typedef struct
{
	uint8_t CLA;  /* Command class */
	uint8_t INS;  /* Operation code */
	uint8_t P1;   /* Selection Mode */
	uint8_t P2;   /* Selection Option */
} SC_Header;

/* ADPU-Body command structure -----------------------------------------------*/
typedef struct
{
	uint8_t LC;           /* Data field length */
	uint8_t Data[LCmax];  /* Command parameters */
	uint8_t LE;           /* Expected length of data to be returned */
} SC_Body;

/* ADPU Command structure ----------------------------------------------------*/
typedef struct
{
	SC_Header Header;
	SC_Body Body;
} SC_ADPU_Commands;

/* SC response structure -----------------------------------------------------*/
typedef struct
{
	uint8_t Data[LCmax];  /* Data returned from the card */
	uint8_t SW1;          /* Command Processing status */
	uint8_t SW2;          /* Command Processing qualification */
} SC_ADPU_Responce;

/*SC smartcard reader protocol structure--------------------------------------*/
typedef struct
{
	uint8_t SC_Protocol;
	uint8_t SC_Convention;
	uint8_t SC_T0CWI;
	uint8_t SC_T1CWI;
	uint8_t SC_BWI;
	uint8_t SC_BGT;
	uint8_t SC_WTX;
	uint8_t SC_TC1N;
	uint8_t SC_IFS;
	uint8_t SC_T1NAD;
	uint8_t SC_T1PCB;
	uint8_t SC_T1Start;
	uint8_t SC_T1Block;
	uint8_t SC_Retry;
	uint32_t SC_iBWT;
	uint32_t SC_iCWT;
	SC_State SC_CurState;
	uint32_t SC_workbaudrate;
} SC_READER_SET;

#endif /* ICC_CONF_H_ */
