// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : pci_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#include "debug.h"
#include "iomacros.h"
#include "pci_drv.h"
#include "cpm_drv.h"


/*******************************************************************************
* Function Name  : PCI_Init_Key
* Description    : PCIԿʼ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
static void PCI_Init_Key(void)
{
	PCI->PCI_WRITEEN = 0x40000000;
	PCI->PCI_WRITEEN = 0x80000000;
	PCI->PCI_WRITEEN = 0xC0000000;
}

/*******************************************************************************
* Function Name  : Nvram_Init_Key
* Description    : NVramԿʼ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
static void Nvram_Init_Key(void)
{
	PCI->PCI_NVRAMKEY = 0x35;
	PCI->PCI_NVRAMKEY = 0x79;
	PCI->PCI_NVRAMKEY = 0xBD;
	PCI->PCI_NVRAMKEY = 0xF1;
}

/*******************************************************************************
* Function Name  : PCI_Clk_Init_Key
* Description    : PCI ʱԿʼ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
static void PCI_Clk_Init_Key(void)
{
	PCI->PCI_CLKCER = 0x12;
	PCI->PCI_CLKCER = 0x15;
	PCI->PCI_CLKCER = 0x35;
	PCI->PCI_CLKCER = 0x24;
}

/*******************************************************************************
* Function Name  : PCI_Init
* Description    : PCIģʼ
* 				         ע⣺
* 				         ÿϵǰҪִиó򣬷޷дnvramе
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_Init(uint8_t clk_sel)
{
	//enable register operation
	PCI_Init_Key();
	PCI->PCI_WRITEEN = 0xC000003F;

	//input RTC clock
	PCI->PCI_RTCIER |= 0x01;		//RTC Interface Enable
	while((PCI->PCI_RTCIER & 0x80000000) != 0x80000000);
	PCI->PCI_RTCIER &= ~(1 << 6);
		
	//RTC32K Clock Enable
	PCI->PCI_DETPCR |= 0x80000000;//֤Ŧ۵е
	
	//2019-4-3
	//IRC frequency and current trim config 
	//PCI->PCI_IRCCR = 0x84;	 //ʹRTCʱӺtrim
	
	if(clk_sel == EXTERNAL_CLK_SEL)
	{
		PCI_Clk_Init_Key();
		PCI->PCI_CLKSR |= 0x00000080;	
	}
}

/*******************************************************************************
* Function Name  : Nvram_Init
* Description    : nvramģʼ
* 				         ע⣺
* 				    дNVRAMǰҪʹܣȡʱҪ
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void Nvram_Init(void)
{
	Nvram_Init_Key();

	PCI->PCI_NVRAMCR |= ((uint32_t)1 << 31);
}

/*******************************************************************************
* Function Name  : Nvram_Write
* Description    : NVramд
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void Nvram_Write(void)
{
	Nvram_Init();

	IO_WRITE32(SRAM_START_ADDR_M4, 0x12345678);
	IO_WRITE32((SRAM_START_ADDR_M4 + 0x40), 0xfedcba89);
}

/*******************************************************************************
* Function Name  : Nvram_Read
* Description    : NVram
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void Nvram_Read(void)
{
	printf("[SRAM_START_ADDR]: %08x\r\n", *(unsigned int *)SRAM_START_ADDR_M4);
	printf("[SRAM_START_ADDR+0x200]: %08x\r\n", *(unsigned int *)(SRAM_START_ADDR_M4 + 0x40));
}

/*******************************************************************************
* Function Name  : Nvram_Status
* Description    : жNVramǷ
* Input          : None
*
* Output         : 0 - ûб
*                  1 - 
* Return         : None
******************************************************************************/
int Nvram_Status(void)
{
	unsigned int flag;

	flag = PCI->PCI_DETSLR;
	if((flag & PCI_NVRAM_FLAG) == PCI_NVRAM_FLAG)
	{
		return 1;
	}

	return 0;
}
/*******************************************************************************
* Function Name  : PCI_SDIOInit
* Description    : SDIOʼfail num
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_SDIO_Init(void)
{			
	PCI->PCI_CR &= ~(0x0F << 8);
	
	//0
	PCI->PCI_SDIOCR = 0x000f0000;

}

/*******************************************************************************
* Function Name  : PCI_IntInit
* Description    : PCIİȫж
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_Interrupt_Init(void)
{
	//ʹPCIжϹ
	PCI->PCI_DETSR |= ((uint32_t)0xE6 << 24);
	NVIC_Init(3, 3, PCI_DET_IRQn, 2);
}

/*******************************************************************************
* Function Name  : PCI_SDIO_Static_Set
* Description    : SDIO̬ʽSDIOܽſԽӵأcmp_dataֵд0
* Input          : channel - SDIOͨ
*                  cmp_data - Ƚϵѹֵ:0 - ·SDIOΪ͵ѹά֣ߵѹ1 - ·SDIOΪߵѹά֣͵ѹ
* Output         : None
* Return         : None
******************************************************************************/
void PCI_SDIO_Static_Set(unsigned char channel, unsigned char cmp_data)
{
	//sdio touch set
	if (cmp_data == 0)
	{
		PCI->PCI_SDIOCR &= (~channel);
	}
	else
	{
		PCI->PCI_SDIOCR |= channel;
	}
	//enable
	PCI->PCI_SDIOCR |= (channel << 24);//enable
}

/*******************************************************************************
* Function Name  : PCI_SDIO_Dynamic_Set
* Description    : SDIO̬ʽ
* Input          : channel - SDIOͨ
* Output         : None
* Return         : None
******************************************************************************/
void PCI_SDIO_Dynamic_Set(unsigned char channel)
{
	PCI->PCI_SDIOCR |= (channel << 20);

	//dynamic seed
	PCI->PCI_SDIOCR |= (0x55 << 8);

	//enasble sdio 0_1
	if((channel & PCI_SDIO_DYNAMIC_CHANNEL_0) == PCI_SDIO_DYNAMIC_CHANNEL_0)
	{
		PCI->PCI_SDIOCR |= (0x03 << 24);
	}
	
	//enasble sdio 2_3
	if((channel & PCI_SDIO_DYNAMIC_CHANNEL_1) == PCI_SDIO_DYNAMIC_CHANNEL_1)
	{
		PCI->PCI_SDIOCR |= (0x03 << 26);
	}

	//enasble sdio 4_5
	if((channel & PCI_SDIO_DYNAMIC_CHANNEL_2) == PCI_SDIO_DYNAMIC_CHANNEL_2)
	{
		PCI->PCI_SDIOCR |= (0x03 << 28);
	}

	//enasble sdio 6_7
	if((channel & PCI_SDIO_DYNAMIC_CHANNEL_3) == PCI_SDIO_DYNAMIC_CHANNEL_3)
	{
		PCI->PCI_SDIOCR |= ((uint32_t)0x03 << 30);
	}
}

/*******************************************************************************
* Function Name  : PCI_VD_Set
* Description    : VD
* Input          : VDH_Value:ѹֵ
*                  VDL_Value:ѹֵ
* Output         : None
* Return         : None
******************************************************************************/
void PCI_VD_Set(unsigned char VDH_Value, unsigned char VDL_Value)
{
	//set H_VD value
	PCI->PCI_VDCR &= ~(3<<2);
	PCI->PCI_VDCR |= (VDH_Value<<2);

	//set L_VD value
	PCI->PCI_VDCR &= ~(7<<4);
	PCI->PCI_VDCR |= (VDL_Value<<4);

	//enable H_VD and L_VD detect
	PCI->PCI_VDCR |= 0x03;
	PCI->PCI_VDCR |= ((uint32_t)1<<31);
}

/*******************************************************************************
* Function Name  : PCI_TD_Set
* Description    : TD
* Input          : TDH_Value:¼ֵ
*                  TDL_Value:¼ֵ
*                  TD_Trim  :¶ֵ
* Output         : None
* Return         : None
******************************************************************************/
void PCI_TD_Set(unsigned char TDH_Value, unsigned char TDL_Value, unsigned char TD_Trim)
{
	//set H_TD value
	PCI->PCI_TDCR &= ~0x3F;
	PCI->PCI_TDCR |= TDH_Value;

	//set L_TD value
	PCI->PCI_TDCR &= ~(0x1F << 8);
	PCI->PCI_TDCR |= (TDL_Value << 8);

	//enable TD detect
	PCI->PCI_TDCR |= ((uint32_t)1<<31);
}

/*******************************************************************************
* Function Name  : PCI_Clear_Status
* Description    : PCIȫ־λnvram־λ
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_Clear_Status(void)
{
	PCI->PCI_DETSLR |= ((0xFF) | (0xFF << 8) | (0x01 << 16));
	PCI->PCI_DETSR |= ((0xFF << 8) | (0xFF << 16));
}

/*******************************************************************************
* Function Name  : PCI_Detect
* Description    : жPCIȫԴ
* Input          : None
* Output         : None
* Return         : 0 - ûд
*                  1 - д
******************************************************************************/
int PCI_Detect(void)
{
	unsigned int flag;
	int ret_val = 0;

	flag = PCI->PCI_DETSLR;

	//PCI VD
	if((flag & PCI_VD_L_FLAG) == PCI_VD_L_FLAG)
	{
		printf("PCI VD_L touch!\r\n");
		ret_val = 1;
	}

	if((flag & PCI_VD_H_FLAG) == PCI_VD_H_FLAG)
	{
		printf("PCI VD_H touch!\r\n");
		ret_val = 1;
	}

	//PCI TD
	if((flag & PCI_TD_L_FLAG) == PCI_TD_L_FLAG)
	{
		printf("PCI TD_L touch!\r\n");
		ret_val = 1;
	}

	if((flag & PCI_TD_H_FLAG) == PCI_TD_H_FLAG)
	{
		printf("PCI TD_H touch!\r\n");
		ret_val = 1;
	}

	//PCI SDIO
	if((flag & PCI_SDIO_CHANNEL_0) == PCI_SDIO_CHANNEL_0)
	{
		if((PCI->PCI_SDIOCR & 0x100000) == 0x100000)
		{
			printf("SDIO dynamic-0 touch!\r\n");
		}
		else
		{
			printf("SDIO static-0 touch!\r\n");
		}
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_1) == PCI_SDIO_CHANNEL_1)
	{
		printf("SDIO static-1 touch!\r\n");
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_2) == PCI_SDIO_CHANNEL_2)
	{
		if((PCI->PCI_SDIOCR & 0x200000) == 0x200000)
		{
			printf("SDIO dynamic-1 touch!\r\n");
		}
		else
		{
			printf("SDIO static-2 touch!\r\n");
		}
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_3) == PCI_SDIO_CHANNEL_3)
	{
		printf("SDIO static-3 touch!\r\n");
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_4) == PCI_SDIO_CHANNEL_4)
	{
		if((PCI->PCI_SDIOCR & 0x400000) == 0x400000)
		{
			printf("SDIO dynamic-2 touch!\r\n");
		}
		else
		{
			printf("SDIO static-4 touch!\r\n");
		}
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_5) == PCI_SDIO_CHANNEL_5)
	{
		printf("SDIO static-5 touch!\r\n");
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_6) == PCI_SDIO_CHANNEL_6)
	{
		if((PCI->PCI_SDIOCR & 0x800000) == 0x800000)
		{
			printf("SDIO dynamic-3 touch!\r\n");
		}
		else
		{
			printf("SDIO static-6 touch!\r\n");
		}
		ret_val = 1;
	}

	if((flag & PCI_SDIO_CHANNEL_7) == PCI_SDIO_CHANNEL_7)
	{
		printf("SDIO static-7 touch!\r\n");
		ret_val = 1;
	}

	//NVRAM flag
	if((flag & PCI_NVRAM_FLAG) == PCI_NVRAM_FLAG)
	{
		printf("nvram clean!\r\n");
	}

	return ret_val;
}

/*******************************************************************************
* Function Name  : PCI_IRQHandler
* Description    : PCIȫжϴ
*                  PCIжϹܣоƬϵҪáϵòᱣ
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_DET_IRQHandler(void)
{
	unsigned int flag;

	flag = PCI->PCI_DETSLR;

	//PCI VD
	if((flag & PCI_VD_L_FLAG) == PCI_VD_L_FLAG)
	{
		printf("PCI VD_L touch!\r\n");
		PCI->PCI_DETSLR |= PCI_VD_L_FLAG;
	}

	if((flag & PCI_VD_H_FLAG) == PCI_VD_H_FLAG)
	{
		printf("PCI VD_H touch!\r\n");
		PCI->PCI_DETSLR |= PCI_VD_H_FLAG;
	}

	//PCI TD
	if((flag & PCI_TD_L_FLAG) == PCI_TD_L_FLAG)
	{
		printf("PCI TD_L touch!\r\n");
		PCI->PCI_DETSLR |= PCI_TD_L_FLAG;
	}

	if((flag & PCI_TD_H_FLAG) == PCI_TD_H_FLAG)
	{
		printf("PCI TD_H touch!\r\n");
		PCI->PCI_DETSLR |= PCI_TD_H_FLAG;
	}

	//PCI SDIO
	if((flag & PCI_SDIO_CHANNEL_0) == PCI_SDIO_CHANNEL_0)
	{
		if((PCI->PCI_SDIOCR & 0x100000) == 0x100000)
		{
			printf("SDIO dynamic-0 touch!\r\n");
		}
		else
		{
			printf("SDIO static-0 touch!\r\n");
		}

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_0;
	}

	if((flag & PCI_SDIO_CHANNEL_1) == PCI_SDIO_CHANNEL_1)
	{
		printf("SDIO static-1 touch!\r\n");

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_1;
	}

	if((flag & PCI_SDIO_CHANNEL_2) == PCI_SDIO_CHANNEL_2)
	{
		if((PCI->PCI_SDIOCR & 0x200000) == 0x200000)
		{
			printf("SDIO dynamic-1 touch!\r\n");
		}
		else
		{
			printf("SDIO static-2 touch!\r\n");
		}

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_2;
	}

	if((flag & PCI_SDIO_CHANNEL_3) == PCI_SDIO_CHANNEL_3)
	{
		printf("SDIO static-3 touch!\r\n");

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_3;
	}

	if((flag & PCI_SDIO_CHANNEL_4) == PCI_SDIO_CHANNEL_4)
	{
		if((PCI->PCI_SDIOCR & 0x400000) == 0x400000)
		{
			printf("SDIO dynamic-2 touch!\r\n");
		}
		else
		{
			printf("SDIO static-4 touch!\r\n");
		}

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_4;
	}

	if((flag & PCI_SDIO_CHANNEL_5) == PCI_SDIO_CHANNEL_5)
	{
		printf("SDIO static-5 touch!\r\n");

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_5;
	}

	if((flag & PCI_SDIO_CHANNEL_6) == PCI_SDIO_CHANNEL_6)
	{
		if((PCI->PCI_SDIOCR & 0x800000) == 0x800000)
		{
			printf("SDIO dynamic-3 touch!\r\n");
		}
		else
		{
			printf("SDIO static-6 touch!\r\n");
		}

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_6;
	}

	if((flag & PCI_SDIO_CHANNEL_7) == PCI_SDIO_CHANNEL_7)
	{
		printf("SDIO static-7 touch!\r\n");

		PCI->PCI_DETSLR |= PCI_SDIO_CHANNEL_7;
	}

	//NVRAM flag
	if((flag & PCI_NVRAM_FLAG) == PCI_NVRAM_FLAG)
	{
		printf("nvram clean!\r\n");
		PCI->PCI_DETSLR |= PCI_NVRAM_FLAG;
	}
	
	PCI->PCI_DETSR |= ((0xFF << 8) | (0xFF << 16));
}

/*******************************************************************************
* Function Name  : PCI_SDIO_PullControl
* Description    : PCI SDIOú
* Input          : channel - SDIOͨţPCI_SDIO_STATIC_CHANNEL_0
*                  mode    - Pull up or Pull down
* Output         : None
* Return         : None
******************************************************************************/
void PCI_SDIO_PullControl(unsigned char channel, unsigned char mode)
{
	if(mode == SDIO_PULL_UP)
	{
		PCI->PCI_SDIOPCR &= ~(channel << 16);
		PCI->PCI_SDIOPCR |= (channel << 24);
	}
	else
	{
		PCI->PCI_SDIOPCR &= ~(channel << 24);
		PCI->PCI_SDIOPCR |= (channel << 16);
	}

	//enable
	PCI->PCI_SDIOPCR |= 0x01;
}

/*******************************************************************************
* Function Name  : PCI_SDIO_Current_Trim
* Description    : PCI SDIOܵTrimֵ
* Input          : channel - SDIOͨţPCI_SDIO_STATIC_CHANNEL_0
*                  mode    - Pull up or Pull down
* Output         : None
* Return         : None
******************************************************************************/
void PCI_SDIO_Current_Trim(unsigned char value)
{
	PCI->PCI_SDIOPCR &= 0xFFFFFFF1;

	PCI->PCI_SDIOPCR |= value << 1;
}


/*******************************************************************************
* Function Name  : SDIO_ConfigGpio
* Description    : SDIOóGPIO;
* Input          : - SDIO_PINx: EPORT Pinwhere x can be 0~7 to select the SDIO peripheral.
*                  - GpioDirGPIO   GPIO_OUTPUT  GPIO_INPUT
*
* Output         : None
* Return         : 0 - ɹ;other - ʧ
******************************************************************************/
int SDIO_ConfigGpio(SDIO_PINx GpioNo, uint8_t GpioDir)
{
	//uint32_t temp = 0x01;
	assert_param(IS_PCI_PINx(GpioNo));
	assert_param(IS_GPIO_DIR_BIT(GpioDir));

	if (GpioNo > SDIO_PIN7)
		return -1;

	//ʹPCI SDIOΪGPIO
	PCI->PCI_SDIOPCR |= (0x01 << 8);

	//PCI SDIOGPIOʱҪóΪģʽ
	PCI_SDIO_Current_Trim(SDIO_CURRENT_LEVEL_2);
	PCI_SDIO_PullControl(GpioNo, SDIO_PULL_DOWN);


	if (GpioDir == GPIO_INPUT)//EPORT_EPDDR
	{
		PCI->PCI_SDIOGPIOCR &= ~((1 << GpioNo) << 16);
	}
	else
	{
		PCI->PCI_SDIOGPIOCR |= ((1 << GpioNo) << 16);
	}

	return 0;
}

/*******************************************************************************
* Function Name  : SDIO_WriteGpioData
* Description    : SDIO_PINxӦŵĵƽ
* Input          : - SDIO_PINx: SDIO Pinwhere x can be 0~7 to select the SDIO peripheral.
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : 0: óɹ    otherʧ
******************************************************************************/
int SDIO_WriteGpioData(SDIO_PINx GpioNo, uint8_t bitVal)
{

	assert_param(IS_PCI_PINx(GpioNo));
	assert_param(IS_GPIO_BIT_ACTION(bitVal));

	if (GpioNo > SDIO_PIN7)
		return -1;

	PCI->PCI_SDIOGPIOCR |= ((1 << GpioNo) << 16);

	if (bitVal == Bit_SET)
	{
		PCI->PCI_SDIOGPIOCR |= (Bit_SET << GpioNo);
	}
	else
	{
		PCI->PCI_SDIOGPIOCR &= ~(Bit_SET << GpioNo);
	}

	return 0;
}

/*******************************************************************************
* Function Name  : SDIO_ReadGpioData
* Description    : ȡSDIO_PINxӦŵĵƽ
* Input          : - SDIO_PINx: SDIO Pinwhere x can be 0~7 to select the SDIO peripheral.
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ  -1ʧ
******************************************************************************/
int SDIO_ReadGpioData(SDIO_PINx GpioNo)
{
	int  bitstatus = 0x00;

	assert_param(IS_PCI_PINx(GpioNo));

	if (GpioNo > SDIO_PIN7)
		return -1;

	PCI->PCI_SDIOGPIOCR &= ~((1 << GpioNo) << 16);

	bitstatus = (PCI->PCI_SDIOGPIOCR >> 8);
	if (bitstatus & (Bit_SET << GpioNo))
	{
		return Bit_SET;
	}
	else
	{
		return Bit_RESET;
	}
}

/*******************************************************************************
* Function Name  : PCI_NVSRAM_WtiteByte
* Description    : ֽnvramд
* Input          : - addr: дݵĵַ.
*                  - datд
*
* Output         : None
* Return         : other - ɹ;0 - ʧ
******************************************************************************/
BOOL PCI_NVSRAM_WtiteByte(uint32_t addr, uint8_t dat)
{
	if((addr < SRAM_START_ADDR_M4) || (addr > SRAM_END_ADDR_M4))
		return FALSE;

	IO_WRITE8(addr, dat);

	return TRUE;
}

/*******************************************************************************
* Function Name  : PCI_NVSRAM_ReadByte
* Description    : ֽڶȡnvram
* Input          : - addr: ݵĵַ.
*                  - dat
*
* Output         : None
* Return         : other - ɹ;0 - ʧ
******************************************************************************/
BOOL PCI_NVSRAM_ReadByte(uint32_t addr, uint8_t *dat)
{
	if((addr < SRAM_START_ADDR_M4) || (addr > SRAM_END_ADDR_M4))
		return FALSE;

	*dat = IO_READ8(addr);

	return TRUE;
}

/*******************************************************************************
* Function Name  : PCI_Open
* Description    : PCIģOpen
*                  PCI_Initļ򻯰棬ִPCI_OpenܹTDVDSDIOȼĴܹдNVSRAMȫ־λ
*                  úʹõǰǣùPCI_Init 
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_Open(void)
{
	//enable register operation
	PCI_Init_Key();
	PCI->PCI_WRITEEN = 0xC0000020;//*

	PCI->PCI_RTCIER |= 0x01;
	while((PCI->PCI_RTCIER & 0x80000000) != 0x80000000);
	PCI->PCI_RTCIER &= ~(1 << 6);
}

/*******************************************************************************
* Function Name  : PCI_Close
* Description    : PCIģClose
* 				   رPCIģĶдܣPCI_InitPCI_Openӵ3uAģ
*                  ͬʱ޷ȡNVSRAMݣȡPCIĴ־λ
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void PCI_Close(void)
{
	PCI->PCI_RTCIER &= ~0x01;
}
