// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : pwm_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "pwm_drv.h"
#include "ccm_drv.h"
#include "debug.h"
#include "string.h"
#include "iomacros.h"

/*******************************************************************************
* Function Name  : PWM_OutputInit
* Description    : PWMʼ
* Input          : - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*                  - PWM_Prescaler: PWMԤƵֵ=PWM_Prescaler+1
*                  - PWM_CsrClock Source Selection
*                             ȡֵPWM_CLK_DIV_1PWM_CLK_DIV_16PWM_CLK_DIV_8PWM_CLK_DIV_4PWM_CLK_DIV_2
*                  - PWM_PeriodPWM Counter/Timer Loaded Value, data range : 65535~0 (Unit : 1 PWM clock cycle)
*                                One PWM cycle width = PWM_Period + 1
*                  - PWM_WidthPWM Comparator Register Value;data range : 65535~0 (Unit : 1 PWM clock cycle)
*                               used to determine PWM output duty ratio
*                               PWM_Width >= PWM_Period : PWM output is always high
*                               PWM_Width >= PWM_Period : PWM output high = (PWM_Width + 1) unit
*                               PWM_Width = 0 : PWM output high = 1 unit
*                  - PWM_CHxINV_FlgTimer  Inverter ON/OFF flag
*                                    1 = Inverter ON
*                                    0 = Inverter OFF
*
* Output         : None
* Return         : None
*******************************************************************************/
void PWM_OutputInit(uint8_t PWMx, uint8_t PWM_Prescaler, uint8_t PWM_Csr, uint16_t PWM_Period, uint16_t PWM_Width, uint8_t PWM_CHxINV_Flg)
{
	assert_param(IS_PWM_PORTx(PWMx));
	assert_param(IS_PWM_CLK_DIV(PWM_Csr));
	
	PWM_INPUT_OUTPUT_EN;

	switch(PWMx)
	{
		case PWM_PORT0:
			PWM->PWM_CR_CH0 &= ~CH0EN;

			PWM->PWM_CSR &= (~(7<<0));//Timer0 Clock Source Selection
			PWM->PWM_CSR |= ((PWM_Csr&0x07)<<0);
			PWM->PWM_PR_CP0 = (PWM_Prescaler&0xff);
			PWM->PWM_CR_CH0 |= CH0MOD;//1 = Auto-load Mode
			PWM->PWM_CMR0 = PWM_Width;
			PWM->PWM_CNR0 = PWM_Period;
			PWM->PWM_PCR_PDDR |= 0x1;
			PWM->PWM_PCR_PULLEN |= 0x1;
			PWM->PWM_PCR_PDR |= 0x01;
			PWM->PWM_CR_CH0 |= (PWM_CHxINV_Flg<<2);
			
			PWM->PWM_CR_CH0 |= CH0EN;
			break;
		case PWM_PORT1:
		
			PWM->PWM_CR_CH1 &= ~CH1EN;

			PWM->PWM_CSR &= (~(7<<4));//Timer1 Clock Source Selection
			PWM->PWM_CSR |= ((PWM_Csr&0x07)<<4);
			PWM->PWM_PR_CP0 = (PWM_Prescaler&0xff);
			PWM->PWM_CR_CH1 |= CH1MOD;//1 = Auto-load Mode
			PWM->PWM_CMR1 = PWM_Width;
			PWM->PWM_CNR1 = PWM_Period;
			PWM->PWM_PCR_PDDR |= 0x2;
			PWM->PWM_PCR_PULLEN |= 0x2;
			PWM->PWM_PCR_PDR |= 0x02;
			PWM->PWM_CR_CH1 |= (PWM_CHxINV_Flg<<2);
			
			PWM->PWM_CR_CH1 |= CH1EN;
			break;
		case PWM_PORT2:
			PWM->PWM_CR_CH2 &= ~CH2EN;
			
			PWM->PWM_CSR &= (~(7<<8));//Timer2 Clock Source Selection
			PWM->PWM_CSR |= (PWM_Csr&0x07)<<8;
			PWM->PWM_PR_CP1 = (PWM_Prescaler&0xff);
			PWM->PWM_CR_CH2 = CH2MOD;
			PWM->PWM_CMR2 = PWM_Width;
			PWM->PWM_CNR2 = PWM_Period;
			PWM->PWM_PCR_PDDR |= 0x4;
			PWM->PWM_PCR_PULLEN |= 0x4;
			PWM->PWM_PCR_PDR |= 0x04;
			PWM->PWM_CR_CH2 |= (PWM_CHxINV_Flg<<2);
			
			PWM->PWM_CR_CH2 |= CH2EN;
			break;
		case PWM_PORT3:
			PWM->PWM_CR_CH3 &= ~CH3EN;
			
			PWM->PWM_CSR &= (~(7<<12));//Timer3 Clock Source Selection
			PWM->PWM_CSR |= (PWM_Csr&0x07)<<12;
			PWM->PWM_PR_CP1 = (PWM_Prescaler&0xff);
			PWM->PWM_CR_CH3 = CH3MOD;
			PWM->PWM_CMR3 = PWM_Width;
			PWM->PWM_CNR3 = PWM_Period;
			PWM->PWM_PCR_PDDR |= 0x8;
			PWM->PWM_PCR_PULLEN |= 0x8;
			PWM->PWM_PCR_PDR |= 0x08;
//			PWM->PWM_CR_CH3 |= (PWM_CHxINV_Flg<<2);
			
			PWM->PWM_CR_CH3 |= CH3EN;
			break;
		default:
			break;
	}
}

/*******************************************************************************
* Function Name  : PWM_InputInit
* Description    : PWMʼɴж
* Input          : - - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*                  - PWM_Prescaler: PWMԤƵֵ
*                  - PWM_CsrClock Source Selection
*                             ȡֵPWM_CLK_DIV_1PWM_CLK_DIV_16PWM_CLK_DIV_8PWM_CLK_DIV_4PWM_CLK_DIV_2
*                  - PWM_PeriodPWM Counter/Timer Loaded Value, data range : 65535~0 (Unit : 1 PWM clock cycle)
*                                One PWM cycle width = PWM_Period + 1
*
* Output         : None
* Return         : None
*******************************************************************************/
void PWM_InputInit(uint8_t PWMx, uint8_t pwm_prescaler, uint8_t pwm_csr,uint16_t pwm_period)
{
	
	assert_param(IS_PWM_PORTx(PWMx));
	PWM_INPUT_OUTPUT_EN;
	
	switch(PWMx)
	{
		case PWM_PORT0:
			PWM->PWM_CR_CH0 &= ~CH0EN;

			PWM->PWM_CSR |= pwm_csr;
			PWM->PWM_PR_CP0 = pwm_prescaler;
			PWM->PWM_CR_CH0 |= CH0MOD;
			PWM->PWM_CNR0 = pwm_period;
			PWM->PWM_PCR_PDDR &= 0xe;
			PWM->PWM_CCR0_CH0 = (uint8_t)(CAPCHxEN|FL_IE|RL_IE);
			PWM->PWM_CR_CH0 |= CH0EN;
		NVIC_Init(3, 3, PWM0_IRQn, 2);
		 break;

		case PWM_PORT1:

			PWM->PWM_CR_CH1 &= ~CH1EN;

			PWM->PWM_CSR |= pwm_csr<<4;
			PWM->PWM_PR_CP0 = pwm_prescaler;
			PWM->PWM_CR_CH1 |= CH1MOD;
			PWM->PWM_CNR1 = pwm_period;
			PWM->PWM_PCR_PDDR &= 0xd;
			PWM->PWM_CCR0_CH1 = (uint8_t)(CAPCHxEN|FL_IE|RL_IE);

			PWM->PWM_CR_CH1 |= CH1EN;
			NVIC_Init(3, 3, PWM1_IRQn, 2);
		 break;

		case PWM_PORT2:
			PWM->PWM_CR_CH2 &= ~CH2EN;

			PWM->PWM_CSR |= pwm_csr<<8;
			PWM->PWM_PR_CP1 = pwm_prescaler;
			PWM->PWM_CR_CH2 |= CH2MOD;
			PWM->PWM_CNR2 = pwm_period;
			PWM->PWM_PCR_PDDR &= 0x0b;
			PWM->PWM_CCR1_CH2 = (uint8_t)(CAPCHxEN|FL_IE|RL_IE);

			PWM->PWM_CR_CH2 |= CH2EN;
		NVIC_Init(3, 3, PWM2_IRQn, 2);
		 break;
		case PWM_PORT3:
			PWM->PWM_CR_CH3 &= ~CH3EN;

			PWM->PWM_CSR |= pwm_csr<<12;
			PWM->PWM_PR_CP1 = pwm_prescaler;
			PWM->PWM_CR_CH3 |= CH3MOD;
			PWM->PWM_CNR3 = pwm_period;
			PWM->PWM_PCR_PDDR &= 0x07;
			PWM->PWM_CCR1_CH3 = (uint8_t)(CAPCHxEN|FL_IE|RL_IE);

			PWM->PWM_CR_CH3 |= CH3EN;
		NVIC_Init(3, 3, PWM3_IRQn, 2);
		 break;
		default:			
		 break;
	}
	
}

/*******************************************************************************
* Function Name  : PWM_ISR
* Description    : PWM_жϴ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void PWM_ISR(void)
{
	uint8_t  bitstatus = 0x00;

	//DelayMS(10);		//
	bitstatus = PWM->PWM_PCR_PDR;
	////printf(" %02x", PWM->PWM_PCR_PDR);
	
	if(PWM->PWM_CCR0_CH0&CAPIF)	//PORT0
	{
		if (bitstatus &(Bit_SET<<PWM_PORT0))	//ߵƽ
		{
			//printf("Enter PWM_PORT0_RISING interrupt.\r\n");
		}
		else	//͵ƽ
		{
			//printf("Enter PWM_PORT0_FALLING interrupt.\r\n");
		}		
		PWM->PWM_CCR0_CH0 |= CAPIF;
	}

	if(PWM->PWM_CCR0_CH1&CAPIF)	//PORT1
	{
		if (bitstatus &(Bit_SET<<PWM_PORT1))	//ߵƽ
		{
			//printf("Enter PWM_PORT1_RISING interrupt.\r\n");
		}
		else	//͵ƽ
		{
			//printf("Enter PWM_PORT1_FALLING interrupt.\r\n");
		}		
		PWM->PWM_CCR0_CH1 |= CAPIF;
	}

	if(PWM->PWM_CCR1_CH2&CAPIF)	//PORT2
	{
		if (bitstatus &(Bit_SET<<PWM_PORT2))	//ߵƽ
		{
			//printf("Enter PWM_PORT2_RISING interrupt.\r\n");
		}
		else	//͵ƽ
		{
			//printf("Enter PWM_PORT2_FALLING interrupt.\r\n");
		}		
		PWM->PWM_CCR1_CH2 |= CAPIF;
	}

	if(PWM->PWM_CCR1_CH3&CAPIF)	//PORT3
	{
		if (bitstatus &(Bit_SET<<PWM_PORT3))	//ߵƽ
		{
			//printf("Enter PWM_PORT3_RISING interrupt.\r\n");
		}
		else	//͵ƽ
		{
			//printf("Enter PWM_PORT3_FALLING interrupt.\r\n");
		}		
		PWM->PWM_CCR1_CH3 |= CAPIF;
	}
}

void PWM0_IRQHandler(void)
{
	PWM_ISR();
}


void PWM1_IRQHandler(void)
{
	PWM_ISR();
}


void PWM2_IRQHandler(void)
{
	PWM_ISR();
}


void PWM3_IRQHandler(void)
{
	PWM_ISR();
}



/*******************************************************************************
* Function Name  : PWM_Start
* Description    : PWM
* Input          : - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*
* Output         : None
* Return         : None
*******************************************************************************/
void PWM_Start(uint8_t PWMx)
{
	switch(PWMx)
	{
		case PWM_PORT0:
			PWM->PWM_CR_CH0 |= CH0EN;
			break;
		case PWM_PORT1:
			PWM->PWM_CR_CH1 |= CH1EN;
			break;
		case PWM_PORT2:
			PWM->PWM_CR_CH2 |= CH2EN;
			break;
		case PWM_PORT3:
			PWM->PWM_CR_CH3 |= CH3EN;
			break;
		default:
			break;
	}
}

/*******************************************************************************
* Function Name  : PWM_Stop
* Description    : PWMر
* Input          : - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*
* Output         : None
* Return         : None
*******************************************************************************/
void PWM_Stop(uint8_t PWMx)
{
	switch(PWMx)
	{
		case PWM_PORT0:
			PWM->PWM_CR_CH0 &= ~CH0EN;
			break;
		case PWM_PORT1:
			PWM->PWM_CR_CH1 &= ~CH1EN;
			break;
		case PWM_PORT2:
			PWM->PWM_CR_CH2 &= ~CH2EN;
			break;
		case PWM_PORT3:
			PWM->PWM_CR_CH3 &= ~CH3EN;
			break;
		default:
			break;
	}
}

/*******************************************************************************
* Function Name  : PWM_ConfigGpio
* Description    : PWMóGPIO;
* Input          : - PWMx: PWM channelwhere x can be 0, 1, 2 to select the PWM peripheral.
*                  - PWM_Port: 0,1,2,3
*                  - SPI_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*                  - PullUp_EnӦĽʹ
*                               PWM_PULLUP_EN:  enable
*                               PWM_PULLUP_DIS: disable
*
* Output         : None
* Return         : None
******************************************************************************/
int8_t PWM_ConfigGpio(uint8_t PWMx, uint8_t GPIO_Dir, uint8_t PullUp_En)
{
	uint8_t ch = PWMx;

	assert_param(IS_PWM_PULLUP_STA(PullUp_En));
	assert_param(IS_GPIO_DIR_BIT(GPIO_Dir));

	if (ch > PWM_PORT3)
		return -1;

	PWM_INPUT_OUTPUT_EN;

	if (GPIO_Dir == GPIO_OUTPUT)
	{
		PWM->PWM_PCR_PDDR |= GPIO_Dir<<ch;//output
	}
	else if (GPIO_Dir == GPIO_INPUT)
	{
		PWM->PWM_PCR_PDDR &= (~(GPIO_Dir<<ch));//input
	}

	PWM->PWM_PCR_PULLEN &= (~(1<<ch));
	PWM->PWM_PCR_PULLEN |= PullUp_En<<ch;

	return 0;
}

/*******************************************************************************
* Function Name  : PWM_ReadGpioData
* Description    : ȡPWMxӦŵĵƽ
* Input          : - PWMx: PWM channelwhere x can be 0, 1, 2 to select the PWM peripheral.
*                  - PWM_Port: 0,1,2,3
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ  -1fail
******************************************************************************/
int8_t PWM_ReadGpioData(uint8_t PWMx)
{
	int8_t  bitstatus = 0x00;

	if (PWMx > PWM_PORT3)
		return -1;

	bitstatus = PWM->PWM_PCR_PDR;

	if (bitstatus &(Bit_SET<<PWMx))
		bitstatus = Bit_SET;
	else
		bitstatus = Bit_RESET;

	return bitstatus;
}

/*******************************************************************************
* Function Name  : PWM_WriteGpioData
* Description    : PWMxӦŵĵƽ
* Input          : - PWMx: PWM channelwhere x can be 0, 1, 2 to select the PWM peripheral.
*                  - PWM_Port: 0,1,2,3
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : 0: óɹ    otherʧ
******************************************************************************/
int8_t PWM_WriteGpioData(uint8_t PWMx, uint8_t bitVal)
{
	assert_param(IS_GPIO_BIT_ACTION(bitVal));

	if (PWMx > PWM_PORT3)
		return -1;
	if (bitVal > 2)
		return -2;

	if (bitVal == Bit_SET)
		PWM->PWM_PCR_PDR |= (Bit_SET<<PWMx);
	else
		PWM->PWM_PCR_PDR &= (~(Bit_SET<<PWMx));

	return 0;
}

