// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : pwm_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef PWM_DRV_H_
#define PWM_DRV_H_

#include "pwm_reg.h"

#define PWM        ((PWM_TypeDef *)(PWM_BASE_ADDR))

//PWM_PORT DEFINITION
#define PWM_PORT0		0
#define PWM_PORT1		1
#define PWM_PORT2		2
#define PWM_PORT3		3

#define IS_PWM_PORTx(port) (((port) == PWM_PORT0) || \
                            ((port) == PWM_PORT1) || \
                            ((port) == PWM_PORT2) || \
                            ((port) == PWM_PORT3))


//PWM Clock divider
#define PWM_CLK_DIV_1   4
#define PWM_CLK_DIV_2   0
#define PWM_CLK_DIV_4   1
#define PWM_CLK_DIV_8   2
#define PWM_CLK_DIV_16  3
#define IS_PWM_CLK_DIV(div) (((div) == PWM_CLK_DIV_1) || \
                             ((div) == PWM_CLK_DIV_2) || \
                             ((div) == PWM_CLK_DIV_4) || \
                             ((div) == PWM_CLK_DIV_8) || \
                             ((div) == PWM_CLK_DIV_16))


#define PWM_PULLUP_EN   1
#define PWM_PULLUP_DIS  0

#define IS_PWM_PULLUP_STA(sta) (((sta) == PWM_PULLUP_EN) || \
                                 ((sta) == PWM_PULLUP_DIS))

/*******************************************************************************
* Function Name  : PWM_OutputInit
* Description    : PWMʼ
* Input          : - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*                  - PWM_Prescaler: PWMԤƵֵ
*                  - PWM_CsrClock Source Selection
*                             ȡֵPWM_CLK_DIV_1PWM_CLK_DIV_16PWM_CLK_DIV_8PWM_CLK_DIV_4PWM_CLK_DIV_2
*                  - PWM_PeriodPWM Counter/Timer Loaded Value, data range : 65535~0 (Unit : 1 PWM clock cycle)
*                                One PWM cycle width = PWM_Period + 1
*                  - PWM_WidthPWM Comparator Register Value;data range : 65535~0 (Unit : 1 PWM clock cycle)
*                               used to determine PWM output duty ratio
*                               PWM_Width >= PWM_Period : PWM output is always high
*                               PWM_Width >= PWM_Period : PWM output high = (PWM_Width + 1) unit
*                               PWM_Width = 0 : PWM output high = 1 unit
*                  - PWM_CHxINV_FlgTimer  Inverter ON/OFF flag
*                                    1 = Inverter ON
*                                    0 = Inverter OFF
*
* Output         : None
* Return         : None
*******************************************************************************/
extern void PWM_OutputInit(uint8_t PWMx, uint8_t PWM_Prescaler, uint8_t PWM_Csr, uint16_t PWM_Period,  uint16_t PWM_Width, uint8_t PWM_CHxINV_Flg);

/*******************************************************************************
* Function Name  : PWM_InputInit
* Description    : PWMʼɴж
* Input          : - - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*                  - PWM_Prescaler: PWMԤƵֵ
*                  - PWM_CsrClock Source Selection
*                             ȡֵPWM_CLK_DIV_1PWM_CLK_DIV_16PWM_CLK_DIV_8PWM_CLK_DIV_4PWM_CLK_DIV_2
*                  - PWM_PeriodPWM Counter/Timer Loaded Value, data range : 65535~0 (Unit : 1 PWM clock cycle)
*                                One PWM cycle width = PWM_Period + 1
*
* Output         : None
* Return         : None
*******************************************************************************/
extern void PWM_InputInit(uint8_t PWMx, uint8_t pwm_prescaler, uint8_t pwm_csr,uint16_t pwm_period);

/*******************************************************************************
* Function Name  : PWM_ISR
* Description    : PWMжϴ
* Input          : - None
*
* Output         : None
* Return         : None
*******************************************************************************/
extern void PWM_ISR(void);

/*******************************************************************************
* Function Name  : PWM_Start
* Description    : PWM
* Input          : - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*
* Output         : None
* Return         : None
*******************************************************************************/
extern void PWM_Start(uint8_t PWMx);

/*******************************************************************************
* Function Name  : PWM_Stop
* Description    : PWMر
* Input          : - PWMx : PWM channelwhere x can be 0, 1, 2, 3 to select the PWM peripheral.
*
* Output         : None
* Return         : None
*******************************************************************************/
extern void PWM_Stop(uint8_t PWMx);


/*******************************************************************************
* Function Name  : PWM_ConfigGpio
* Description    : PWMóGPIO;
* Input          : - PWMx: PWM channelwhere x can be 0, 1, 2 to select the PWM peripheral.
*
*                  - SPI_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*                  - PullUp_EnӦĽʹ
*                               PWM_PULLUP_EN:  enable
*                               PWM_PULLUP_DIS: disable
*
* Output         : None
* Return         : None
******************************************************************************/
int8_t PWM_ConfigGpio(uint8_t PWMx, uint8_t GPIO_Dir, uint8_t PullUp_En);


/*******************************************************************************
* Function Name  : PWM_ReadGpioData
* Description    : ȡPWMxӦŵĵƽ
* Input          : - PWMx: PWM channelwhere x can be 0, 1, 2 to select the PWM peripheral.
*                  - PWM_Port: 0,1,2,3
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ  -1fail
******************************************************************************/
int8_t PWM_ReadGpioData(uint8_t PWMx);

/*******************************************************************************
* Function Name  : PWM_WriteGpioData
* Description    : PWMxӦŵĵƽ
* Input          : - PWMx: PWM channelwhere x can be 0, 1, 2 to select the PWM peripheral.
*                  - PWM_Port: 0,1,2,3
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : 0: óɹ    otherʧ
******************************************************************************/
int8_t PWM_WriteGpioData(uint8_t PWMx, uint8_t bitVal);


#endif /* PWM_DRV_H_ */
