// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : rtc_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef RTC_DRV_H_
#define RTC_DRV_H_

#include "memmap.h"
#include "rtc_reg.h"

#define RTC		                      ((RTC_TypeDef *)(PMURTC_BASE_ADDR))

#define RTC_SEL_MASK                (0x03 << 8)
#define RTC_LOW_LEVEL_INT           (0x00 << 8)
#define RTC_RISING_EDGE_INT         (0x01 << 8)
#define RTC_FALLING_EDGE_INT        (0x02 << 8)
#define RTC_RISING_FALLING_EDGE_INT (0x03 << 8)

#define DAY_ALARM                    0x01
#define HOUR_ALARM                   0x02
#define MINUTE_ALARM                 0x03
#define SECOND_ALARM                 0x04
#define ALL_ALARM                    0x05

#define IS_RTC_ALARM_MOD(MOD) (((MOD) == DAY_ALARM) ||    \
                               ((MOD) == HOUR_ALARM) ||   \
                               ((MOD) == MINUTE_ALARM) || \
                               ((MOD) == SECOND_ALARM) || \
                               ((MOD) == ALL_ALARM))


#define	Day_intf	    0x00400000
#define	Hou_intf	    0x00200000
#define	Min_intf	    0x00100000
#define	Sec_intf	    0x00080000
#define	Ala_intf	    0x00040000
#define	CLK_1KHz_intf	0x00020000
#define	CLK_32KHz_intf	0x00010000


typedef struct _tm{
	// date and time components
	uint16_t day;
	uint8_t  hour;
	uint8_t  minute;
	uint8_t  second;
}tm;


extern uint32_t g_rtc_int_sta;

/*******************************************************************************
* Function Name  : RTC_ISR
* Description    : RTCжϴ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
extern void RTC_ISR(void);

/*******************************************************************************
* Function Name  : RTC_Init
* Description    : RTCʼ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
extern void RTC_Init(uint8_t clk_sel);

/*******************************************************************************
* Function Name  : RTC_SetTime
* Description    : RTCֵ
* Input          :  - ttmṹ
*                        t->day:RTCdayֵ
*                        t->hour:RTChourֵ
*                        t->minute:RTCminuteֵ
*                        t->second:RTCsecondֵ
*
* Output         : None
* Return         : None
******************************************************************************/
extern void RTC_SetTime(tm t);

/*******************************************************************************
* Function Name  : RTC_SetAlarm
* Description    : RTCֵ
* Input          :  - ttmṹ壬alarmֵ
*                   - AlarmModalarmģʽ
*                               DAY_ALARMday
*                               HOUR_ALARMhour
*                               MINUTE_ALARMminute
*                               SECOND_ALARMsecond
*                               ALL_ALARMdayhourminutesecond
*
* Output         : None
* Return         : None
******************************************************************************/
extern void RTC_SetAlarm(tm t, uint8_t AlarmMod);

/*******************************************************************************
* Function Name  : RTC_GetTime
* Description    : ȡRTCֵ
* Input          : None
*
* Output         : - ttmṹ壬
*                       t->day:ȡRTCdayֵ
*                       t->hour:ȡRTChourֵ
*                       t->minute:ȡRTCminuteֵ
*                       t->second:ȡRTCsecondֵ
* Return         : None
******************************************************************************/
extern void RTC_GetTime(tm *t);




#endif /* RTC_DRV_H_ */
