// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : spi_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#include "debug.h"
#include "edma_drv.h"
#include "spi_drv.h"


/*******************************************************************************
* Function Name  : SPI_StructInit
* Description    : Fills each SPI_InitStruct member with its default value.
* Input          : - SPI_InitStruct : pointer to a SPI_InitTypeDef structure
*                    which will be initialized.
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_StructInit(uint8_t mode,SPI_InitTypeDef* SPI_InitStruct)
{
	/*--------------- Reset SPI init structure parameters values -----------------*/
	/* Initialize the SPI_Direction member */
	SPI_InitStruct->SPI_Direction = SPI_Direction_2Lines_FullDuplex;

	/* initialize the SPI_Mode member */
	SPI_InitStruct->SPI_Mode = mode;

	/* initialize the SPI_DataSize member */
	SPI_InitStruct->SPI_DataSize = SPI_DataSize_8b;

	/* Initialize the SPI_CPOL member */
	SPI_InitStruct->SPI_CPOL = SPI_CPOL_Low;

	/* Initialize the SPI_CPHA member */
	SPI_InitStruct->SPI_CPHA = SPI_CPHA_1Edge;


	/* Initialize the SPI_BaudRatePrescaler member */
	SPI_InitStruct->SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;

	/* Initialize the SPI_FirstBit member */
	SPI_InitStruct->SPI_FirstBit = SPI_FirstBit_MSB;

}


/*******************************************************************************
* Function Name  : SPI_Init
* Description    : SPIʼ
*
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_InitStruct: pointer to a SPI_InitTypeDef structure that
*                    contains the configuration information for the specified
*                    SPI peripheral.
* Output         : None
* Return         : None
******************************************************************************/
void SPI_Init(SPI_TypeDef* SPIx, SPI_InitTypeDef* SPI_InitStruct)
{
	uint8_t tmpreg = 0;

	//clear possible errors
	SPIx->SPISRHW = 0xffff;

	//flush FIFO
	SPIx->SPITXFCR = SPITXFCR_TXFCLR_MASK;
	SPIx->SPIRXFCR = SPIRXFCR_RXFCLR_MASK;

	tmpreg = SPIx->SPIFR;
	tmpreg &= ~SPIFR_FMSZ_MASK;
	tmpreg |= SPI_InitStruct->SPI_DataSize;
	SPIx->SPIFR = tmpreg;

	/*---------------------------- SPIx CR1 Configuration ------------------------*/
	 /* Get the SPIx CR1 value */
	tmpreg = SPIx->SPICR1;
	/* Clear SPIE, SPE, SWOM, MSTR , MSTR, CPOL , CPHA , SSOE and LSBFirstbits */
	tmpreg &= SPICR1_CLEAR_MASK;
	tmpreg |= (uint8_t)(SPI_InitStruct->SPI_CPHA |
			         SPI_InitStruct->SPI_CPOL |
			         SPI_InitStruct->SPI_FirstBit |
			         SPI_InitStruct->SPI_Mode);
	/* Write to SPIx CR1 */
	SPIx->SPICR1 = tmpreg;


	if (SPI_InitStruct->SPI_Mode == SPI_Mode_Master)
	{
		SPIx->SPIBR = SPI_InitStruct->SPI_BaudRatePrescaler;

		SPIx->SPIPURD = 0x00;
		//SPI DATA Direction: ss is output  sck is output  MOSI is output MISO is input
		SPIx->SPIDDR = SPIDDR_SS_MASK | SPIDDR_SCK_MASK | SPIDDR_MOSI_MASK;
		//SS high
		SPIx->SPIPORT |= SPIPORT_SS_MASK;
	}
	else
	{
		SPIx->SPIPURD = 0x00;
		//SPI DATA Direction: ss is input  sck is input  MOSI is input MISO is output
		SPIx->SPIDDR = SPIDDR_MISO_MASK;
	}
	//Guard time
	SPIx->SPICR2 = 0x00;
	/* Activate the SPI mode*/
	SPIx->SPICR1 |= SPI_ENABLE;
}


/*******************************************************************************
* Function Name  : SPI_MasterSendData
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - Send: ݻ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterSendData(SPI_TypeDef* SPIx, uint8_t *Send, uint32_t Len)
{
	uint8_t ucTemp = 0;
	
	ucTemp = ucTemp;

	while(Len != 0)
	{
		while(((SPIx->SPISRHW)&(SPISR_TXFFULL_MASK)));
		SPIx->SPIDR = *Send++;
		while(!(0x80 & SPIx->SPISR));
		while(((SPIx->SPISRHW) & (SPISR_RXFEMP_MASK)));
		ucTemp = SPIx->SPIDR;
		Len--;
	}
}

/*******************************************************************************
* Function Name  : SPI_MasterFixedSendData
* Description    : SPIΪ豸ͳLenͬSend
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - Send: ͵
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterFixedSendData(SPI_TypeDef* SPIx, uint8_t Send, uint16_t Len)
{
	uint8_t ucTemp = 0;
	
	ucTemp = ucTemp;

	while(Len != 0)
	{

		while(((SPIx->SPISRHW)&(SPISR_TXFFULL_MASK)));
		SPIx->SPIDR = Send;
		while(!(0x80 & SPIx->SPISR));
		while(((SPIx->SPISRHW) & (SPISR_RXFEMP_MASK)));
		ucTemp = SPIx->SPIDR;
		Len--;
	}
}

/*******************************************************************************
* Function Name  : SPI_MasterReceiveData
* Description    : SPIΪ豸Դ豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*
*                  - LenҪݵĳ
*
* Output         : - Revc: ݻ
* Return         : None
******************************************************************************/
void SPI_MasterReceiveData(SPI_TypeDef* SPIx, uint8_t *Revc, uint16_t Len)
{
	uint8_t ucTemp = 0xa5;
	
	while(Len != 0)
	{
		while(((SPIx->SPISRHW)&(SPISR_TXFFULL_MASK)));
		SPIx->SPIDR = ucTemp;
		while(!(0x80 & SPIx->SPISR));
		while(((SPIx->SPISRHW) & (SPISR_RXFEMP_MASK)));

		*Revc++ = SPIx->SPIDR;
		Len--;
	}
}

/*******************************************************************************
* Function Name  : SPI_MasterSendData
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - Send: ݻ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_SlaveSendData(SPI_TypeDef* SPIx, uint8_t *Send, uint16_t Len)
{
	while(Len != 0)
	{

		while(((SPIx->SPISRHW)&(SPISR_TXFFULL_MASK)));
		SPIx->SPIDR = *Send++;
		while(!(0x80 & SPIx->SPISR));
		Len--;
	}
}

/*******************************************************************************
* Function Name  : SPI_SlaveReceiveData
* Description    : SPIΪ豸豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*
*                  - LenҪݵĳ
*
* Output         : - Revc: ݻ
* Return         : None
******************************************************************************/
void SPI_SlaveReceiveData(SPI_TypeDef* SPIx, uint8_t *Revc, uint16_t Len)
{

	while(Len != 0)
	{
		while(((SPIx->SPISRHW) & (SPISR_RXFEMP_MASK)));
		*Revc++ = SPIx->SPIDR;
		Len--;
	}
}


/*******************************************************************************
* Function Name  : SPI_Cmd
* Description    : Enables or disables the specified SPI peripheral.
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - NewState: new state of the SPIx peripheral.
*                    This parameter can be: ENABLE or DISABLE.
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_Cmd(SPI_TypeDef* SPIx, FunctionalState NewState)
{
	if (NewState != DISABLE)
	{
		/* Enable the selected SPI peripheral */
		SPIx->SPICR1 |= CR1_SPE_Set;
	}
	else
	{
		/* Disable the selected SPI peripheral */
		SPIx->SPICR1 &= CR1_SPE_Reset;
	}
}

/*******************************************************************************
* Function Name  : SPI_ConfigGpio
* Description    : SPIóGPIO;
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_PinSPIӦPINţȡֵSPI_MISOSPI_MOSISPI_SCKSPI_SS
*                  - SPI_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_ConfigGpio(SPI_TypeDef* SPIx, SPI_PINx SPI_Pin, uint8_t SPI_Dir)
{

	if (SPI_Dir == GPIO_OUTPUT)
	{
		//ֲTable 18-4
//		if((SPI_Pin == SPI_MISO) || (SPI_Pin == SPI_SS))//xukai20181209ʵӦУssܵΪGPIOSPIȻSPIܣSSʱSPICR2Ϊ0x01
		if(SPI_Pin == SPI_MISO)
		{
			SPIx->SPICR2 = 0x01;
		}
		else if((SPI_Pin == SPI_MOSI) || (SPI_Pin == SPI_SCK))
		{
			SPIx->SPICR1 = 0x10;
			SPIx->SPICR2 = 0x01;
		}

		SPIx->SPIDDR  |= (1<<SPI_Pin);//output
	}
	else if (SPI_Dir == GPIO_INPUT)
	{
		SPIx->SPIDDR &= (~(1<<SPI_Pin));//input
	}
}


/*******************************************************************************
* Function Name  : SPI_ReadGpioData
* Description    : ȡSPI_PinӦŵĵƽ
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_PinSPIӦPINţȡֵSPI_MISOSPI_MOSISPI_SCKSPI_SS
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ
******************************************************************************/
uint8_t SPI_ReadGpioData(SPI_TypeDef* SPIx, SPI_PINx SPI_Pin)
{
	uint8_t bitstatus = 0x00;

	bitstatus = SPIx->SPIPORT;

	if (bitstatus &(Bit_SET<<SPI_Pin))
		bitstatus = Bit_SET;
	else
		bitstatus = Bit_RESET;

	return bitstatus;
}

/*******************************************************************************
* Function Name  : SPI_WriteGpioData
* Description    : SPI_PinӦŵĵƽ
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_PinSPIӦPINţȡֵSPI_MISOSPI_MOSISPI_SCKSPI_SS
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_WriteGpioData(SPI_TypeDef* SPIx, SPI_PINx SPI_Pin, uint8_t bitVal)
{
	if (bitVal == Bit_SET)
		SPIx->SPIPORT |= (Bit_SET<<SPI_Pin);
	else
		SPIx->SPIPORT &= (~(Bit_SET<<SPI_Pin));
}


/*******************************************************************************
* Function Name  : SPI_MasterSendDataByEDMA
* Description    : SPIΪ豸Ͳ
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - sendAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterSendDataByEDMA(SPI_TypeDef* SPIx, uint32_t sendAddr, uint32_t Len)
{
	if (SPIx == SPI1)
		EDMA_CHx(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE3, EDMA_SPI1, Len);
	else if (SPIx == SPI2)
		EDMA_CHx(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE3, EDMA_SPI2, Len);
	else if (SPIx == SPI3)
		EDMA_CHx(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE3, EDMA_SPI3, Len);

	SPIx->SPITXFCR |= SPITXFCR_TXFCLR_MASK;		//FIFO
}
/*******************************************************************************
* Function Name  : SPI_MasterOnlySendDataByEDMA
* Description    : SPIΪ豸ݣıԴbuf
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - sendAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterOnlySendDataByEDMA(SPI_TypeDef* SPIx, uint32_t sendAddr, uint32_t Len)
{
	if (SPIx == SPI1)
		EDMA_CHx(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE2, EDMA_SPI1, Len);
	else if (SPIx == SPI2)
		EDMA_CHx(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE2, EDMA_SPI2, Len);
	else if (SPIx == SPI3)
		EDMA_CHx(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE2, EDMA_SPI3, Len);

	SPIx->SPITXFCR |= SPITXFCR_TXFCLR_MASK;		//FIFO
}
/*******************************************************************************
* Function Name  : SPI_MasterOnlySendSameDataByEDMA
* Description    : SPIΪ豸ݣıԴbuf
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - sendAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterOnlySendSameDataByEDMA(SPI_TypeDef* SPIx, uint32_t sendAddr, uint32_t Len)
{
	if (SPIx == SPI1)
		EDMA_CHx_SAMEDATA(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE2, EDMA_SPI1, Len);
	else if (SPIx == SPI2)
		EDMA_CHx_SAMEDATA(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE2, EDMA_SPI2, Len);
	else if (SPIx == SPI3)
		EDMA_CHx_SAMEDATA(EDMA_CH0, sendAddr, sendAddr, EDMAC_TTYPE2, EDMA_SPI3, Len);

	SPIx->SPITXFCR |= SPITXFCR_TXFCLR_MASK;		//FIFO
}
/*******************************************************************************
* Function Name  : SPI_MasterReceiveDataByEDMA
* Description    : SPIΪ豸Դ豸
* Input          : - SPIx: where x can be 1, 2, 3  to select the SPI peripheral.
*				   - recvAddr: ݻ׵ַ
*                  - LenҪݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterReceiveDataByEDMA(SPI_TypeDef* SPIx, uint32_t recvAddr, uint32_t Len)
{

	if (SPIx == SPI1)
		EDMA_CHx(EDMA_CH0, recvAddr, recvAddr, EDMAC_TTYPE3, EDMA_SPI1, Len);
	else if (SPIx == SPI2)
		EDMA_CHx(EDMA_CH0, recvAddr, recvAddr, EDMAC_TTYPE3, EDMA_SPI2, Len);
	else if (SPIx == SPI3)
		EDMA_CHx(EDMA_CH0, recvAddr, recvAddr, EDMAC_TTYPE3, EDMA_SPI3, Len);

	SPIx->SPIRXFCR |= SPIRXFCR_RXFCLR_MASK;  //FIFO
}
/*******************************************************************************
* Function Name  : SPI_MasterReceiveDataByEDMA
* Description    : SPIΪ豸Դ豸
* Input          : - SPIx: where x can be 1, 2, 3  to select the SPI peripheral.
*				   - recvAddr: ݻ׵ַ
*                  - LenҪݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterOnlyReceiveDataByEDMA(SPI_TypeDef* SPIx, uint32_t recvAddr, uint32_t Len)
{
	if (SPIx == SPI1)
		EDMA_CHx(EDMA_CH0, recvAddr, recvAddr, EDMAC_TTYPE1, EDMA_SPI1, Len);
	else if (SPIx == SPI2)
		EDMA_CHx(EDMA_CH0, recvAddr, recvAddr, EDMAC_TTYPE1, EDMA_SPI2, Len);
	else if (SPIx == SPI3)
		EDMA_CHx(EDMA_CH0, recvAddr, recvAddr, EDMAC_TTYPE1, EDMA_SPI3, Len);

	SPIx->SPIRXFCR |= SPIRXFCR_RXFCLR_MASK;  //FIFO
}
/*******************************************************************************
* Function Name  : SPI_EnableDMA
* Description    : SPI DMAܵʹ
* Input          : - SPIx: where x can be 1, 2, 3  to select the SPI peripheral.
*				   - enable: Ƿʹ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_EnableDMA(SPI_TypeDef* SPIx,bool enable)
{
	if(enable == TRUE)
	{
		SPIx->SPIDMACR = SPISR_TXDMAE_MASK|SPISR_RXDMAE_MASK;
	}
	else
	{
		SPIx->SPIDMACR &= (~(SPISR_TXDMAE_MASK|SPISR_RXDMAE_MASK));
	}
}
/*******************************************************************************
* Function Name  : SPI_StructInit_NFC
* Description    : Fills each SPI_InitStruct member with its default value.
* Input          : - SPI_InitStruct : pointer to a SPI_InitTypeDef structure
*                    which will be initialized.
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_StructInit_NFC(uint8_t mode,SPI_InitTypeDef* SPI_InitStruct)
{
	/*--------------- Reset SPI init structure parameters values -----------------*/
	/* Initialize the SPI_Direction member */
	SPI_InitStruct->SPI_Direction = SPI_Direction_2Lines_FullDuplex;

	/* initialize the SPI_Mode member */
	SPI_InitStruct->SPI_Mode = mode;

	/* initialize the SPI_DataSize member */
	SPI_InitStruct->SPI_DataSize = SPI_DataSize_8b;

	/* Initialize the SPI_CPOL member */
	SPI_InitStruct->SPI_CPOL = SPI_CPOL_Low;

	/* Initialize the SPI_CPHA member */
	SPI_InitStruct->SPI_CPHA = SPI_CPHA_1Edge;

	/* Initialize the SPI_BaudRatePrescaler member */
	SPI_InitStruct->SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;

	/* Initialize the SPI_FirstBit member */
	SPI_InitStruct->SPI_FirstBit = SPI_FirstBit_MSB;

}
