// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : spi_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef SPI_DRV_H_
#define SPI_DRV_H_

#include "spi_reg.h"
#include "memmap.h"

/* SPI Init structure definition */
typedef struct
{

  uint8_t SPI_Direction;
  uint8_t SPI_Mode;
  uint8_t SPI_DataSize;
  uint8_t SPI_CPOL;//SPIʱʱźSCLKĵƽ1:ʱߵƽ; 0:ʱ͵ƽ
  uint8_t SPI_CPHA;//SPISCLKڼؿʼ0:һؿʼ; 1:ڶؿʼ
  uint8_t SPI_BaudRatePrescaler;//SPI Baud Rate
  uint8_t SPI_FirstBit;//LSB-First Enable Bit
  uint8_t SPI_CRCPolynomial;

}SPI_InitTypeDef;


typedef enum
{
    SPI_MISO = 0,
    SPI_MOSI,
    SPI_SCK,
    SPI_SS
}SPI_PINx;

#define IS_SPI_PINx(pin) (((pin) == SPI_MISO) || \
                          ((pin) == SPI_MOSI) || \
                          ((pin) == SPI_SCK) || \
                          ((pin) == SPI_SS))


#define SPI1		((SPI_TypeDef *)(SPI1_BASE_ADDR))
#define SPI2		((SPI_TypeDef *)(SPI2_BASE_ADDR))
#define SPI3		((SPI_TypeDef *)(SPI3_BASE_ADDR))



#define IS_SPI_ALL_PERIPH(PERIPH) (((PERIPH) == SPI1) || \
		                           ((PERIPH) == SPI2) || \
                                   ((PERIPH) == SPI3))

/* SPI master/slave mode */
#define SPI_Mode_Master                 ((uint8_t)0x10)
#define SPI_Mode_Slave                  ((uint8_t)0x00)

#define IS_SPI_MODE(MODE) (((MODE) == SPI_Mode_Master) || \
                           ((MODE) == SPI_Mode_Slave))


/* SPI data direction mode */
#define SPI_Direction_2Lines_FullDuplex ((uint8_t)0x00)
#define SPI_Direction_1Line_RxOrTx      ((uint8_t)0x01)


#define IS_SPI_DIRECTION_MODE(MODE) (((MODE) == SPI_Direction_2Lines_FullDuplex) || \
                                     ((MODE) == SPI_Direction_1Line_RxOrTx))

/* SPI data size */
#define SPI_DataSize_16b             (0x0f)
#define SPI_DataSize_8b              (0x07)


#define IS_SPI_DATASIZE(DATASIZE) (((DATASIZE) == SPI_DataSize_16b) || \
                                   ((DATASIZE) == SPI_DataSize_8b))


/* SPI Clock Polarity */
#define SPI_CPOL_Low                    ((uint8_t)0x00)
#define SPI_CPOL_High                   ((uint8_t)0x08)

#define IS_SPI_CPOL(CPOL) (((CPOL) == SPI_CPOL_Low) || \
                           ((CPOL) == SPI_CPOL_High))

/* SPI Clock Phase */
#define SPI_CPHA_1Edge                  ((uint16_t)0x00)
#define SPI_CPHA_2Edge                  ((uint16_t)0x04)

#define IS_SPI_CPHA(CPHA) (((CPHA) == SPI_CPHA_1Edge) || \
                           ((CPHA) == SPI_CPHA_2Edge))

/* SPI BaudRate Prescaler  */
#define SPI_BaudRatePrescaler_2         (0x00)
#define SPI_BaudRatePrescaler_4         (0x01)
#define SPI_BaudRatePrescaler_6         (0x20)
#define SPI_BaudRatePrescaler_8         (0x02)
#define SPI_BaudRatePrescaler_10        (0x40)
#define SPI_BaudRatePrescaler_20        (0x41)
#define SPI_BaudRatePrescaler_40        (0x42)
#define SPI_BaudRatePrescaler_64        (0x05)
#define SPI_BaudRatePrescaler_256       (0x07)


#define IS_SPI_BAUDRATE_PRESCALER(PRESCALER) (((PRESCALER) == SPI_BaudRatePrescaler_2) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_4) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_6) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_8) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_10) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_20) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_40) || \
                                              ((PRESCALER) == SPI_BaudRatePrescaler_64))

/* SPI MSB/LSB transmission */
#define SPI_FirstBit_MSB                ((uint8_t)0x00)
#define SPI_FirstBit_LSB                ((uint8_t)0x01)

#define IS_SPI_FIRST_BIT(BIT) (((BIT) == SPI_FirstBit_MSB) || \
                               ((BIT) == SPI_FirstBit_LSB))


#define CR1_SPE_Set    ((uint8_t)0x40)
#define CR1_SPE_Reset  ((uint8_t)0xBF)

#define SPI_CS_L(x)		    do{x->SPIPORT &= 0xF7;}while(0)
#define SPI_CS_H(x)		    do{x->SPIPORT |= 0x08;}while(0)

//#define SPI1_CS_L		    do{SPI1->SPIPORT &= 0xF7;}while(0)
//#define SPI1_CS_H		    do{SPI1->SPIPORT |= 0x08;}while(0)

//#define SPI2_CS_L		    do{SPI2->SPIPORT &= 0xF7;}while(0)
//#define SPI2_CS_H		    do{SPI2->SPIPORT |= 0x08;}while(0)

//#define SPI3_CS_L		    do{SPI3->SPIPORT &= 0xF7;}while(0)
//#define SPI3_CS_H		    do{SPI3->SPIPORT |= 0x08;}while(0)

#define SPISR_TXDMAE_MASK      (0x02)
#define SPISR_RXDMAE_MASK      (0x01)

/*******************************************************************************
* Function Name  : SPI_StructInit
* Description    : Fills each SPI_InitStruct member with its default value.
* Input          : - SPI_InitStruct : pointer to a SPI_InitTypeDef structure
*                    which will be initialized.
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_StructInit(uint8_t mode,SPI_InitTypeDef* SPI_InitStruct);

/*******************************************************************************
* Function Name  : SPI_Init
* Description    : SPIʼ
*
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_InitStruct: pointer to a SPI_InitTypeDef structure that
*                    contains the configuration information for the specified
*                    SPI peripheral.
* Output         : None
* Return         : None
******************************************************************************/
void SPI_Init(SPI_TypeDef* SPIx, SPI_InitTypeDef* SPI_InitStruct);

/*******************************************************************************
* Function Name  : SPI_Cmd
* Description    : Enables or disables the specified SPI peripheral.
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - NewState: new state of the SPIx peripheral.
*                    This parameter can be: ENABLE or DISABLE.
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_Cmd(SPI_TypeDef* SPIx, FunctionalState NewState);

/*******************************************************************************
* Function Name  : SPI_MasterSendData
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - Send: ݻ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterSendData(SPI_TypeDef* SPIx, uint8_t *Send, uint32_t Len);

/*******************************************************************************
* Function Name  : SPI_MasterFixedSendData
* Description    : SPIΪ豸ͳLenͬSend
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - Send: ͵
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterFixedSendData(SPI_TypeDef* SPIx, uint8_t Send, uint16_t Len);

/*******************************************************************************
* Function Name  : SPI_MasterReceiveData
* Description    : SPIΪ豸Դ豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*
*                  - LenҪݵĳ
*
* Output         : - Revc: ݻ
* Return         : None
******************************************************************************/
void SPI_MasterReceiveData(SPI_TypeDef* SPIx, uint8_t *Recv, uint16_t Len);

/*******************************************************************************
* Function Name  : SPI_MasterSendData
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - Send: ݻ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_SlaveSendData(SPI_TypeDef* SPIx, uint8_t *Send, uint16_t Len);

/*******************************************************************************
* Function Name  : SPI_SlaveReceiveData
* Description    : SPIΪ豸豸
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*
*                  - LenҪݵĳ
*
* Output         : - Revc: ݻ
* Return         : None
******************************************************************************/
void SPI_SlaveReceiveData(SPI_TypeDef* SPIx, uint8_t *Revc, uint16_t Len);


/*******************************************************************************
* Function Name  : SPI_ConfigGpio
* Description    : SPIóGPIO;
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_PinSPIӦPINţȡֵSPI_MISOSPI_MOSISPI_SCKSPI_SS
*                  - SPI_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_ConfigGpio(SPI_TypeDef* SPIx, SPI_PINx SPI_Pin, uint8_t SPI_Dir);

/*******************************************************************************
* Function Name  : SPI_ReadGpioData
* Description    : ȡSPI_PinӦŵĵƽ
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_PinSPIӦPINţȡֵSPI_MISOSPI_MOSISPI_SCKSPI_SS
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ
******************************************************************************/
uint8_t SPI_ReadGpioData(SPI_TypeDef* SPIx, SPI_PINx SPI_Pin);

/*******************************************************************************
* Function Name  : SPI_WriteGpioData
* Description    : SPI_PinӦŵĵƽ
* Input          : - SPIx: where x can be 1, 2  to select the SPI peripheral.
*                  - SPI_PinSPIӦPINţȡֵSPI_MISOSPI_MOSISPI_SCKSPI_SS
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_WriteGpioData(SPI_TypeDef* SPIx, SPI_PINx SPI_Pin, uint8_t bitVal);

/*******************************************************************************
* Function Name  : SPI_MasterReceiveDataByEDMA
* Description    : SPIΪ豸Ͳ
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*				   - recvAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterReceiveDataByEDMA(SPI_TypeDef* SPIx, uint32_t recvAddr, uint32_t Len);

/*******************************************************************************
* Function Name  : SPI_MasterOnlyReceiveDataByEDMA
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - recvAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterOnlyReceiveDataByEDMA(SPI_TypeDef* SPIx, uint32_t recvAddr, uint32_t Len);


/*******************************************************************************
* Function Name  : SPI_MasterSendDataByEDMA
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - sendAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterSendDataByEDMA(SPI_TypeDef* SPIx, uint32_t sendAddr, uint32_t Len);

/*******************************************************************************
* Function Name  : SPI_MasterOnlySendDataByEDMA
* Description    : SPIΪ豸
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - sendAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterOnlySendDataByEDMA(SPI_TypeDef* SPIx, uint32_t sendAddr, uint32_t Len);

/*******************************************************************************
* Function Name  : SPI_MasterOnlySendSameDataByEDMA
* Description    : SPIΪ豸ݣıԴbuf
* Input          : - SPIx: where x can be 1, 2 ,3 to select the SPI peripheral.
*                  - sendAddr: ݻ׵ַ
*                  - Lenݵĳ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_MasterOnlySendSameDataByEDMA(SPI_TypeDef* SPIx, uint32_t sendAddr, uint32_t Len);

/*******************************************************************************
* Function Name  : SPI_EnableDMA
* Description    : SPI DMAܵʹ
* Input          : - SPIx: where x can be 1, 2, 3  to select the SPI peripheral.
*				   - enable: Ƿʹ
*
* Output         : None
* Return         : None
******************************************************************************/
void SPI_EnableDMA(SPI_TypeDef* SPIx,bool enable);

/*******************************************************************************
* Function Name  : SPI_StructInit_NFC
* Description    : Fills each SPI_InitStruct member with its default value.
* Input          : - SPI_InitStruct : pointer to a SPI_InitTypeDef structure
*                    which will be initialized.
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_StructInit_NFC(uint8_t mode,SPI_InitTypeDef* SPI_InitStruct);

#endif /* SPI_DRV_H_ */
