// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : ssi_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#include "delay.h"
#include "debug.h"
#include "common.h"
#include "ssi_drv.h"
#include "dmac_drv.h"
#include "delay.h"

volatile int dma_int =0;

/*******************************************************************************
* Function Name  : SSI_Standard_Init
* Description    : SSI Standard ʼ
* Input          : - SSIx: SSI ַ
*
* Output         : None
* Return         : None
******************************************************************************/
void SSI_Standard_Init(SSI_TypeDef *SSIx)
{
	while(SSIx->SSI_SR&SR_BUSY){;}

	SSIx->SSI_SSIENR = 0x00;
	SSIx->SSI_CTRLR1 = 0x00;
	SSIx->SSI_CTRLR0 =  0x07;
	SSIx->SSI_BAUDR = 0x06;
	SSIx->SSI_TXFTLR = 0x00;
	SSIx->SSI_RXFTLR = 0x00;
	SSIx->SSI_SPI_CTRLR0 = 0x70000000;
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}	

/*******************************************************************************
* Function Name  : SSI_Standard_Init
* Description    : SSI Standard ʼ
* Input          : - SSIx: SSI ַ
*
* Output         : None
* Return         : None
******************************************************************************/
void SSI_STD_Init(SSI_TypeDef *SSIx)
{
	//-----------------DUAL MODE-----------------------
	SSIx->SSI_SSIENR = 0x00;	 
	SSIx->SSI_BAUDR =  0X06;
//	SSIx->SSI_CTRLR1 = ((data_len == 0)?0:(data_len-1));
	SSIx->SSI_CTRLR1 = 0;

	SSIx->SSI_CTRLR0 = 07;
	SSIx->SSI_SPI_CTRLR0 = 0;
		
	SSIx->SSI_CTRLR0 |= (CTRLR0_SPI_FRF_STD|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(0x07)|CTRLR0_TMOD_TX_AND_RX);
	SSIx->SSI_SPI_CTRLR0|= ( CTRLR0_TRANS_TYPE_TT0  |CTRLR0_INST_L_VALUE(2)               		//CMD_Len: 0=0bit1=4bit2=8bit, 3=16bit
													|CTRLR0_WAIT_CYCLES_VALUE(0)      		    //dummy_clock max=31 
													|CTRLR0_CLK_STRETCH_EN_MASK );
	
	SSIx->SSI_SPI_CTRLR0|=CTRLR0_ADDR_L_VALUE(0x06);     // 3 Byte addr     //addr_len= n*4 bit ( max_n=15)
	
//	SSIx->SSI_TXFTLR = 0x00010000;
//	SSIx->SSI_RXFTLR = 0x7;
	SSIx->SSI_TXFTLR = 0x00;
	SSIx->SSI_RXFTLR = 0x00;
	
//	if(QUAD_BAUDR==0x02&&g_sys_clk>=80000000)//high speed
//	{
//	 SSIx->SSI_RX_SAMPLE_DELAY=0x00010001;
//	}
	
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}

/*******************************************************************************
* Function Name  : SSI_EFlash_QPI_Enter
* Description    : SSI EFlash QPIģʽ
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QPI_Enter(SSI_TypeDef *SSIx)
{
	volatile unsigned char status;
	volatile unsigned char temp;
	
	if(SSIx==SSI2)	
	{
	  rSWAPCR|=0x18;
	}	
	
	SSI_Standard_Init(SSIx);
	
	SSIx->SSI_DR=QPI_ENTER_CMD;

	__asm("nop"); __asm("nop"); __asm("nop");
	

	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
	while(SSIx->SSI_SR&SR_BUSY){;}
		
	while(SSIx->SSI_SR&SR_RFNE)
	{
			temp=SSIx->SSI_DR;
	}
	
	do
	{
		status=SSI_EFlash_QPI_Get_Status1(SSIx);		 
	}while(status&0x01);
	
}

/*******************************************************************************
* Function Name  : SSI_EFlash_QPI_EXIT
* Description    : SSI EFlash ˳QPIģʽ
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QPI_EXIT(SSI_TypeDef *SSIx)
{
	volatile unsigned char status;
	volatile unsigned char temp;
	
	SSI_QPI_Init(SSIx, CMD_WRITE,1,2);
	
	SSIx->SSI_DR=QPI_EXIT_CMD;

	__asm("nop"); __asm("nop"); __asm("nop");
	
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
	while(SSIx->SSI_SR&SR_BUSY){;}
	
	SSI_Standard_Init(SSIx);
	do
	{
		status=SSI_EFlash_Get_Status1(SSIx);		 
	}while(status&0x01);
	
}

void SSI_DUAL_Init(SSI_TypeDef *SSIx, uint32_t data_len, uint8_t waitCycles)
{
	SSIx->SSI_SSIENR = 0x00;	 
	SSIx->SSI_BAUDR =  QUAD_BAUDR;
	SSIx->SSI_CTRLR1 = data_len-1;
		
	SSIx->SSI_CTRLR0 = 07;
	SSIx->SSI_SPI_CTRLR0 = 0;
		
	SSIx->SSI_CTRLR0 |= (CTRLR0_SPI_FRF_DUAL|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(0x07)|CTRLR0_TMOD_RX_ONLY);
	SSIx->SSI_SPI_CTRLR0|= ( CTRLR0_TRANS_TYPE_TT0
													|CTRLR0_ADDR_L_VALUE(0x06)
													|CTRLR0_INST_L_VALUE(0x02)
													|CTRLR0_WAIT_CYCLES_VALUE(waitCycles)
													|CTRLR0_CLK_STRETCH_EN_MASK);
	SSIx->SSI_TXFTLR = 0x00010000;
	SSIx->SSI_RXFTLR = 0x7;	
	
	if(QUAD_BAUDR==0x02&&g_sys_clk>=80000000)//high speed
	{
	 SSIx->SSI_RX_SAMPLE_DELAY=0x00010001;
	}
	
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}

/*******************************************************************************
* Function Name  : SSI_QUAD_Init
* Description    : SSI QUAD ʼ
* Input          : - SSIx      : SSI ַ
*								 - read      д־
*								 - num       ֡
*								 - waitCyclesȴʱ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_QUAD_Init(SSI_TypeDef *SSIx, int read, int num, int waitCycles)
{
	while(SSIx->SSI_SR&SR_BUSY){;}
	
	 SSIx->SSI_SSIENR = 0x00;
		
	 SSIx->SSI_CTRLR1 = ((num == 0)?0:(num-1));
	 SSIx->SSI_BAUDR =  0x50;

	 if(read)
	 {
//	    SSIx->SSI_CTRLR0 = 0x00800807;
	    SSIx->SSI_CTRLR0 =(CTRLR0_SPI_FRF_QUAD|CTRLR0_SSTE_MASK(0)|CTRLR0_TMOD_RX_ONLY|
							CTRLR0_SCPOL_MASK(0)|CTRLR0_SCPH_MASK(0)|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(7));
//	    SSIx->SSI_TXFTLR = 0x00010000;
//		SSIx->SSI_RXFTLR = 0x07;
	    SSIx->SSI_TXFTLR = TXFTLR_TFT(0)|TXFTLR_TXFTHR(1);
		SSIx->SSI_RXFTLR = RXFTLR_TFT(7);	 
	 }
	else
	{		 
//	    SSIx->SSI_CTRLR0 = 0x00800407;
	    SSIx->SSI_CTRLR0 =(CTRLR0_SPI_FRF_QUAD|CTRLR0_SSTE_MASK(0)|CTRLR0_TMOD_TX_ONLY|
							CTRLR0_SCPOL_MASK(0)|CTRLR0_SCPH_MASK(0)|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(7));   
	    SSIx->SSI_TXFTLR = TXFTLR_TFT(0)|TXFTLR_TXFTHR(1);
		SSIx->SSI_RXFTLR = RXFTLR_TFT(1);
	}	 

//	if( SSIx->SSI_BAUDR==0x02&&g_sys_clk>=80000000)//high speed
	{
//	 SSIx->SSI_RX_SAMPLE_DELAY=0x00010001;
	 SSIx->SSI_RX_SAMPLE_DELAY=RXSDR_SE(1)|RXSDR_RSD(1);
	}
//    SSIx->SSI_SPI_CTRLR0 =  0x40000218|(waitCycles<<11);
    SSIx->SSI_SPI_CTRLR0 = CTRLR0_TRANS_TYPE_TT0|CTRLR0_CLK_STRETCH_EN_MASK|                                     //ʱչʱҪ
						   CTRLR0_INST_L_VALUE(2)|CTRLR0_ADDR_L_VALUE(6)|CTRLR0_WAIT_CYCLES_VALUE(waitCycles);

	
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}


/*******************************************************************************
* Function Name  : SSI_QPI_Init
* Description    : SSI QUAD ʼ (byteΪ䵥λ)
* Input          : - SSIx      : SSI ַ
*								 - mode      ִģʽ
*								   CMD_READ		 Ͷָ޵ַλ
*								   CMD_WRITE	 дָ޵ַλ֧inst+addrģʽóinst+3Bݣóinst+addr(addr+data),ΪӲʱaddressеܣinst+3Bûеַλ3Bݼ
*								   DATA_READ	 Ͷָеַλ
*								   DATA_WRITE	 дָеַλ
*								 - data_len  ֡   (дΪ256 byte,Ϊ64k byte)
*								 - waitCyclesȴʱ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_QPI_Init(SSI_TypeDef *SSIx, QPI_OPT_MODE mode, uint32_t data_len, uint8_t waitCycles)
{
	SSIx->SSI_SSIENR = 0x00;	 
	SSIx->SSI_BAUDR =  QUAD_BAUDR;
	SSIx->SSI_CTRLR1 = data_len-1;
		
	SSIx->SSI_CTRLR0 = 07;
	SSIx->SSI_SPI_CTRLR0 = 0;
		
	if(mode == CMD_READ)
	{
		SSIx->SSI_CTRLR0 |= (CTRLR0_SPI_FRF_QUAD|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(0x07)|CTRLR0_TMOD_RX_ONLY);
		SSIx->SSI_SPI_CTRLR0|= ( CTRLR0_TRANS_TYPE_TT2
														|CTRLR0_ADDR_L_VALUE(0x00)
														|CTRLR0_INST_L_VALUE(0x02)
														|CTRLR0_WAIT_CYCLES_VALUE(waitCycles)
														|CTRLR0_CLK_STRETCH_EN_MASK);
		SSIx->SSI_TXFTLR = 0x00010000;
	  SSIx->SSI_RXFTLR = 0x07;
	}
	else if(mode == CMD_WRITE)
	{
		SSIx->SSI_CTRLR0 |= (CTRLR0_SPI_FRF_QUAD|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(0x07)|CTRLR0_TMOD_TX_ONLY);
		SSIx->SSI_SPI_CTRLR0|= ( CTRLR0_TRANS_TYPE_TT2
														|CTRLR0_ADDR_L_VALUE(0x00)
														|CTRLR0_INST_L_VALUE(0x02)
														|CTRLR0_WAIT_CYCLES_VALUE(waitCycles)
														|CTRLR0_CLK_STRETCH_EN_MASK);
		SSIx->SSI_TXFTLR = 0x00;
		SSIx->SSI_RXFTLR = 0x00;
	}
	else if(mode == DATA_READ)
	{
		SSIx->SSI_CTRLR0 |= (CTRLR0_SPI_FRF_QUAD|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(0x07)|CTRLR0_TMOD_RX_ONLY);
		SSIx->SSI_SPI_CTRLR0|= ( CTRLR0_TRANS_TYPE_TT2
														|CTRLR0_ADDR_L_VALUE(0x06)
														|CTRLR0_INST_L_VALUE(0x02)
														|CTRLR0_WAIT_CYCLES_VALUE(waitCycles)
														|CTRLR0_CLK_STRETCH_EN_MASK);
		SSIx->SSI_TXFTLR = 0x00010000;
	  SSIx->SSI_RXFTLR = 0x07;
	}
	else //DATA_WRITE
	{
		SSIx->SSI_CTRLR0 |= (CTRLR0_SPI_FRF_QUAD|CTRLR0_FRF_MOT|CTRLR0_DFS_VALUE(0x07)|CTRLR0_TMOD_TX_ONLY);
		SSIx->SSI_SPI_CTRLR0|= ( CTRLR0_TRANS_TYPE_TT2
														|CTRLR0_ADDR_L_VALUE(0x06)
														|CTRLR0_INST_L_VALUE(0x02)
														|CTRLR0_WAIT_CYCLES_VALUE(waitCycles)
														|CTRLR0_CLK_STRETCH_EN_MASK);
		SSIx->SSI_TXFTLR = 0x00020000;
		SSIx->SSI_RXFTLR = 0x00;
		SSIx->SSI_DMATDLR = 0x02;	
	}

	
	if(QUAD_BAUDR==0x02&&g_sys_clk>=80000000)//high speed
	{
	 SSIx->SSI_RX_SAMPLE_DELAY=0x00010001;
	}
	
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;
}

 /*******************************************************************************
* Function Name  : SSI_QUAD_Burst_Init
* Description    : SSI QUAD ʼ  (byteΪ䵥λBurst,䳤Ҫ4ı
* Input          : - SSIx      : SSI ַ
*								 - read      д־ 1:read  0:write
*								 - num       ֡   (дΪ256 byte,Ϊ64k byte)
*								 - waitCyclesȴʱ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_QUAD_Burst_Init(SSI_TypeDef *SSIx, int read, int num, int waitCycles)
{
	 while(SSIx->SSI_SR&SR_BUSY){;}
	
   SSIx->SSI_SSIENR = 0x00;
	 SSIx->SSI_CTRLR1 = num-1;
	 if(read)
	 {
	    SSIx->SSI_CTRLR0 = 0x00800807;
	 }
   else
   {		 
	    SSIx->SSI_CTRLR0 = 0x00800407;
	 }	 
	 SSIx->SSI_BAUDR =  QUAD_BAUDR;
	 if(read)
	 {
	    SSIx->SSI_TXFTLR = 0x00010000;
	    SSIx->SSI_RXFTLR = 0x07;	 
		  SSIx->SSI_DMARDLR= 0x03;		
	 }
   else
   {		 
	    SSIx->SSI_TXFTLR = 0x00020000;
	    SSIx->SSI_RXFTLR = 0x00;
	 }	 
	 if(QUAD_BAUDR==0x02&&g_sys_clk>=80000000)//high speed
	 {
		 SSIx->SSI_RX_SAMPLE_DELAY=0x00010000;
	 }
   SSIx->SSI_SPI_CTRLR0 = 0x40000218|(waitCycles<<11);
	 SSIx->SSI_IMR = 0x00;
	 SSIx->SSI_SSIENR = 0x01;
}


/*******************************************************************************
* Function Name  : SSI_DMA_ISR
* Description    : SSI жӦ
* Input          : None
* Output         : None
* Return         : None
******************************************************************************/
void SSI_DMA_ISR(void)
{
	if(m_dma_control->DMA_STATTFR&0x01) dma_int=1;
	if(m_dma_control->DMA_STATTFR&0x02) dma_int=2;
//	if(m_dma_control->DMA_STATTFR&0x04) dma_int=3;
//	if(m_dma_control->DMA_STATTFR&0x08) dma_int=4;

//	printf("=================DMA Channel %d ISR=============channel\r\n\r\n",dma_int-1);
//	printf("TFR: 0x%x\r\n",m_dma_control->DMA_STATTFR);
//	printf("BLOCK: 0x%x\r\n",m_dma_control->DMA_STATBLOCK);
//	printf("SRC: 0x%x\r\n",m_dma_control->DMA_STATSRC);
//	printf("DST: 0x%x\r\n",m_dma_control->DMA_STATDST);
//	printf("ERR: 0x%x\r\n",m_dma_control->DMA_STATERR);
//		
//	printf("Total int: 0x%x\r\n\r\n",m_dma_control->DMA_STATUSINT);

//	m_dma_control->DMA_CLRTFR = m_dma_control->DMA_STATTFR;
//	m_dma_control->DMA_CLRBLOCK=m_dma_control->DMA_STATBLOCK;
//	m_dma_control->DMA_CLRSRC=m_dma_control->DMA_STATSRC;
//	m_dma_control->DMA_CLRDST=m_dma_control->DMA_STATDST;
//	m_dma_control->DMA_CLRERR=m_dma_control->DMA_STATERR;
}


//#ifdef  SSI_DMA_INT
//#if 0
//void DMA_IRQHandler()
//{
//   printf("DMA0 IRQ!\r\n");
//	 isr_dma();
//}
//#endif

//void DMA2_IRQHandler()
//{
//   printf("DMA1 IRQ!\r\n");
//	 isr_dma();
//}
//#endif
	

unsigned char SSI_EFlash_QPI_Get_Status2(SSI_TypeDef *SSIx)
{ 
	volatile unsigned char retVal;
	
	SSI_QPI_Init(SSIx, CMD_READ , 1, 0);
		
	SSIx->SSI_DR=GET_SAT2_CMD;
	
	__asm("nop"); __asm("nop"); __asm("nop");
		
//	while((SSIx->SSI_SR&SR_BUSY)!=SR_BUSY){}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while(SSIx->SSI_SR&SR_BUSY){}
	
	 while(SSIx->SSI_SR&SR_RFNE)
	 {
			retVal=SSIx->SSI_DR;
	 }	
	
	 return retVal;
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_Prog_Status12 
* Description    : SSI Flash ״̬1״̬2 
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_Prog_Status12(SSI_TypeDef *SSIx, unsigned char val1, unsigned char val2)
{
	volatile unsigned char status;
	volatile unsigned char retVal;
	
	//SSI_EFlash_Write_Disable(SSIx);
	//SSI_EFlash_Write_Enable_for_Volatile(SSIx);

	SSI_EFlash_Write_Enable(SSIx);
	
	SSIx->SSI_DR=PROG_STA1_CMD;
	SSIx->SSI_DR=val1;
	SSIx->SSI_DR=val2;

	__asm("nop"); __asm("nop"); __asm("nop");

//	while((SSIx->SSI_SR&SR_BUSY)==0x00){;}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
		retVal=SSIx->SSI_DR;
	}	

	do
	{
	 status=SSI_EFlash_Get_Status1(SSIx);		 
	}while(status&1);
}
	
/*******************************************************************************
* Function Name  : SSI_EFlash_Set_Read_Para
* Description    : SSI EFlash ö
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_Set_Read_Para(SSI_TypeDef *SSIx)
{
	volatile unsigned char temp;

	SSIx->SSI_SSIENR = 0x00;
	SSIx->SSI_CTRLR1 = 0x00;
	SSIx->SSI_CTRLR0 = 0x00800407;
	SSIx->SSI_SPI_CTRLR0 = 0x40000202|(6<<11);
	SSIx->SSI_BAUDR =  0x10;
	SSIx->SSI_IMR = 0x00;
	SSIx->SSI_SSIENR = 0x01;

	SSIx->SSI_DR=SET_READ_PARA_CMD;
	SSIx->SSI_DR=0x21; //0x21;

	__asm("nop"); __asm("nop"); __asm("nop");
		
//	while((SSIx->SSI_SR&SR_BUSY)==0x00){;}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
		temp=SSIx->SSI_DR;
	}		
}		

/*******************************************************************************
* Function Name  : Read_ID_Test
* Description    : ȡSPIFlash ID  
* Input          : - SSIx: SSI ַ
* Return         : Flash ID
******************************************************************************/
uint16_t Read_ID(SSI_TypeDef *SSIx)
{
	int i;
	volatile unsigned short ID;

	SSIx->SSI_DR=READ_ID_CMD;
	SSIx->SSI_DR = 0x00;
	SSIx->SSI_DR = 0x00;
	SSIx->SSI_DR = 0x00;

	SSIx->SSI_DR=DUMMY_BYTE;
	SSIx->SSI_DR=DUMMY_BYTE;	

	__asm("nop");	__asm("nop");	__asm("nop");

	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
	while(SSIx->SSI_SR&SR_BUSY){;}

	for(i=0; i<4; i++)
	{
		ID = SSIx->SSI_DR;
	}

	ID = SSIx->SSI_DR;
	ID |= SSIx->SSI_DR << 8;
	
	return ID;
}		
uint16_t SSI_EFlash_Reset(SSI_TypeDef *SSIx)
{
	volatile unsigned short ID;

	SSIx->SSI_DR=ENABLE_RESET;

	__asm("nop");	__asm("nop");	__asm("nop");

	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
	while(SSIx->SSI_SR&SR_BUSY){;}
		
	SSIx->SSI_DR=RESET_DEVICE;
		
	__asm("nop");	__asm("nop");	__asm("nop");

	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
		ID=SSIx->SSI_DR;
	}
	
	return ID;
}
/*******************************************************************************
* Function Name  : SSI_EFlash_Read
* Description    : SSI Flash   
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_DUAL_Read(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{   
	volatile unsigned char temp;
	int i=0;
	
	SSI_DUAL_Init(SSIx,num,8);

	SSIx->SSI_DR = DUAL_READ_CMD;
	SSIx->SSI_DR = addr;
	
	__asm("nop"); __asm("nop"); __asm("nop");
	
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while( ((num>0)&&(SSIx->SSI_SR&SR_BUSY))||(SSIx->SSI_SR&SR_RFNE) )
	{
		if(SSIx->SSI_SR&SR_RFNE)
		{
				*(buf+i)=SSIx->SSI_DR;
				i++;
				num--;
		}	
	}	
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_Get_Status1
* Description    : ȡSSI Flash ״̬1 
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
unsigned char SSI_EFlash_Get_Status1(SSI_TypeDef *SSIx)
{ 
	volatile unsigned char retVal;

	SSIx->SSI_DR=GET_SAT1_CMD;
	SSIx->SSI_DR=DUMMY_BYTE;	
	
	__asm("nop"); __asm("nop"); __asm("nop");

//	while((SSIx->SSI_SR&SR_BUSY)!=SR_BUSY){;}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
		retVal=SSIx->SSI_DR;
	}	

	return retVal;
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_Get_Status2
* Description    : ȡSSI Flash ״̬2 
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
unsigned char SSI_EFlash_Get_Status2(SSI_TypeDef *SSIx)
{
	volatile unsigned char retVal;
		
	SSIx->SSI_DR=GET_SAT2_CMD;
	SSIx->SSI_DR=DUMMY_BYTE;
	
	__asm("nop"); __asm("nop"); __asm("nop");
		
//	while((SSIx->SSI_SR&SR_BUSY)!=SR_BUSY){}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while(SSIx->SSI_SR&SR_BUSY){}
	
	 while(SSIx->SSI_SR&SR_RFNE)
	 {
			retVal=SSIx->SSI_DR;
	 }	
	
	 return retVal;
}

/*******************************************************************************
* Function Name  : SSI_EFlash_Get_Status1
* Description    : ȡSSI Flash ״̬1 
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
unsigned char SSI_EFlash_QPI_Get_Status1(SSI_TypeDef *SSIx)
{ 
	volatile unsigned char retVal;
	
	SSI_QPI_Init(SSIx, CMD_READ , 1, 2);

	SSIx->SSI_DR=GET_SAT1_CMD;	
	
	__asm("nop"); __asm("nop"); __asm("nop");

//	while((SSIx->SSI_SR&SR_BUSY)!=SR_BUSY){;}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
		retVal=SSIx->SSI_DR;
	}	

	return retVal;
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_Write_Enable
* Description    : SSI Flashдʹ 
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_Write_Enable(SSI_TypeDef *SSIx)
{
	volatile unsigned char status;
	volatile unsigned char temp;
	
	if((SSI_EFlash_Get_Status1(SSIx)&0x02) == 0)
	{
		SSIx->SSI_DR=WRITE_EN_CMD;
	
		__asm("nop"); __asm("nop"); __asm("nop");
		
		while((SSIx->SSI_SR&SR_TFE)==0x00){;}
			
		while(SSIx->SSI_SR&SR_BUSY){;}
		
		while(SSIx->SSI_SR&SR_RFNE)
		{
				temp=SSIx->SSI_DR;//fifo
		}	
		
		do
		{
			status=SSI_EFlash_Get_Status1(SSIx);		 
		}while(status&0x01);
	}
}	
void SSI_EFlash_Write_Disable(SSI_TypeDef *SSIx)
{
	volatile unsigned char status;
	volatile unsigned char temp;
	
	if((SSI_EFlash_Get_Status1(SSIx)&0x02) == 0)
	{
		SSIx->SSI_DR=WRITE_DIS_CMD;
	
		__asm("nop"); __asm("nop"); __asm("nop");
		
		while((SSIx->SSI_SR&SR_TFE)==0x00){;}
			
		while(SSIx->SSI_SR&SR_BUSY){;}
		
		while(SSIx->SSI_SR&SR_RFNE)
		{
				temp=SSIx->SSI_DR;//fifo
		}	
		
		do
		{
			status=SSI_EFlash_Get_Status1(SSIx);		 
		}while(status&0x01);
	}
}	
void SSI_EFlash_Write_Enable_for_Volatile(SSI_TypeDef *SSIx)
{
	volatile unsigned char status;
	volatile unsigned char temp;
	
	if((SSI_EFlash_Get_Status1(SSIx)&0x02) == 0)
	{
		SSIx->SSI_DR=WRITE_EN_Volatile_CMD;
	
		__asm("nop"); __asm("nop"); __asm("nop");
		
		while((SSIx->SSI_SR&SR_TFE)==0x00){;}
			
		while(SSIx->SSI_SR&SR_BUSY){;}
		
		while(SSIx->SSI_SR&SR_RFNE)
		{
				temp=SSIx->SSI_DR;//fifo
		}	
		
		do
		{
			status=SSI_EFlash_Get_Status1(SSIx);		 
		}while(status&0x01);
	}
}
/*******************************************************************************
* Function Name  : SSI_EFlash_QPI_Write_Enable
* Description    : SSI Flashдʹ 
* Input          : - SSIx: SSI ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QPI_Write_Enable(SSI_TypeDef *SSIx)
{
	volatile unsigned char status;
	volatile unsigned char temp;
	
	if((SSI_EFlash_QPI_Get_Status1(SSIx)&0x02) == 0)
	{
		SSI_QPI_Init(SSIx, CMD_WRITE , 1, 2);
		
		SSIx->SSI_DR=WRITE_EN_CMD;
	
		__asm("nop"); __asm("nop"); __asm("nop");
		
		while((SSIx->SSI_SR&SR_TFE)==0x00){;}
			
		while(SSIx->SSI_SR&SR_BUSY){;}
		
		do
		{
			status=SSI_EFlash_QPI_Get_Status1(SSIx);		 
		}while(status&0x01);
	}
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_Sector_Erase
* Description    : SSI Flash 
* Input          : - SSIx: SSI ַ
* Input          : - addr: ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_Sector_Erase(SSI_TypeDef *SSIx, unsigned int addr)
{
	volatile unsigned char status;
	unsigned char temp;
	
	temp = temp;
	
	SSI_EFlash_Write_Enable(SSIx);

	SSIx->SSI_DR=SECT_ERASE_CMD;
	SSIx->SSI_DR=(addr>>16)&0xff;
	SSIx->SSI_DR=(addr>>8)&0xff;
	SSIx->SSI_DR=(addr>>0)&0xff;
	
	__asm("nop"); __asm("nop"); __asm("nop");
	
	while((SSIx->SSI_SR&SR_TFE)==0x00){;}
		
	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
			temp=SSIx->SSI_DR;//fifo
	}

	do
	{
	 status=SSI_EFlash_Get_Status1(SSIx);
	 //printf("status: 0x%02x\r\n", status);
	}while(status&0x01);
	 
}

/*******************************************************************************
* Function Name  : SSI_EFlash_QPI_Sector_Erase
* Description    : SSI Flash 
* Input          : - SSIx: SSI ַ
* Input          : - addr: ַ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QPI_Sector_Erase(SSI_TypeDef *SSIx, unsigned int addr)
{
	volatile unsigned char status;
	unsigned char temp;
	
	temp = temp;
	
	SSI_EFlash_QPI_Write_Enable(SSIx);
	
	SSI_QPI_Init(SSIx, CMD_WRITE , 3, 2);  

	SSIx->SSI_DR=SECT_ERASE_CMD;
	SSIx->SSI_DR=(addr>>16)&0xFF;
	SSIx->SSI_DR=(addr>>8)&0xFF;
	SSIx->SSI_DR=addr&0xFF;
	
	__asm("nop"); __asm("nop"); __asm("nop");
	
	while((SSIx->SSI_SR&SR_TFE)==0x00){;}
		
	while(SSIx->SSI_SR&SR_BUSY){;}

	do
	{
	 status=SSI_EFlash_QPI_Get_Status1(SSIx);
	 //printf("status: 0x%02x\r\n", status);
	}while(status&0x01);
	
	while(SSIx->SSI_SR&SR_BUSY){;}
	 
}

/*******************************************************************************
* Function Name  : SSI_EFlash_Program
* Description    : SSI Flash д  
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_Program(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{
	volatile unsigned char temp;
	volatile unsigned char status;
	
	SSI_EFlash_Write_Enable(SSIx);	

	SSIx->SSI_DR=PAGE_PROG_CMD;
	SSIx->SSI_DR=(addr>>16)&0xff;
	SSIx->SSI_DR=(addr>>8)&0xff;
	SSIx->SSI_DR=(addr>>0)&0xff;
	 
#ifdef INTERRUPT_TEST
	 SSIx->SSI_IMR=0x01;
#endif	 

	while(num>0)
	{	
	if(SSIx->SSI_SR&SR_TFNF)
	{
		 SSIx->SSI_DR=*buf;
		 buf++;
		 num--; 
	}	

	if(SSIx->SSI_SR&SR_RFNE)
	{
		 temp=SSIx->SSI_DR;
	}	
	}	


	while(SSIx->SSI_SR&SR_BUSY){;}
	 

	while(SSIx->SSI_SR&SR_RFNE)
	{
		 temp=SSIx->SSI_DR;
	}	
	 
	do
	{
	 status=SSI_EFlash_Get_Status1(SSIx);
	 //printf("[status]-2: 0x%02x\r\n", status);
	}while(status&0x01);
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_Read
* Description    : SSI Flash   
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_Read(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{   
	volatile unsigned char temp;
	int i=0,j=0;
	int txnum;

	txnum=num;
	SSIx->SSI_DR = READ_CMD;
	SSIx->SSI_DR = (addr>>16)&0xff;
	SSIx->SSI_DR = (addr>>8)&0xff;
	SSIx->SSI_DR = (addr>>0)&0xff;
	
	__asm("nop"); __asm("nop"); __asm("nop");
	
	while(num>0)
	{
		if( (SSIx->SSI_SR&SR_TFNF)&&(txnum>0) )
		{
			SSIx->SSI_DR=DUMMY_BYTE;
			txnum--;
		}	 
			 
		if(SSIx->SSI_SR&SR_RFNE)
		{
			if(j < 4)
			{
				temp=SSIx->SSI_DR;
				j++;
			}
			else
			{
				*(buf+i)=SSIx->SSI_DR;
				 i++;
				 num--;
			}
		}
	}

	while(SSIx->SSI_SR&SR_BUSY){;}
}	

/*******************************************************************************
* Function Name  : SSI_EFlash_QUAD_Program
* Description    : SSI Flash QUAD ʽд  
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QUAD_Program(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{
	volatile unsigned char temp;
	volatile unsigned char status;

	SSI_Standard_Init(SSIx);
	SSI_EFlash_Write_Enable(SSIx);	
	
	SSI_QUAD_Init(SSIx, 0, num, 8);


	SSIx->SSI_DR=QUAD_PROG_CMD;
	SSIx->SSI_DR=addr;

//	__asm("nop");__asm("nop");__asm("nop");
//	while((SSIx->SSI_SR&SR_BUSY)==0x00){;}
//	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}
//	while(SSIx->SSI_SR&SR_BUSY){;}

	while(num>0)
	{
		 if(SSIx->SSI_SR&SR_TFNF)
		 {
			 SSIx->SSI_DR=*buf;
			 buf++;
			 num--; 
		 }

//		 if(SSIx->SSI_SR&SR_RFNE)
//		 {
//			 temp=SSIx->SSI_DR;
//		 }	
	}	

	while(SSIx->SSI_SR&SR_BUSY){;}

	while(SSIx->SSI_SR&SR_RFNE)
	{
		 temp=SSIx->SSI_DR;
	}	

	SSI_Standard_Init(SSIx);
	do
	{
	 status=SSI_EFlash_Get_Status1(SSIx);
	 //printf("[status]-2: 0x%02x\r\n", status);
	}while(status&0x01);	
}

/*******************************************************************************
* Function Name  : SSI_EFlash_QUAD_Read
* Description    : SSI Flash QUAD ʽ  
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QUAD_Read(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{
	volatile unsigned char temp;
	volatile unsigned char status;
	int i=0;
	//	int dataFlag=0;

	SSI_QUAD_Init(SSIx,1, num, 8);
	
	SSIx->SSI_DR=QUAD_READ_CMD;
	SSIx->SSI_DR=addr;
	
	
	__asm("nop");__asm("nop");__asm("nop");
	
//	while((SSIx->SSI_SR&SR_BUSY)!= SR_BUSY){;}
	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

//	while( ((num>0)&&(SSIx->SSI_SR&SR_BUSY))||(SSIx->SSI_SR&SR_RFNE) )
	while( num>0 )
	{
//		printf("num =%d   SSIx->SSI_SR=0x %x  \r\n",num,SSIx->SSI_SR);
		if(SSIx->SSI_SR&SR_RFNE)
		{
				*(buf+i)=SSIx->SSI_DR;
				i++;
				num--;
				if(num == 0) 
					break;
		}	
	}	
	
    while(SSIx->SSI_SR&SR_BUSY){;}
}

/*******************************************************************************
* Function Name  : SSI_EFlash_QPI_Program
* Description    : SSI Flash QUAD ʽд  
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QPI_Program(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{
	volatile unsigned char temp;
	volatile unsigned char status;
	
	SSI_EFlash_QPI_Write_Enable(SSIx);
	SSI_QPI_Init(SSIx, DATA_WRITE, num, 2);
	
	SSIx->SSI_DR=PAGE_PROG_CMD;
	SSIx->SSI_DR=addr;

	while(num>0)
	{	
		if(SSIx->SSI_SR&SR_TFNF)
		{
		 SSIx->SSI_DR=*buf;
		 buf++;
		 num--; 
		}	
	}	

	while(SSIx->SSI_SR&SR_BUSY){;}

	do
	{
	 status=SSI_EFlash_QPI_Get_Status1(SSIx);
	 //printf("[status]-2: 0x%02x\r\n", status);
	}while(status&0x01);	
}

/*******************************************************************************
* Function Name  : SSI_EFlash_QPI_Read
* Description    : SSI Flash QUAD ʽ  
* Input          : - SSIx: SSI ַ
*								 - addrݵַ
*								 - buf ݻ
*								 - num ݳ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_EFlash_QPI_Read(SSI_TypeDef *SSIx, unsigned int addr, unsigned char *buf, int num)
{
//	volatile unsigned char temp;
//	volatile unsigned char status;
	int i=0;
	
	SSI_QPI_Init(SSIx, DATA_READ, num, 2);

	SSIx->SSI_DR=QPI_READ_CMD;
	SSIx->SSI_DR=addr;
	
	__asm("nop"); __asm("nop"); __asm("nop");

	while((SSIx->SSI_SR&SR_TFE)!=SR_TFE){;}

	while( ((num>0)&&(SSIx->SSI_SR&SR_BUSY))||(SSIx->SSI_SR&SR_RFNE) )
	{
		if(SSIx->SSI_SR&SR_RFNE)
		{
				*(buf+i)=SSIx->SSI_DR;
				i++;
				num--;
		}	
	}		 
}

/*******************************************************************************
* Function Name  : SSI_STD_DMA_Trig
* Description    : SSI STD DMA  
* Input          : - SSIx   : SSI ַ
*								 - cmd    ָ
*								 - addr   ַ
*								 - dmaConf䷽ʽ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_STD_DMA_Trig(SSI_TypeDef *SSIx, unsigned cmd, unsigned int addr, int dmaConf)
{	
	SSIx->SSI_DR=cmd;
	SSIx->SSI_DR=(addr>>16)&0xff;
	SSIx->SSI_DR=(addr>>8)&0xff;
	SSIx->SSI_DR=(addr>>0)&0xff;
	
//	__asm("nop"); __asm("nop"); __asm("nop");
//	
//	while((SSIx->SSI_SR&SR_TFE)==0x00){;}
//		
//	while(SSIx->SSI_SR&SR_BUSY){;}

//	while(SSIx->SSI_SR&SR_RFNE)
//	{
//			temp=SSIx->SSI_DR;//fifo
//	}

	SSIx->SSI_DMACR=dmaConf;
}	

/*******************************************************************************
* Function Name  : SSI_STD_DMA_Read
* Description    : spi dmaպ
* Input          : - SPIx: SPI ַ
*                - dma_ch: DMA channel
*				          - pread: ݵַ
*                  - addr: spi flashַ
*				        - length  ݳ
*				        - binten : Ƿжģʽ
*
* Output         : None
* Return         : None
******************************************************************************/
void SSI_STD_DMA_Read(SSI_TypeDef *SSIx,int dma_ch, uint8_t* pread, uint32_t addr, uint32_t length, BOOL binten)
{
    int num;

    m_dma_control->DMA_CONFIG = 1;
    //Rx
    m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)&SSIx->SSI_DR;
    m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)pread;
    if(binten == TRUE)
    {
        m_dma2_channel[dma_ch]->DMA_CTRL = INTEN|DIEC|SNOCHG|P2M_DMA|DWIDTH_B|SWIDTH_B;
    }
    else
    {
        m_dma2_channel[dma_ch]->DMA_CTRL = DIEC|SNOCHG|P2M_DMA|DWIDTH_B|SWIDTH_B;
    }
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = (length + 4);  //󳤶Ϊ0x0FFF

    m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_SRC_HARD);
    if(SSIx == SSI1)
    {
        m_dma2_channel[dma_ch]->DMA_CFG_HIGH = SRC_PER_SPI_RX(2);
    }
    else if(SSIx == SSI2)
    {
        m_dma2_channel[dma_ch]->DMA_CFG_HIGH = SRC_PER_SPI_RX(5);
    }
    else
    {
        printf("unknown SSIx\r\n");
    }
    //enable dma channel
    if(binten == TRUE)
    {
        m_dma_control->DMA_MASKTFR = CHANNEL_UMASK(dma_ch);
    }	
    m_dma_control->DMA_CHEN = (CHANNEL_WRITE_ENABLE(dma_ch)|CHANNEL_ENABLE(dma_ch));
    //printf("1m_dma_control->DMA_RAWTFR: 0x%08x\r\n", m_dma_control->DMA_RAWTFR);
	

    num=length;
    SSI_STD_DMA_Trig(SSIx, READ_CMD, addr, DMACR_RDMAE); 

    while(num)
    {
        if( (SSIx->SSI_SR&SR_TFNF)&&(num>0) )
        {	
            SSIx->SSI_DR=DUMMY_BYTE;
            num--;	
            while(0);
        }	
    }
    while(SSIx->SSI_SR&SR_BUSY){}

    if(TRUE == binten)
    {
        while(1)
        {
            if(dma_int == CHANNEL_STAT(dma_ch) )
            {
                dma_int = 0;
                break;
            }
        }			
    }	 
    else
    {			
        while((m_dma_control->DMA_RAWTFR & CHANNEL_STAT(dma_ch)) != CHANNEL_STAT(dma_ch)){} 		
        m_dma_control->DMA_CLRTFR =  m_dma_control->DMA_STATTFR;
    }	 

    while(m_dma_control->DMA_CHEN & CHANNEL_STAT(dma_ch)){}
		
    m_dma_control->DMA_CHEN = 0;
    m_dma_control->DMA_CONFIG = 0;
    SSIx->SSI_DMACR = 0;

}

/*******************************************************************************
* Function Name  : SSI_STD_DMA_Send
* Description    : spi dmaͺ
* Input          : - SPIx: SPI ַ
*                - dma_ch: DMA channel
*				          - psend: ݵַ
*                  _ addr: SPI flashַ
*				         - length ݳ
*				         - binten: Ƿжģʽ
*
* Output         : None
* Return         : None
******************************************************************************/
void SSI_STD_DMA_Send(SSI_TypeDef *SSIx,int dma_ch, uint8_t* psend, uint32_t addr, uint32_t length, BOOL binten)
{
    int num;
	
	m_dma_control->DMA_CONFIG = 1;
	//Tx
	m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)psend;
	m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)&SSIx->SSI_DR;
	if(binten == TRUE)
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = INTEN|DNOCHG|SIEC|M2P_DMA|DWIDTH_B|SWIDTH_B|SBSIZE_4|DBSIZE_4;
	}
	else
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = DNOCHG|SIEC|M2P_DMA|DWIDTH_B|SWIDTH_B|SBSIZE_4|DBSIZE_4;
	}
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = length ;  //󳤶Ϊ0x0FFF
	m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_DST_HARD);
	if(SSIx == SSI1)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(3);
	}
	else if(SSIx == SSI2)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(2);
	}
    else
    {
        printf("unknown SSIx\r\n");
    }
	//enable dma channel
	if(binten == TRUE)
	{	
	   m_dma_control->DMA_MASKTFR = CHANNEL_UMASK(dma_ch);
	}
	m_dma_control->DMA_CHEN = (CHANNEL_WRITE_ENABLE(dma_ch)|CHANNEL_ENABLE(dma_ch));
	//printf("1m_dma_control->DMA_RAWTFR: 0x%08x\r\n", m_dma_control->DMA_RAWTFR);
	
	SSI_STD_DMA_Trig(SSIx, PAGE_PROG_CMD, addr, DMACR_TDMAE);
	
	   num=length;
    while(num)
    {
        if( (SSIx->SSI_SR&SR_TFNF)&&(num>0) )
        {	
            SSIx->SSI_DR=DUMMY_BYTE;
            num--;	
            while(0);
        }
    }
    while(SSIx->SSI_SR&SR_BUSY){}
    if(TRUE == binten)
    {
        while(1)
        {
            if(dma_int == CHANNEL_STAT(dma_ch) )
            {
                dma_int = 0;
                break;
            }
        }			

    }	 
    else
    {
        while((m_dma_control->DMA_RAWTFR & CHANNEL_STAT(dma_ch)) != CHANNEL_STAT(dma_ch)){ }    		
        m_dma_control->DMA_CLRTFR =  m_dma_control->DMA_STATTFR;
    }	 

    while(m_dma_control->DMA_CHEN & CHANNEL_STAT(dma_ch)){}
    m_dma_control->DMA_CHEN = 0;
    m_dma_control->DMA_CONFIG = 0;
    SSIx->SSI_DMACR = 0;
}

/*******************************************************************************
* Function Name  : SSI_QUAD_DMA_Trig
* Description    : SSI QUAD DMA  
* Input          : - SSIx   : SSI ַ
*								 - cmd    ָ
*								 - addr   ַ
*								 - dmaConf䷽ʽ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_DUAL_DMA_Trig(SSI_TypeDef *SSIx, unsigned cmd, unsigned int addr, int dmaConf)
{
	SSIx->SSI_DR=cmd;
	SSIx->SSI_DR=addr;

	SSIx->SSI_DMACR=dmaConf;	 
}	


/*******************************************************************************
* Function Name  : SSI_DUAL_DMA_Read
* Description    : spi dmaպ
* Input          : - SPIx: SPI ַ
*                - dma_ch: DMA channel
*				          - pread: ݵַ
*                  - addr: spi flashַ
*				        - length  ݳ
*				        - binten : Ƿжģʽ
*
* Output         : None
* Return         : None
******************************************************************************/
void SSI_DUAL_DMA_Read(SSI_TypeDef *SSIx,int dma_ch, uint8_t* pread, uint32_t addr, uint32_t length, BOOL binten)
{
//	unsigned char status;
//	int num2;
//	int j;
	
	m_dma_control->DMA_CONFIG = 1;
	//Rx
	m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)&SSIx->SSI_DR;
	m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)pread;
	if(binten == TRUE)
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = INTEN|DIEC|SNOCHG|P2M_DMA|DWIDTH_B|SWIDTH_B;
	}
	else
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = DIEC|SNOCHG|P2M_DMA|DWIDTH_B|SWIDTH_B;
	}
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = length ;  //󳤶Ϊ0x0FFF
	m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_SRC_HARD);
	if(SSIx == SSI1)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = SRC_PER_SPI_RX(2);
	}
	else if(SSIx == SSI2)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = SRC_PER_SPI_RX(5);
	}
	else
	{
		printf("unknown SSIx\r\n");
		while(1){;}	
	}
	//enable dma channel
	if(binten == TRUE)
	{
	   m_dma_control->DMA_MASKTFR = CHANNEL_UMASK(dma_ch);
	}	
	m_dma_control->DMA_CHEN = CHANNEL_WRITE_ENABLE(dma_ch)|CHANNEL_ENABLE(dma_ch);
	//printf("1m_dma_control->DMA_RAWTFR: 0x%08x\r\n", m_dma_control->DMA_RAWTFR);
	
	SSI_DUAL_DMA_Trig(SSIx, DUAL_READ_CMD, addr, DMACR_RDMAE);
	
	if(binten == TRUE)
	{
			while(1)
			{
				if(dma_int == CHANNEL_STAT(dma_ch) )
				{
					dma_int = 0;
					break;
				}
			}
	}	 
	else
	{			
		while((m_dma_control->DMA_RAWTFR & CHANNEL_STAT(dma_ch)) != CHANNEL_STAT(dma_ch));    		
		m_dma_control->DMA_CLRTFR =  m_dma_control->DMA_STATTFR;
	}	 

	while(m_dma_control->DMA_CHEN & CHANNEL_STAT(dma_ch))
	{
	   ;
	}	
	m_dma_control->DMA_CHEN = 0;
	m_dma_control->DMA_CONFIG = 0;
	SSIx->SSI_DMACR = 0;
}

void SSI_DUAL_DMA_Send(SSI_TypeDef *SSIx,int dma_ch, uint8_t* psend, uint32_t addr, uint32_t length, BOOL binten)
{
	
	m_dma_control->DMA_CONFIG = 1;
	//Tx
	m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)psend;
	m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)&SSIx->SSI_DR;
	if(binten == TRUE)
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = INTEN|DNOCHG|SIEC|M2P_DMA|DWIDTH_B|SWIDTH_B;
	}
	else
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = DNOCHG|SIEC|M2P_DMA|DWIDTH_B|SWIDTH_B;
	}
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = length ;  //󳤶Ϊ0x0FFF
	m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_DST_HARD);
	if(SSIx == SSI1)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(3);
	}
	else if(SSIx == SSI2)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(2);
	}
    else
    {
        printf("unknown SSIx\r\n");
    }
	//enable dma channel
	if(binten == TRUE)
	{	
	   m_dma_control->DMA_MASKTFR = CHANNEL_UMASK(dma_ch);
	}
	m_dma_control->DMA_CHEN = (CHANNEL_WRITE_ENABLE(dma_ch)|CHANNEL_ENABLE(dma_ch));
	//printf("1m_dma_control->DMA_RAWTFR: 0x%08x\r\n", m_dma_control->DMA_RAWTFR);
	
	SSI_STD_DMA_Trig(SSIx, PAGE_PROG_CMD, addr, DMACR_TDMAE);
	
	if(TRUE == binten)
    {
        while(1)
        {
            if(dma_int == CHANNEL_STAT(dma_ch) )
            {
                dma_int = 0;
                break;
            }
        }			

    }	 
    else
    {
        while((m_dma_control->DMA_RAWTFR & CHANNEL_STAT(dma_ch)) != CHANNEL_STAT(dma_ch)){ }    		
        m_dma_control->DMA_CLRTFR =  m_dma_control->DMA_STATTFR;
    }	 

    while(m_dma_control->DMA_CHEN & CHANNEL_STAT(dma_ch)){}
    m_dma_control->DMA_CHEN = 0;
    m_dma_control->DMA_CONFIG = 0;
    SSIx->SSI_DMACR = 0;
}

/*******************************************************************************
* Function Name  : SSI_DUAL_DMA_Trig
* Description    : SSI QUAD DMA  
* Input          : - SSIx   : SSI ַ
*								 - cmd    ָ
*								 - addr   ַ
*								 - dmaConf䷽ʽ
* Output         : None
* Return         : None
******************************************************************************/
void SSI_QUAD_DMA_Trig(SSI_TypeDef *SSIx, unsigned cmd, unsigned int addr, int dmaConf)
{
	SSIx->SSI_DR=cmd;
	SSIx->SSI_DR=addr;	
    SSIx->SSI_DMACR=dmaConf; 
}	

/*******************************************************************************
* Function Name  : SSI_QUAD_DMA_Read
* Description    : spi dmaպ
* Input          : - SPIx: SPI ַ
*                - dma_ch: DMA channel
*				          - pread: ݵַ
*                  - addr: spi flashַ
*				        - length  ݳ
*				        - binten : Ƿжģʽ
*
* Output         : None
* Return         : STATUS
******************************************************************************/
uint16_t SSI_QUAD_DMA_Read(SSI_TypeDef *SSIx,int dma_ch, uint8_t* pread, uint32_t addr, uint32_t length, BOOL binten)
{
//  uint32_t timeout = SSI_TIMEOUT_COUNTERS(1000);
	
	SSI_QUAD_Init(SSI1, 1, length, 8); //flush FIFO              //
	DMA_REG_Init(DMA2_BASE_ADDR);

	m_dma_control->DMA_CONFIG = 1;
	//Rx
	m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)&SSIx->SSI_DR;
	m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)pread;
	if(binten == TRUE)
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = INTEN|DIEC|SNOCHG|P2M_DMA|DWIDTH_B|SWIDTH_B;
	}
	else
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = DIEC|SNOCHG|P2M_DMA|DWIDTH_B|SWIDTH_B;     //
	}
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = length ;  //󳤶Ϊ0x0FFF
	m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_SRC_HARD);
	if(SSIx == SSI1)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = SRC_PER_SPI_RX(2);
	}
	else if(SSIx == SSI2)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = SRC_PER_SPI_RX(5);
	}
	else
	{
		printf("unknown SSIx\r\n");
        return 6;	
	}
	//enable dma channel
	if(binten == TRUE)
	{	
	   m_dma_control->DMA_MASKTFR = CHANNEL_UMASK(dma_ch);
	}	
	m_dma_control->DMA_CHEN = CHANNEL_WRITE_ENABLE(dma_ch)|CHANNEL_ENABLE(dma_ch);
	//printf("1m_dma_control->DMA_RAWTFR: 0x%08x\r\n", m_dma_control->DMA_RAWTFR);
	
	SSI_QUAD_DMA_Trig(SSIx, QUAD_READ_CMD, addr, DMACR_RDMAE);
	
	if(binten == TRUE)
	{
        while(1)
        {
            if(dma_int == CHANNEL_STAT(dma_ch) )
            {
                dma_int = 0;
                break;
            }
        }
    }	 
    else
    {			
//        while((m_dma_control->DMA_RAWTFR & CHANNEL_STAT(dma_ch)) != CHANNEL_STAT(dma_ch)){}		
//        m_dma_control->DMA_CLRTFR =  m_dma_control->DMA_STATTFR;
    }	 

    while(m_dma_control->DMA_CHEN & CHANNEL_STAT(dma_ch)){}
    m_dma_control->DMA_CHEN = 0;
    m_dma_control->DMA_CONFIG = 0;
    SSIx->SSI_DMACR = 0;

    return 0;
}

uint16_t SSI_QUAD_DMA_Send(SSI_TypeDef *SSIx,int dma_ch, uint8_t* psend, uint32_t addr, uint32_t length, BOOL binten)
{
	#if 0
//  uint32_t timeout = SSI_TIMEOUT_COUNTERS(1000);
	int  num;
	
	SSI_QUAD_Init(SSI1, 0, length, 0); //flush FIFO              //
	DMA_REG_Init(DMA2_BASE_ADDR);

	m_dma_control->DMA_CONFIG = 1;
	//Rx
	m_dma2_channel[dma_ch]->DMA_SADDR = (uint32_t)psend;
	m_dma2_channel[dma_ch]->DMA_DADDR = (uint32_t)&SSIx->SSI_DR;
	if(binten == TRUE)
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = INTEN|DNOCHG|SIEC|M2P_DMA|DWIDTH_B|SWIDTH_B;
	}
	else
	{
		m_dma2_channel[dma_ch]->DMA_CTRL = DNOCHG|SIEC|M2P_DMA|DWIDTH_B|SWIDTH_B;//|SBSIZE_4|DBSIZE_4;
	}
	m_dma2_channel[dma_ch]->DMA_CTRL_HIGH = length ;  //󳤶Ϊ0x0FFF
	m_dma2_channel[dma_ch]->DMA_CFG = (HS_SEL_DST_HARD);
	if(SSIx == SSI1)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(3);
	}
	else if(SSIx == SSI2)
	{
		m_dma2_channel[dma_ch]->DMA_CFG_HIGH = DST_PER_SPI_TX(2);
	}
	else
	{
		printf("unknown SSIx\r\n");
        return 6;	
	}
	//enable dma channel
	if(binten == TRUE)
	{	
	   m_dma_control->DMA_MASKTFR = CHANNEL_UMASK(dma_ch);
	}	
	m_dma_control->DMA_CHEN = CHANNEL_WRITE_ENABLE(dma_ch)|CHANNEL_ENABLE(dma_ch);
	//printf("1m_dma_control->DMA_RAWTFR: 0x%08x\r\n", m_dma_control->DMA_RAWTFR);
	
	SSI_QUAD_DMA_Trig(SSIx, QUAD_PROG_CMD, addr, DMACR_RDMAE);
	
	if(binten == TRUE)
	{
        while(1)
        {
            if(dma_int == CHANNEL_STAT(dma_ch) )
            {
                dma_int = 0;
                break;
            }
        }
    }	 
    else
    {			
//        while((m_dma_control->DMA_RAWTFR & CHANNEL_STAT(dma_ch)) != CHANNEL_STAT(dma_ch)){}		
//        m_dma_control->DMA_CLRTFR =  m_dma_control->DMA_STATTFR;
    }	 

    while(m_dma_control->DMA_CHEN & CHANNEL_STAT(dma_ch)){}
    m_dma_control->DMA_CHEN = 0;
    m_dma_control->DMA_CONFIG = 0;
    SSIx->SSI_DMACR = 0;

    return 0;
	#endif	
}
