// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : uart_drv.c
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#include "debug.h"
#include "cpm_drv.h"
#include "iomacros.h"
#include "uart_drv.h"
#include "uart_reg.h"
#include "dmac_drv.h"
#include "stddef.h"
#include "stdarg.h"
#include "stdlib.h"
//#include "uart_dma.h"

volatile uint8_t uart_idle_flag ;

/*
typedef struct _uart_recv_buf
{
	uint16_t rp;//read pointer
	uint16_t wp;//write pointer
	uint16_t recv_mode;
	uint16_t ret_val;
	uint8_t  dat[UART_RECV_MAX_LEN];//buffer
}UartRecvBufStruct;
*/


UART_InitTypeDef  UART_BT_InitStruct;

/*******************************************************************************
* Function Name  : UART_ISR
* Description    : UARTжϴ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
static void UART_ISR(UART_TypeDef *UARTx, UartRecvBufStruct *UARTxRecvBufStruct)
{
    uint32_t timeout = UART_TIMEOUT(1000);
    
    UARTxRecvBufStruct->ret_val = STATUS_OK;

	if(UART_DMA_MODE == UARTxRecvBufStruct->recv_mode)
	{
		if(UARTx->SCISR1 & SCISR1_IDLE_MASK)      //
		{
			while(((UARTx->SCIFSR)&SCIFSR_REMPTY_MASK) == 0) // ȴյDMA
			{
				if((UARTx->SCIFSR)&SCIFSR_RTOS_MASK)
				{
					UARTx->SCIFSR2 = 0x0F;
					break;
				}

                if(!timeout)
                {
                    UARTxRecvBufStruct->ret_val |= STATUS_UART_TIMEOUT;
                    return;
                }
                timeout --;
			}
			uart_idle_flag = 1;
		}
	}
	
	if(UART_INT_MODE == UARTxRecvBufStruct->recv_mode)
	{
		if((UARTx->SCIFCR&SCIFCR_RFEN) || (UARTx->SCIFCR&SCIFCR_TFEN))
		{
	#if 0  //ҪFIFO RFTSж϶Ӧ
			if((UARTx->SCIFSR & SCIFSR_RFTS_MASK) == SCIFSR_RFTS_MASK)
			{
				for(uint8_t loop = 0;; loop<UART_FIFO_TRIGGER_LEVEL; loop ++)
				{
					UARTxRecvBufStruct->dat[UARTxRecvBufStruct->wp + loop] = UARTx->SCIDRL;
				}
				UARTxRecvBufStruct->wp += loop;
				if (UARTxRecvBufStruct->wp >= UART_RECV_MAX_LEN)
					UARTxRecvBufStruct->wp = 0;
				UARTxRecvBufStruct->ret_val = STATUS_OK;
			}
	#endif
			
			while((!(UARTx->SCIFSR & SCIFSR_REMPTY_MASK)) && (timeout))
			{
				UARTxRecvBufStruct->dat[UARTxRecvBufStruct->wp] = UARTx->SCIDRL;
				UARTxRecvBufStruct->wp += 1;
				if (UARTxRecvBufStruct->wp >= UART_RECV_MAX_LEN)
					UARTxRecvBufStruct->wp = 0;
				UARTxRecvBufStruct->ret_val = STATUS_OK;
				timeout --;
			}
            		
			if(!timeout)
			{
				UARTxRecvBufStruct->ret_val |= STATUS_UART_TIMEOUT;
			}
		
		}
		else
		{
			//reveice data
			if((UARTx->SCISR1&SCISR1_RDRF_MASK) == SCISR1_RDRF_MASK)
			{
				UARTxRecvBufStruct->dat[UARTxRecvBufStruct->wp] = UARTx->SCIDRL;
				UARTxRecvBufStruct->wp += 1;
				if (UARTxRecvBufStruct->wp >= UART_RECV_MAX_LEN)
					UARTxRecvBufStruct->wp = 0;
				UARTxRecvBufStruct->ret_val = STATUS_OK;
			}
			
		} 
	}

    if((UARTx->SCISR1&SCISR1_OR_MASK) == SCISR1_OR_MASK)
    {
        UARTxRecvBufStruct->ret_val |= STATUS_UART_OVERRUN;
        uint8_t tmp = UARTx->SCIDRL;   //clear error flag
    }

    if(UARTx->SCIFSR2&SCISR2_FOR_MASK)     //fifo full
    {
        uint8_t tmp = UARTx->SCIDRL;
        UARTx->SCIFSR2 |= SCISR2_FOR_MASK;
        UARTxRecvBufStruct->ret_val |= STATUS_UART_FIFO_OVERRUN;
    }

    //timeout
    if((UARTx->SCIFSR & SCIFSR_RTOS_MASK ) == SCIFSR_RTOS_MASK)
    {
        uint8_t tmp = UARTx->SCIDRL;
        UARTx->SCIFCR2 |= (SCIFCR2_RXFCLR);
        UARTxRecvBufStruct->ret_val |= STATUS_UART_TIMEOUT;
    }

}

/*******************************************************************************
* Function Name  : UART3_ISR
* Description    : UART2жϴ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
//void SCI1_IRQHandler(void)
//{
//	UART_ISR(SCI1, &g_UartRecvBufStruct);
//}

/*******************************************************************************
* Function Name  : UART4_ISR
* Description    : UART4жϴ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
void SCI2_IRQHandler(void)
{
	UART_ISR(SCI2, &g_UartRecvBufStruct);
}

/*******************************************************************************
* Function Name  : UART2_ISR
* Description    : UART2жϴ
* Input          : None
*
* Output         : None
* Return         : None
******************************************************************************/
//void SCI3_IRQHandler(void)
//{
//	UART_ISR(SCI3, &g_UartRecvBufStruct);
//}


/*******************************************************************************
* Function Name  : UART_Init
* Description    : UARTʼ
* Input          : - UARTx: ȡֵSCI2 or SCI_BT
*                  - UART_InitStructUARTx  initialization parameter structure Poniter
*
* Output         : None
* Return         : None
******************************************************************************/
void UART_Init(UART_TypeDef *UARTx, UART_InitTypeDef *UART_InitStruct)
{
	uint16_t bandrate_I;
	uint8_t bandrate_F;
	uint8_t bandrate_h;
	uint8_t bandrate_l;

	if (SCI1 == UARTx)//SCI1
	{
		g_UartRecvBufStruct.wp = 0;
		g_UartRecvBufStruct.rp = 0; 
		if ((UART_InitStruct->UART_Mode == UART_INT_MODE)|| (UART_InitStruct->UART_Mode == UART_DMA_MODE))
		{
			NVIC_Init(3, 3, SCI1_IRQn, 2);
		}
	}
	else if (SCI2 == UARTx)//SCI2
	{
		g_UartRecvBufStruct.wp = 0;
		g_UartRecvBufStruct.rp = 0; 
		if ((UART_InitStruct->UART_Mode == UART_INT_MODE)|| (UART_InitStruct->UART_Mode == UART_DMA_MODE))
		{
			NVIC_Init(3, 3, SCI2_IRQn, 2);
		}
	}
	else//SCI3
	{
		g_UartRecvBufStruct.wp = 0;
		g_UartRecvBufStruct.rp = 0; 
		if ((UART_InitStruct->UART_Mode == UART_INT_MODE)|| (UART_InitStruct->UART_Mode == UART_DMA_MODE))
		{
			NVIC_Init(3, 3, SCI3_IRQn, 2);
		}
	}

	bandrate_I = ((uint16_t)(g_ips_clk*4/UART_InitStruct->UART_BaudRate))>>6;
	bandrate_F = (((uint16_t)(g_ips_clk*8/UART_InitStruct->UART_BaudRate)+1)/2)&0x003f;
	bandrate_h = (uint8_t)((bandrate_I>>8)&0x00ff);
	bandrate_l = (uint8_t)(bandrate_I&0x00ff);

	UARTx->SCIBRDF = bandrate_F;		//Write float before Interger
	UARTx->SCIBDH = bandrate_h;
	UARTx->SCIBDL = bandrate_l;

	UARTx->SCICR1 = 0x00;
	UARTx->SCICR1 |= UART_InitStruct->UART_FrameLength; //11 bit frame
	if (UART_InitStruct->UART_Parity == UART_PARITY_NONE)
	{
		UARTx->SCICR1 |= ParityDIS;	//parity disable
	}
	else
	{
		UARTx->SCICR1 |= ParityEN;	//parity enable
		UARTx->SCICR1 |= UART_InitStruct->UART_Parity;//odd parity
	}
 
	UARTx->SCICR2 = 0;
	UARTx->SCIRXTOCTR = UART_InitStruct->UART_TimeoutCounter;

	if (UART_InitStruct->UART_Mode == UART_INT_MODE)
	{
		g_UartRecvBufStruct.recv_mode = UART_INT_MODE;
		//Setup SCI interrupt
		UARTx->SCICR2 |= SCICR2_RIE_MASK|SCICR2_RE_MASK;
		UARTx->SCIFCR2 |= (SCIFCR2_RXFTOE|SCIFCR2_RXFTOIE|SCIFCR2_RXFCLR);
		UARTx->SCIFCR = (SCIFCR_RFEN|SCIFCR_RXFLSEL_1_8);
	} 
    
	if(UART_InitStruct->UART_Mode == UART_DMA_MODE)
	{
		g_UartRecvBufStruct.recv_mode = UART_DMA_MODE;
		UARTx->SCIFCR = 0;
		UARTx->SCIFCR  |= (SCIFCR_RFEN|SCIFCR_TFEN); 
		UARTx->SCIFCR2 |= (SCIFCR2_TXFCLR|SCIFCR2_RXFCLR|SCIFCR_RXFLSEL_1_8|SCIFCR_TXFLSEL_1_8);//enable fifo 
		UARTx->SCICR1  |=  SCICR1_ILT_MASK;
		UARTx->SCICR2  |= (SCICR2_RE_MASK|SCICR2_ILIE_MASK);
	}
} 

/*******************************************************************************
* Function Name  : UART_ByteRecieved
* Description    : UARTյ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*
* Output         : - None
*
* Return         : - TRUE:յ
*                  - FALSE:ûнյ
******************************************************************************/
int8_t UART_ByteRecieved(UART_TypeDef *UARTx)
{
	UartRecvBufStruct *UARTxRecvBufStruct;


	if (UARTxRecvBufStruct->wp != UARTxRecvBufStruct->rp)
	{
		return TRUE;
	}
	return FALSE;
}
/*******************************************************************************
* Function Name  : UART_RecvByte
* Description    : UARTһֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*
*
* Output         : - datݻ
*
* Return         : - STATUS_OK:յ
*                  - STATUS_NULL:ûнյ
******************************************************************************/
uint16_t UART_RecvByte(UART_TypeDef *UARTx, uint8_t *dat)
{
	UartRecvBufStruct *UARTxRecvBufStruct;
    uint16_t ret;

	UARTxRecvBufStruct = &g_UartRecvBufStruct;

	if (UARTxRecvBufStruct->wp != UARTxRecvBufStruct->rp)
	{
		*dat = UARTxRecvBufStruct->dat[UARTxRecvBufStruct->rp];
		UARTxRecvBufStruct->rp += 1;
		if (UARTxRecvBufStruct->rp >= UART_RECV_MAX_LEN)
			UARTxRecvBufStruct->rp = 0;
		return STATUS_OK;
	}
    
    if(STATUS_OK != UARTxRecvBufStruct->ret_val)
    {
        ret = UARTxRecvBufStruct->ret_val;
        UARTxRecvBufStruct->ret_val = STATUS_OK; 
    }
    else
    {
        ret = STATUS_NULL;
    }
    
	return ret;
}

/*******************************************************************************
* Function Name  : UART_ReceiveByte
* Description    : UARTһֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*
* Output         : None
* Return         : յһֽ
******************************************************************************/
uint8_t UART_ReceiveByte(UART_TypeDef *UARTx)
{  
	uint8_t tmp;
	uint32_t timeout = UART_TIMEOUT(1000);

	while(0 == (UARTx->SCISR1 & SCISR1_RDRF_MASK))
	{
		if(!timeout)
		{
            g_UartRecvBufStruct.ret_val = STATUS_UART_TIMEOUT;
			return 0;
		}
		timeout --;
	}

	tmp = UARTx->SCIDRL;

	return tmp;

}
/*******************************************************************************
* Function Name  : UART_ReceiveData
* Description    : UARTնֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - recv_bufݻ
*                  - lenյݳ
*
* Output         : None
* Return         : STATUS
******************************************************************************/
uint16_t UART_ReceiveData(UART_TypeDef *UARTx, uint8_t *recv_buf, uint32_t len)
{
	uint32_t i;

    g_UartRecvBufStruct.ret_val = STATUS_OK;

	SCI_ENABLE_RECEIVER(UARTx);
	for(i = 0; i < len; i++)
	{
		recv_buf[i] = UART_ReceiveByte(UARTx);
        if(STATUS_OK != g_UartRecvBufStruct.ret_val)
        {
            break;
        }

	}
	SCI_DISABLE_RECEIVER(UARTx);

    return g_UartRecvBufStruct.ret_val;
}

/*******************************************************************************
* Function Name  : UART_SendByte
* Description    : UARTһֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - SendByte͵ֽ
*
* Output         : None
* Return         : STATUS
******************************************************************************/
uint16_t UART_SendByte(UART_TypeDef *UARTx, uint8_t outbyte)
{
    uint32_t timeout = UART_TIMEOUT(1000);

	while((UARTx->SCISR1 & SCISR1_TC_MASK)==0)
    {
        if(!timeout)
        {
            return STATUS_UART_TIMEOUT;
        }
        timeout --;
    }
	while((UARTx->SCISR1 & SCISR1_TDRE_MASK)==0)
    {
        if(!timeout)
        {
            return STATUS_UART_TIMEOUT;
        }
        timeout --;
    }
	UARTx->SCIDRL = outbyte&0xff;
	while((UARTx->SCISR1 & SCISR1_TC_MASK)==0)
    {
        if(!timeout)
        {
            return STATUS_UART_TIMEOUT;
        }
        timeout --;
    }
    return STATUS_OK;
}

/*******************************************************************************
* Function Name  : UART_SendData
* Description    : UARTͶֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - send_bufݻ
*                  - len͵ݳ
*
* Output         : None
* Return         : None
******************************************************************************/
uint16_t UART_SendData(UART_TypeDef *UARTx, uint8_t *send_buf, uint32_t len)
{
	uint16_t ret;
    uint32_t i;

	SCI_ENABLE_TRANSMITTER(UARTx);
	for (i = 0; i < len; i++)
	{
        ret = UART_SendByte(UARTx, send_buf[i]);
		if(STATUS_OK != ret)
        {
            break;
        }
	}
	SCI_DISABLE_TRANSMITTER(UARTx);

    return ret;
}


/*******************************************************************************
* Function Name  : UART_ConfigGpio
* Description    : UARTóGPIO;
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - UART_PinӦPINţȡֵUART_TXUART_RX
*                  - UART_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*
* Output         : None
* Return         : None
******************************************************************************/
void UART_ConfigGpio(UART_TypeDef *UARTx, UART_PINx UART_Pin, uint8_t UART_Dir)
{
	UARTx->SCICR1 = 0x80;

	if (UART_Dir == GPIO_OUTPUT)
	{
		UARTx->SCIDDR |= (1<<UART_Pin);//output
	}
	else if (UART_Dir == GPIO_INPUT)
	{
		UARTx->SCIDDR &= (~(1<<UART_Pin));//input
	}

}

/*******************************************************************************
* Function Name  : UART_ReadGpioData
* Description    : ȡUART_PinӦŵĵƽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - UART_PinӦPINţȡֵUART_TXUART_RX
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ
******************************************************************************/
int8_t UART_ReadGpioData(UART_TypeDef *UARTx, UART_PINx UART_Pin)
{
	int8_t bitstatus = 0x00;

	bitstatus = UARTx->SCIPORT;

	if (bitstatus &(Bit_SET<<UART_Pin))
		bitstatus = Bit_SET;
	else
		bitstatus = Bit_RESET;

	return bitstatus;
}

/*******************************************************************************
* Function Name  : UART_WriteGpioData
* Description    : UART_PinӦŵĵƽ
* Input          : - - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - UART_PinӦPINţȡֵUART_TXUART_RX
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : None
******************************************************************************/
void UART_WriteGpioData(UART_TypeDef *UARTx, UART_PINx UART_Pin, uint8_t bitVal)
{
	if (bitVal == Bit_SET)
		UARTx->SCIPORT |= (Bit_SET<<UART_Pin);
	else
		UARTx->SCIPORT &= (~(Bit_SET<<UART_Pin));
}

///*******************************************************************************
//* Function Name  : UART_DMAReceiveData
//* Description    : UART DMA
//* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
//*                  - channel:DMAͨ
//*                  - lenݳ
//*
//* Output         : - destݻ
//*                  
//* Return         : STATUS
//******************************************************************************/
//uint16_t UART_DMAReceiveData(UART_TypeDef *UARTx,uint8_t channel,uint8_t *dest,uint32_t len)
//{
//	uint16_t ret;

//	SCI_ENABLE_RECEIVER(UARTx);
//	SCI_ENABLE_DMA(UARTx);
//	ret = dma_uartRX(UARTx,channel,dest,len);
//	SCI_DISABLE_DMA(UARTx);
//	SCI_DISABLE_RECEIVER(UARTx);
//	SCI_FIFO_CLR(UARTx);

//	return ret;
//}

///*******************************************************************************
//* Function Name  : UART_DMASendData
//* Description    : UART DMA
//* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
//*                  - channel:DMAͨ
//*                  - destݻ
//*                  - lenݳ
//*
//* Output         : None
//* Return         : STATUS
//******************************************************************************/
//uint16_t UART_DMASendData(UART_TypeDef *UARTx,uint8_t channel,uint8_t *src,uint32_t len)
//{
//    uint16_t ret; 

//	SCI_DISABLE_RECEIVER(UARTx);
//	SCI_ENABLE_TRANSMITTER(UARTx);
//	SCI_ENABLE_DMA(UARTx);
//	ret = dma_uartTX(UARTx,channel,src,len);
//	SCI_DISABLE_DMA(UARTx);

//   return ret;
//}






