// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : uart_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifndef UART_DRV_H_
#define UART_DRV_H_

#include "memmap.h"
//#include "def.h"
#include "uart_reg.h"

#define SCI1		        ((UART_TypeDef *)(SCI1_BASE_ADDR))
#define SCI2		        ((UART_TypeDef *)(SCI2_BASE_ADDR))
#define SCI3		        ((UART_TypeDef *)(SCI3_BASE_ADDR))

#define SCI_BT          SCI2

/*************Status definition***********************/
#define STATUS_OK                           (unsigned short)0x0000
#define STATUS_NULL                         (unsigned short)0x0001
#define STATUS_BUSY                         (unsigned short)0x0002
#define STATUS_TIMEOUT                      (unsigned short)0x0003
#define STATUS_UNKNOW_ERR                   (unsigned short)0x0004
#define STATUS_ADDR_ERR                     (unsigned short)0x0005
#define STATUS_ID_ERR                       (unsigned short)0x0006
	
//UART 
#define STATUS_UART_OVERRUN                 (unsigned short)0x1001
#define STATUS_UART_NOISE                   (unsigned short)0x1002
#define STATUS_UART_FRAMING_ERR             (unsigned short)0x1004
#define STATUS_UART_PARITY_ERR              (unsigned short)0x1008
#define STATUS_UART_FIFO_PARITY_ERR         (unsigned short)0x1010
#define STATUS_UART_FIFO_FRAMING_ERR        (unsigned short)0x1020
#define STATUS_UART_FIFO_NOISE              (unsigned short)0x1040
#define STATUS_UART_FIFO_OVERRUN            (unsigned short)0x1080
#define STATUS_UART_TIMEOUT                 (unsigned short)0x1100


#define IS_UART_ALL_PERIPH(PERIPH) (((PERIPH) == SCI) || \
                                    ((PERIPH) == SCI_BT))
////////////////////////////////////////////////////////////////////////////////////////

#define SCI2_ENABLE_TRANSMITTER      (SCI2->SCICR2 |= SCICR2_TE_MASK)
#define SCI2_DISABLE_TRANSMITTER     (SCI2->SCICR2 &= ~SCICR2_TE_MASK)

#define SCI2_ENABLE_RECEIVER         (SCI2->SCICR2 |= SCICR2_RE_MASK)
#define SCI2_DISABLE_RECEIVER        (SCI2->SCICR2 &= ~SCICR2_RE_MASK)



////////////////////////////////////////////////////////////////////////////////////////
#define		UART_DATA_FRAME_LEN_10BIT	    0				//10bit frame:----StartBit:1bit DataBit:8bit StopBit:1bit
#define		UART_DATA_FRAME_LEN_11BIT	    SCICR1_M_MASK	//11bit frame:----StartBit:1bit DataBit:9bit StopBit:1bit

#define IS_UART_FRAMW_LENGTH(LENGTH) (((LENGTH) == UART_DATA_FRAME_LEN_10BIT) || \
                                      ((LENGTH) == UART_DATA_FRAME_LEN_11BIT))



#define		ParityDIS	                 0			      	//Parity Disable
#define		ParityEN	                 SCICR1_PE_MASK	//Parity Enable

#define		UART_PARITY_EVE		         0			      	//Even Parity
#define		UART_PARITY_ODD		         SCICR1_PT_MASK	//Odd Parity
#define   UART_PARITY_NONE           2              //NONE Parity


#define IS_UART_PARITY(PARITY) (((PARITY) == UART_PARITY_EVE) || \
                                ((PARITY) == UART_PARITY_ODD) || \
                                ((PARITY) == UART_PARITY_NONE))

#define UART_DELAY               20000
#define UART_TIMEOUT(n)          (n*UART_DELAY)
//#define UART_TIMEOUT_1S              20000000  // sys_clk=120MHz, delay 1MS  sys_clk/6000
/*******************************************************************************
RXFLSEL                     UART_FIFO_LEN
SCIFCR_RXFLSEL_1_8          0x02
SCIFCR_RXFLSEL_1_4          0x04
SCIFCR_RXFLSEL_1_2          0x08
SCIFCR_RXFLSEL_3_4          0x0C
SCIFCR_RXFLSEL_7_8          0x0E
*******************************************************************************/
#define UART_FIFO_TRIGGER_LEVEL            0x02       //RXFLSEL
#define UART_RECV_MAX_LEN                  0x200

typedef struct _uart_recv_buf
{
	uint16_t rp;//read pointer
	uint16_t wp;//write pointer
	uint16_t recv_mode;
	uint16_t ret_val;
	uint8_t  dat[UART_RECV_MAX_LEN];//buffer
}UartRecvBufStruct;

static UartRecvBufStruct g_UartRecvBufStruct = {0, 0, 0, 0};//uartx buffer

typedef enum
{
	UART_NORMAL_MODE = 0,
	UART_INT_MODE,
	UART_EDMA_MODE,
	UART_DMA_MODE

}UART_MODE;

//ѡһģʽ
#define SCI_CPU_EN		0 
#define SCI_DMA_EN		0
#define SCI_INT_EN		1

#define IS_UART_MODE(MODE) (((MODE) == UART_NORMAL_MODE) || \
                            ((MODE) == UART_INT_MODE) || \
                            ((MODE) == UART_EDMA_MODE) || \
                            ((MODE) == UART_DMA_MODE) )

/* UART Init Structure definition */
typedef struct
{

  uint32_t UART_BaudRate;
  uint8_t  UART_FrameLength;
  uint8_t  UART_StopBits;
  uint8_t  UART_Parity;
  uint8_t  UART_Mode;
  uint8_t  UART_TimeoutCounter;

} UART_InitTypeDef;

typedef enum
{
    UART_RX = 0,
    UART_TX,

}UART_PINx;

extern UART_InitTypeDef UART_BT_InitStruct;
/*******************************************************************************
* Function Name  : UART_Init
* Description    : UARTʼ
* Input          : - UARTx: where x can be 0 to 1 to select the UART peripheralȡֵSCI0 or SCI1
*                  - UART_InitStructUARTx  initialization parameter structure Poniter
*
* Output         : None
* Return         : None
******************************************************************************/
extern void UART_Init(UART_TypeDef *UARTx, UART_InitTypeDef *UART_InitStruct);

/*******************************************************************************
* Function Name  : UART_ReceiveByte
* Description    : UARTһֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*
* Output         : None
* Return         : յһֽ
******************************************************************************/
extern uint8_t UART_ReceiveByte(UART_TypeDef *UARTx);
/*******************************************************************************
* Function Name  : UART_ReceiveData
* Description    : UARTնֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - recv_bufݻ
*                  - len͵ݳ
*
* Output         : None
* Return         : STATUS
******************************************************************************/
extern uint16_t UART_ReceiveData(UART_TypeDef *UARTx,uint8_t *recv_buf, uint32_t len);

/*******************************************************************************
* Function Name  : UART_SendByte
* Description    : UARTһֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - SendByte͵ֽ
*
* Output         : None
* Return         : None
******************************************************************************/
extern uint16_t UART_SendByte(UART_TypeDef *UARTx, uint8_t outbyte);

/*******************************************************************************
* Function Name  : UART_SendData
* Description    : UARTͶֽ
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - send_bufݻ
*                  - len͵ݳ
*
* Output         : None
* Return         : None
******************************************************************************/
extern uint16_t UART_SendData(UART_TypeDef *UARTx, uint8_t *send_buf, uint32_t len);

/*******************************************************************************
* Function Name  : UART_ByteRecieved
* Description    : UARTյ
* Input          : - UARTx:ȡֵSCI1 SCI2
*
* Output         : - None
*
* Return         : - TRUE:յ
*                  - FALSE:ûнյ
******************************************************************************/
extern int8_t UART_ByteRecieved(UART_TypeDef *UARTx);

/*******************************************************************************
* Function Name  : UART_RecvByte
* Description    : UARTһֽ
* Input          : - UARTx: where x can be 0 to 1 to select the UART peripheralȡֵSCI0 or SCI1
*
*
* Output         : - datݻ
*
* Return         : - TRUE:յ
*                  - FALSE:ûнյ
******************************************************************************/
extern uint16_t UART_RecvByte(UART_TypeDef *UARTx, uint8_t *dat);

/*******************************************************************************
* Function Name  : UART_ConfigGpio
* Description    : UARTóGPIO;
* Input          : - UARTx: where x can be 1, 2  to select the UART peripheral.
*                  - UART_PinӦPINţȡֵUART_TXUART_RX
*                  - UART_DirGPIO   GPIO_OUTPUT  GPIO_INPUT
*
* Output         : None
* Return         : None
******************************************************************************/
extern void UART_ConfigGpio(UART_TypeDef *UARTx, UART_PINx UART_Pin, uint8_t UART_Dir);

/*******************************************************************************
* Function Name  : UART_ReadGpioData
* Description    : ȡUART_PinӦŵĵƽ
* Input          : - UARTx: where x can be 1, 2  to select the UART peripheral.
*                  - UART_PinӦPINţȡֵUART_TXUART_RX
*
* Output         : None
* Return         : Bit_SET:ߵƽ  Bit_RESET͵ƽ
******************************************************************************/
extern int8_t UART_ReadGpioData(UART_TypeDef *UARTx, UART_PINx UART_Pin);

/*******************************************************************************
* Function Name  : UART_WriteGpioData
* Description    : UART_PinӦŵĵƽ
* Input          : - UARTx: where x can be 1, 2  to select the UART peripheral.
*                  - UART_PinӦPINţȡֵUART_TXUART_RX
*                  - bitValõĵƽBit_SETΪߵƽ  Bit_RESETΪ͵ƽ
*
* Output         : None
* Return         : None
******************************************************************************/
extern void UART_WriteGpioData(UART_TypeDef *UARTx, UART_PINx UART_Pin, uint8_t bitVal);

/*******************************************************************************
* Function Name  : UART_DMAReceiveData
* Description    : UART DMA
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - channel:DMAͨ
*                  - destݻ
*                  - lenݳ
*
* Output         : None
* Return         : None
******************************************************************************/
//extern uint16_t UART_DMAReceiveData(UART_TypeDef *UARTx,uint8_t channel,uint8_t *dest,uint32_t len);
    
/*******************************************************************************
* Function Name  : UART_DMASendData
* Description    : UART DMA
* Input          : - UARTx: where x can be 1 to 3 to select the UART peripheralȡֵUART1/UART2/UART3
*                  - channel:DMAͨ
*                  - destݻ
*                  - lenݳ
*
* Output         : None
* Return         : None
******************************************************************************/
//extern uint16_t UART_DMASendData(UART_TypeDef *UARTx,uint8_t channel,uint8_t *src,uint32_t len);


#endif /* UART_DRV_H_ */

