// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// File name    : wdt_drv.h
// Version      : V0.1
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef __USB_DRV_H__
#define __USB_DRV_H__

#include "type.h"
#include "usb_reg.h"

#define USB_MS
//#define USB_HID
//#define USB_CDC

#define USB_MODE_1D1		0
#define USB_MODE_2D0		1

/*USB variables*/
/* Global USB Register */
extern USBCFIFO_Reg* gUSBC_fifoReg;
extern USBCCommonReg* gUSBC_ComReg;
extern USBCIndexedReg* gUSBC_IdxReg;

//usb the max packet size
extern unsigned short gTxMaxPacket;
extern unsigned short gRxMaxPacket;

//USB version(1--USB2.0;0--USB1.1)
extern unsigned char g_usbVer;

//Suspend Mode Enable Flag(0-disable;1-enable)
extern unsigned char g_suspendMode;

//Save USB Address
extern volatile unsigned char g_USBNewAddr;

//indicate which command is processed currently
extern volatile unsigned char g_bulkPhase;


extern uint8_t g_dev_std_req[];
extern uint8_t g_uchUSBStatus;

//global message queue for USB loop tasks.
extern volatile uint8_t g_msgflags;
extern uint8_t g_msgflags1;


//CDC using
extern const uint8_t USB_cdc_Device_Descriptor[];
//Device Qualifier Descriptor
extern const unsigned char USB_cdc_Device_Qualifier_Descriptor[];

extern const uint8_t USB_cdc_Configuration_Descriptor[] ;
extern const unsigned char USB_cdc_String_Descriptor[];

//USB Language ID
extern const uint8_t UsbLanguageID[] ;

//Manufacturer String
extern const uint8_t UsbStrDescManufacturer[];

//Product String
extern const uint8_t UsbStrDescProduct[];

//Serial Number
extern const uint8_t UsbStrDescSerialNumber[] ;

//standard Device Descriptor
extern const uint8_t USB_Device_Descriptor[18];
//Device Qualifier Descriptor
extern const uint8_t USB_Device_Qualifier_Descriptor[10];
//configuration descriptor
extern const uint8_t USB_Configuration_Descriptor[];
//standard string Descriptor,serial number
extern const uint8_t USB_String_Descriptor[];
//MBR const data
extern const uint8_t MBR[];
//DBR const data
extern const uint8_t DBR[];


//Save USB Address
extern	volatile uint8_t g_USBAddrFlag;

//indicate which sub command is processed currently
extern uint8_t g_subCmd;

//CBW packet
extern uint32_t g_cbwPacket_l[];
extern uint8_t* g_cbwPacket;
	
//CSW packet
extern uint8_t g_cswPacket[];

//USB data buffer1
extern uint32_t g_databuf;
extern uint32_t g_userdatabuf;

//indicate the start sector in reading device
extern uint32_t g_wrLBA;

//indicate the length in Once tranfer from PC to device
extern uint32_t g_transLens;

//indicate the transfer stage of USB1.1
extern uint8_t g_TxRxCounter;

//indicate the mode of reading device
extern uint8_t g_readMode; 
extern uint32_t RWDataAddr;
extern uint32_t RWDataLength;

//ChipTest Status
extern uint16_t g_Chiptest_Status;	//1-Selftest is running;0-Selftest finished
extern uint16_t g_Chiptest_Result;	//0-Selftest pass;Other-Selftest fail
extern uint16_t g_Chiptest_FailtoDo;	//0-After Selftest fail,system halt,Other-After fail,system go-on
extern uint16_t g_Chiptest_Enable;	//0-ChipTest Enable,Other-ChipTest Disable
//ChipTest Parameters
//ChipTest Parameters
extern uint16_t g_trng_mode;
extern uint16_t g_sm1_protect;
extern uint16_t g_sm4_protect;
extern uint16_t g_rsa_protect;

//Bootloader Enable Flag
extern uint16_t g_BL_Enable;

//Bootloader Password Success Flag
extern uint16_t g_PWD_Result;	

//Bootloader Pwd fail Counter
extern uint16_t g_BL_FailCounter;

//Bootloader Pwd Max fail time
extern uint16_t g_BL_MaxFail;

extern const uint8_t USB_hid_Device_Descriptor[18];

//Device Qualifier Descriptor
extern const uint8_t USB_hid_Device_Qualifier_Descriptor[10];

//configuration descriptor
extern const uint8_t USB_hid_Configuration_Descriptor[];

//standard string Descriptor,serial number
extern const uint8_t USB_hid_String_Descriptor[];
extern const uint8_t USB_hid_report_Descriptor2[];

extern unsigned char usb_tx_int_flag ;

/*USB function*/
extern void USBDev_Polling(void);
extern void USB_Init(void);
extern void read_ep_buf(uint8_t *out_buf,uint32_t ep,uint16_t uiLens);
extern void write_ep_buf(uint32_t ep,uint8_t *out_buf,uint16_t uiLens);
extern void USBDev_EP0_SendStall(void);
extern void write_ep0_buf(uint8_t *in_buf,uint16_t uiLen);
extern void USBDev_TestMode(void);
extern void USBDev_Inquiry(void);
extern void USBDev_ModeSense(void);
extern void USBDev_RequestSense(void);
extern void USBDev_ReadCapacity(void);
extern void USBDev_Suspend(void);
extern void USBDev_BusReset(void);
extern void USBDev_Ep0Handler(void);
extern void ClearRx(void);
extern void SetTx(void);
extern void SetFIFO_Addr(uint32_t bufferAddr,uint8_t mode);
extern void USBDev_Packet_BulkIn(uint8_t *pDataBuf,uint16_t uiLength,uint8_t flag);
extern void USB_int_Service(void);
extern void USB_CDC_Init(void);
extern void usb_cdc_send(uint8_t usb_ep, uint8_t *buf, uint16_t len);
extern uint16_t usb_cdc_receive(uint8_t usb_ep, uint8_t *buf);
extern void USB_HID_Init(void);
extern uint16_t USB_HID_EP1_RX_ISR(uint8_t* pData);
extern void USB_HID_EP1_TX_ISR(uint8_t* pData,uint16_t len);
extern void USB_HID_BusReset(void);
extern void USB_HID_EP0_ISR(void);
extern uint16_t usb_receive(uint8_t *buf);
extern void usb_send(uint8_t *buf,uint16_t len);
extern void USB_HID_Ep0Handler(void);
extern void USBDev_EP1_TX_ISR(void);
extern void USBDev_EP0_ISR(void);
extern void USBDev_EP1_RX_ISR(void);
extern void UsbSetTXsize(uint16_t size,uint8_t mode);
extern void USBDev_Data_BulkOut(void);
extern void USBDev_Data_BulkIn(void);
extern void USB_MS_ISR(void);
extern void USB_HID_ISR(void);
extern void USB_CDC_ISR(void);
extern void USBC_IRQHandler(void);
extern void USB_HID_Polling(void);
extern void HID_SetFIFO_Addr(uint32_t bufferAddr,uint8_t mode);

#endif /* __USB_DRV_H__ */
