/*******************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.
  * @file     main.c
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-01
  * @brief    This  file include main function.
 ********************************************************************************/

#include "sys.h"
#include "common.h"
#include "uart.h"
#include "debug.h"
#include "pit.h"
#include "data.h"
#include "function_drv.h"
#include "wdt_drv.h"
#include "wav_dac.h"
#include "bsp.h"
#include "tp_rtp.h"
#include "tp_ctp.h"
#include "sd_update.h"
#include "uart_update.h"
#include "levetop.h"
#include "tftlcd.h"
#include "dma.h"
#include "ioctrl_drv.h"
void PeripheralInit(void)
{
	/* system configurationObtain clock and other relevant parameters */
	Sys_Init();
	Enable_Interrupts;
	// SWD_Enable_Cmd(FALSE);    //SWD funtion disable
	WDT_control(0xf800, 0); // WDT funtion:0:close 1:open
	IOCTRL->SCI_CONTROL_REG |= SCICR_RX1_GINT0_SWAP_MASK;
	/* Uart */
#if UART_DEBUG
	UART_Debug_Init(SCI1, g_ips_clk, 115200); // TX1:uart printf  of  baudrate 115200
#endif
	UartTFT_SCI_Init(COMuart, 115200); // 268x default SCI3 as TFT communication

	/* exSPIFlash initialization */
	exFlash_SPI_Init(); 
	if (W25N01GV_ReadID() == 0xefaa21)
	{
		Flash_Type = 2;
		W25N01GV_Init();
	}
	else
	{
		Flash_ID = W25QXX_ReadID();
		//printf("Flash_ID==0x%x  \r\n", Flash_ID); //debug
		Flash_Type = 0;
		W25QXX_Init();
	}

	/* uart_update check */
#if (U_MCUFLASH_FLAG == 0)
	U_McuFlash_CheckBsp();
#endif

	/* get ui info */
	LT_ReadParam();

	/* Uart re-initialization */
#if !IIC_BUS
	UartTFT_SCI_Init(COMuart, gBaudrate);
#endif

	/* mcu info printf */
	#if 1 
	printf("/**************************************************/\n");
	printf(" date = %s,time = %s    \r\n", (char *)__DATE__, (char *)__TIME__);
	printf(" LT268B QFN48 Demo V1.0  II  \r\n");
	printf(" system clk = %dMHz         \r\n", g_sys_clk / 1000000);
	printf(" ips clk    = %dMHz         \r\n", g_ips_clk / 1000000);
	printf("/**************************************************/\r\n");
	#endif

	/* TFT init */
	LCD_TFT_Init();
//	LCD_XSIZE_TFT=240;  LCD_YSIZE_TFT=240; LCD_BL_Init(249); disp_V_Gray();  while(1);  		//RGB display(TFT test)

	/* pit init */
	PIT1_Init();

	/* RTC init */
	LT_RtcInit();

	/* touch init */
#if (LT_TOUCH_FLAG == 1)
	LT_TpInit();
#elif (LT_TOUCH_FLAG == 2)
	CTP_Init();
#endif

	/* IIC_BUS */
#if IIC_BUS
	// At present, the CTP and IIC slave use the same set of IO. If the IIC slave function is used, the touch function needs to be shielded first
	I2C_Slave_TEST(0x36); // Initialize IIC slave device address
	CTP_Init();
#endif
#if (UARTBUS_OPTION == 2)
	Get_ModBus_CMD_info();
#endif

	/* display UI start page */
	if (Start_page > addr_index[7])
		Display_page(0);
	else
		Display_page(Start_page);

	/* backlight */
	LCD_BL_Init(gDutyBuf[Backlight_On]);
	var[VAR_BL * 2 + 1] = Backlight_On;
//	LCD_BL_Init(gDutyBuf[var[VAR_BL * 2 + 1]]);
	/* touch buzzer */
#if LT_TOUCH_BUZZER
	if (buzzer == 1)
		touch_buzzer_init();
#endif

	/* encoder */
#if LT_ENCODER
	Encoder_Init();
#endif

	/* SDcard */
#if LT_SD_UPDATE
	SD_IN_Enable();
	SD_ud_flag = SD_detection();
#endif
}

int main(void)
{
	/* uart IAP */
#if (U_MCUFLASH_FLAG == 0)
	NVIC_SetVectorTable(0x08008000, 0);
#elif (U_MCUFLASH_FLAG == 1)
	NVIC_SetVectorTable(0x08043000, 0);
#endif

	PeripheralInit();
	while (1)
	{
#if !IIC_BUS
#if (UARTBUS_OPTION == 0)
		Main_From_UartRxBuff();
		if (gUsartRx.Flag)
			LT_ReceiveCmd(gUsartRx.Buf);
#elif (UARTBUS_OPTION == 1)
		LT_ModBus_REG_Cmd();
#elif (UARTBUS_OPTION == 2)
		if (Sum_ModbusTX)
			Uart_cmd_Send();
		LT_ModBus_REG_Cmd();
#endif
#endif

#if IIC_BUS
		LT_IIC_REG_Cmd(); // handle instruction function of writing register
#endif

		if (page_refreshing > 30)
		{
			page_refreshing = 0;
			Get_date_variable_Id_Regular_update(pic_id);
		}

		if (gWavFlag == 1 || gWavType == 1)
			LT_PlayWav_DAC(); // WAV

#if (LT_TOUCH_FLAG == 1 || LT_TOUCH_FLAG == 2)
		gTpInfo.scan(); // Touch Point
#endif
		if (Auto_Backlight)
			BackLight_control(); // backlight control

		
		show_gif();		 // gif
		show_Clock();	 // analog clock
		show_RTC();		 // digital clock
		scroll_text();	 // text scroll
		Display_Curve(); // Curve
		show_Count();	 // time counter
//printf("gEncoderFlag=%d \r\n",gEncoderFlag);
#if LT_ENCODER
		if (gEncoderFlag)
			encoder(); // Encoder
#endif
		// if(Ges_sc_flag) slide_screen();       //unsupported
		if (Gesture_flag)
			Gesture_touch(); // sliding gesture

		Basic_touch();			// basic touch control
		Adj_touch();			// variable adjustment
		Progress_bar_sliding(); // sliding progress bar
		data_input();			// data input
		slideMune();			// slide menu
		RingSld_touch();		// ring progress bar with touch
		Ascii_input();			// ASCII keyboard
		GBK_input();			// GBK keyboard
		VarKey_touch();			// multivariable adjustment
		pop_up_touch();			// pop-up window

#if LT_SD_UPDATE //---SDcard---
		if (SD_ud_flag)
		{
			if (!SD_detection())
				Sd_To_Flash(); // SD_pin detects a low level when an SD card is inserted.
		}
		else
		{
			if (SD_detection())
				SD_ud_flag = 1;
		}
#endif
	}
}
