/*******************************************************************************
  * Copyright(c) 2023 Levetop Semiconductor Co.,Led. All rights reserved.   
  * @file     module_select.h
  * @author   UartTFT Application Team
  * @version  V0.0.1
  * @date     2023-01-07
  * @brief    This file contains the macro definitions that can be modified
  *            on the UartTFT Application.
********************************************************************************/ 

#ifndef _MODULE_SELECT_H
#define _MODULE_SELECT_H

#define	RP4_FLAG	0		
#define	TFT_BITCOLOR	0		// screencolor depth, 0:16bits  1:24bits (Unsupported)
#define R_ANGLE			0		// screen, screen rotate(unsupported)

#define	LT_TFT_PORT		0		// screen, TFT port, 0:SPI  1:8bit 8080  2:16bit 8080(LT268B֧)
#define SCREEN_DIR		1       // screen, direction: 0:vertical screen  1:horizontal screen	


#define UART_DEBUG			0		// uart printf port: 0:SCI3  1:SCI1(custom)

#define LT_TOUCH_FLAG		2 		// device, touch panel type: 0:Null  1:RTP  2:CTP
#define LT_TOUCH_BUZZER		0 		// device, touch with buzzer: 0:disable  1:enable
#define LT_ENCODER			1		// device, encoder function: 0:disable  1:enable(need replace lib)
#define LT_SD_UPDATE		1 		// device, SDcard update: 0:disable  1:enable
	
#define IIC_BUS				0		// communication, 1:IIC protocol
#define UARTBUS_OPTION		0		// communication, 0:Levetop protocol as slave  1:modbus as slave  2:modbus as host


//---------------------------------variable area-------------------------------------------------
#define VAR_ADDR		(0x2000)			    // var address
#define VAR_ADDR_S	    (VAR_ADDR)	            // var start address
#define VAR_ADDR_E	    (VAR_ADDR +0xFF)	    // var end address
#define VAR_SIZE	    ((VAR_ADDR +0xFF)*2)	// var and reg total size

#define VAR_PAGE		(VAR_ADDR +0x00)	// page
#define VAR_BL			(VAR_ADDR +0x01)	// backlight
#define VAR_TIME		(VAR_ADDR +0x02)	// time
#define VAR_TIMESET		(VAR_ADDR +0x08)	// time setting
#define VAR_RSIZE_9		(VAR_ADDR +0x09)	// null
#define VAR_WAV			(VAR_ADDR +0x0a)	// wav
#define VAR_VOLUME		(VAR_ADDR +0x0b)	// sound volume
#define VAR_RTP			(VAR_ADDR +0x0c)	// resistance touch screen
#define VAR_KEY			(VAR_ADDR +0x0d)	// key code trigger
#define VAR_BLAUTO		(VAR_ADDR +0x0e)	// backlight: auto flag
#define VAR_BLSLEEP		(VAR_ADDR +0x0f)	// backlight: brightness during sleep
#define VAR_BLDURATION	(VAR_ADDR +0x10)	// backlight: duration during operation

#define VAR_MODBUS    	((VAR_ADDR +0xFF))	// Modbus COMMREQ save after VAR_SIZE on var(uart.c MODBUS_ADDR)

//---------------------------------reg area-------------------------------------------------
#define VAR_REG_S		(0x7000)				// reg start address
#define VAR_REG_E		(VAR_REG_S+0xFF)		// reg end address

#define VAR_CURVE_WS	(0xC001)	// curve write
#define VAR_CURVE_WE	(0xCFFF)	// curve write
#define VAR_CURVE_CS	(0xE001)	// curve clear
#define VAR_CURVE_CE	(0xEFFF)	// curve clear

//---------------------------------widget-------------------------------------------------
#define PAGECFG_SIZE	3072
#define DATA_WLEN_SIZE	30
#define PDATA_WLEN_SIZE	30
#define STRLEN_SIZE		300

#define BASIC_SIZE			20			// basic touch control
#define ADJUST_SIZE			10			// variable adjustment
#define SLIDE_SIZE			4			// slide progress bar adjustment
#define DIO_SIZE		  	20			// data keyboard
#define KEYBOARD_SIZE		60			// touch count
#define AIO_SIZE		  	10			// ascii keyboard
#define GBKIO_SIZE			10			// gbk keyboard
#define BATCH_SIZE			20			// multivariable adjustment
#define POPUP_SIZE			8			// pop-up window
#define GIF_SIZE			15			// gif               // default 8 
#define RTC_SIZE		  	6			// digital clock
#define CLOCK_SIZE			2			// analog clock 
#define SCROLL_SIZE			4			// text scroll
#define CURVE_SIZE			8			// curve 
#define SLDMENU_SIZE		6			// slide menu
#define RINGSLD_SIZE		4		  	// ring progress bar touch
#define COUNT_SIZE			10			// RTC count
#define CURVE_CHANNEL      	8			// curve channel count


#endif	/* _MODULE_SELECT_H */
